import numpy as np
import matplotlib.pyplot as plt
import math
import os
import subprocess
import fileinput
import re
import sys
import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.cm as cm
import matplotlib.ticker as ticker
from matplotlib.ticker import MultipleLocator,FormatStrFormatter,MaxNLocator
import seaborn as sns
import matplotlib.colors as mcolors
# ----------------------------------------------------
# --- Define Plot style:
# plt.style.use("seaborn-white")
# plt.style.use("seaborn-pastel")
# plt.style.use("seaborn-colorblind")
plt.style.use("seaborn")
# mpl.rcParams['text.usetex'] = True
# mpl.rcParams["font.family"] = "serif"
# mpl.rcParams["font.size"] = "14"
# mpl.rcParams['xtick.bottom'] = True
# mpl.rcParams['xtick.major.size'] = 3
# mpl.rcParams['xtick.minor.size'] = 1.5
# mpl.rcParams['xtick.major.width'] = 0.75
# mpl.rcParams['ytick.left'] = True
# mpl.rcParams['ytick.major.size'] = 3
# mpl.rcParams['ytick.minor.size'] = 1.5
# mpl.rcParams['ytick.major.width'] = 0.75

# #Adjust grid:
# mpl.rcParams.update({"axes.grid" : True}) # Add grid
# mpl.rcParams['axes.labelpad'] = 3
# mpl.rcParams['grid.linewidth'] = 0.25
# mpl.rcParams['grid.alpha'] = 0.9 # 0.75
# mpl.rcParams['grid.linestyle'] = '-'
# mpl.rcParams['grid.color']   = 'gray'#'black'
# mpl.rcParams['text.latex.preamble'] = r'\usepackage{amsfonts}' # Makes Use of \mathbb possible.


mpl.rcParams['text.usetex'] = True
mpl.rcParams["font.family"] = "serif"
mpl.rcParams["font.size"] = "8"
mpl.rcParams['xtick.bottom'] = True
mpl.rcParams['xtick.major.size'] = 2
mpl.rcParams['xtick.minor.size'] = 1.5
mpl.rcParams['xtick.major.width'] = 0.75
mpl.rcParams['xtick.labelsize'] = 8
mpl.rcParams['xtick.major.pad'] = 1

mpl.rcParams['ytick.left'] = True
mpl.rcParams['ytick.major.size'] = 2
mpl.rcParams['ytick.minor.size'] = 1.5
mpl.rcParams['ytick.major.width'] = 0.75
mpl.rcParams['ytick.labelsize'] = 8
mpl.rcParams['ytick.major.pad'] = 1

mpl.rcParams['axes.titlesize'] = 8
mpl.rcParams['axes.titlepad'] = 1
mpl.rcParams['axes.labelsize'] = 8

#Adjust Legend:
mpl.rcParams['legend.frameon'] = True       # Use frame for legend
# mpl.rcParams['legend.framealpha'] = 0.5 
mpl.rcParams['legend.fontsize'] = 8         # fontsize of legend


#Adjust grid:
mpl.rcParams.update({"axes.grid" : True}) # Add grid
mpl.rcParams['axes.labelpad'] = 3
mpl.rcParams['grid.linewidth'] = 0.25
mpl.rcParams['grid.alpha'] = 0.9 # 0.75
mpl.rcParams['grid.linestyle'] = '-'
mpl.rcParams['grid.color']   = 'gray'#'black'
mpl.rcParams['text.latex.preamble'] = r'\usepackage{amsfonts}' # Makes Use of \mathbb possible.
# ----------------------------------------------------------------------------------------
# width = 5.79
# height = width / 1.618 # The golden ratio.
textwidth = 6.26894 #textwidth in inch
width = textwidth * 0.5
height = width / 1.618 # The golden ratio.



#TODO read values from .txt output files....

dataset_numbers = [0, 1, 2, 3, 4 ,5]

for dataset_number in dataset_numbers:

    # fig = plt.figure()      #main
    # fig, ax = plt.subplots(figsize=(width,height))
    fig, ax = plt.subplots()
    fig.subplots_adjust(left=.15, bottom=.16, right=.95, top=.92)

    # ax.tick_params(axis='x',which='major', direction='out',pad=3)

    ax.xaxis.set_major_locator(MultipleLocator(1.0))
    ax.xaxis.set_minor_locator(MultipleLocator(0.5))    
    ax.yaxis.set_major_locator(MultipleLocator(0.5))

    # sampled with N=500 and length=5 via 'PolaPlotLocalEnergy.py'

    dataset = [

        [   # Dataset Ratio r = 0.12
        [14.70179844, 13.6246    , 12.42994508, 11.69773413,11.14159987, 9.500670278, 9.005046347] ,    # input (moisture-content in %)
        # [1.30260521, 1.83366733, 2.40480962, 2.76553106, 3.0260521, 3.80761523, 4.03807615],          # curvature kappa from Simulation GridLevel=3
        # [1.30260521, 1.83366733, 2.41482966, 2.76553106, 3.03607214, 3.81763527,4.04809619],          # curvature kappa from Simulation GridLevel=4
        [1.28256513, 1.79358717, 2.36472946, 2.70541082, 2.9759519,  3.73747495, 3.96793587],           # curvature kappa from Simulation GridLevel=5
        [1.140351217, 1.691038688, 2.243918105, 2.595732726, 2.945361006,4.001528043, 4.312080261]      # curvature kappa from Experiment
        ],

        [   # Dataset Ratio r = 0.17
        [14.75453569,13.71227639,12.54975012,11.83455959,11.29089521,9.620608917,9.101671742],          # input (moisture-content in %)
        # [1.27254509, 1.78356713, 2.36472946, 2.70541082, 2.9759519, 3.78757515,  4.02805611]          # curvature kappa from Simulation GridLevel=3
        # [1.29258517, 1.81362725, 2.39478958, 2.74549098, 3.01603206, 3.83767535, 4.08817635],         # curvature kappa from Simulation GridLevel=4
        [1.28256513, 1.80360721, 2.38476954, 2.73547094, 3.00601202, 3.81763527,4.06813627],            # curvature kappa from Simulation GridLevel=5
        [1.02915975,1.573720805,2.407706364,2.790518802,3.173814476,4.187433094,4.511739072]            # curvature kappa from Experiment
        ],



        [   # Dataset Ratio r = 0.22
        [14.72680026, 13.64338887, 12.41305478, 11.66482931, 11.09781471, 9.435795985, 8.959564147],    # input (moisture-content in %)
        # [1.19238477, 1.71342685, 2.30460922, 2.66533066, 2.93587174, 3.71743487,3.94789579],          # curvature kappa from Simulation GridLevel=3
        # [1.20240481, 1.73346693, 2.3246493,  2.68537074, 2.95591182, 3.74749499,3.97795591],          # curvature kappa from Simulation GridLevel=4
        [1.19238477, 1.72344689, 2.31462926, 2.6753507,  2.94589178, 3.72745491,3.95791583],            # curvature kappa from Simulation GridLevel=5
        [1.058078122, 1.544624544, 2.317033799, 2.686043143, 2.967694189, 3.913528418, 4.262750825]     # curvature kappa from Experiment
        ],


        [   # Dataset Ratio r = 0.34
        [14.98380876,13.97154915,12.77309253,12.00959929,11.42001731,9.561447179,8.964704969],          # input (moisture-content in %)
        # [0.87174349, 1.27254509, 1.75350701, 2.06412826, 2.29458918, 3.03607214,3.26653307],          # curvature kappa from Simulation GridLevel=3
        # [0.87174349, 1.28256513, 1.76352705, 2.0741483,  2.31462926, 3.05611222,3.28657315],          # curvature kappa from Simulation GridLevel=4
        [0.87174349, 1.27254509, 1.75350701, 2.06412826, 2.29458918, 3.03607214,3.26653307],            # curvature kappa from Simulation GridLevel=5
        [0.789078472,1.1299263,1.738136936,2.159520896,2.370047499,3.088299431,3.18097558]              # curvature kappa from Experiment
        ],




        [   # Dataset Ratio r = 0.43
        [15.11316339,14.17997082,13.05739844,12.32309209,11.74608518,9.812372466,9.10519385 ],          # input (moisture-content in %)
        # [0.63126253, 0.93186373, 1.29258517, 1.53306613, 1.71342685, 2.33466934,2.55511022],          # curvature kappa from Simulation GridLevel=3
        # [0.61122244, 0.90180361, 1.25250501, 1.48296593, 1.66332665, 2.26452906, 2.48496994],         # curvature kappa from Simulation GridLevel=4
        [0.6012024,  0.89178357, 1.23246493, 1.46292585, 1.63326653, 2.2244489,2.44488978],             # curvature kappa from Simulation GridLevel=5
        [0.577989364,0.829007544,1.094211707,1.325332511,1.400455154,1.832325697,2.047483977]           # curvature kappa from Experiment
        ],
        


        [   # Dataset Ratio r = 0.49
        [15.30614414,14.49463867,13.46629742,12.78388234,12.23057715,10.21852839,9.341730605],           # input (moisture-content in %)
        # [0.59118236, 0.87174349, 1.24248497, 1.49298597, 1.70340681, 2.46492986,2.79559118],           # curvature kappa from Simulation GridLevel=3
        # [0.54108216, 0.80160321, 1.14228457, 1.37274549, 1.56312625, 2.26452906,2.5751503 ],           # curvature kappa from Simulation GridLevel=4
        [0.56112224, 0.84168337, 1.19238477, 1.43286573, 1.63326653, 2.36472946,2.68537074],             # curvature kappa from Simulation GridLevel=5
        [0.357615902,0.376287785,0.851008627,0.904475291,1.039744708,1.346405241,1.566568558]            # curvature kappa from Experiment
        ]
    ]



    # input = [14.72680026, 13.64338887, 12.41305478, 11.66482931, 11.09781471, 9.435795985, 8.959564147]  #moisture-content (in %)
    # kappa_sim = [1.20401338 , 1.72575251, 2.28762542 , 2.64882943 , 2.92976589 , 3.73244147 , 3.93311037 ]    #kappa from Simulation
    # kappa_exp = [1.058078122,1.544624544, 2.317033799, 2.686043143, 2.967694189, 3.913528418, 4.262750825]    #kappa from Experiment

    # compute difference:
    # relative_error = (np.array(kappa_sim) - np.array(kappa_exp)) / np.array(kappa_exp)
    # print('relative_error:', relative_error)

    relative_error = (np.array(dataset[dataset_number][1]) - np.array(dataset[dataset_number][2])) / np.array(dataset[dataset_number][2])
    print('relative_error:', relative_error)


    # --------------- Plot Lines + Scatter -----------------------
    line_1 = ax.plot(np.array(dataset[dataset_number][0]), np.array(dataset[dataset_number][1]),                    # data
                #  color='forestgreen',              # linecolor
                marker='D',                         # each marker will be rendered as a circle
                markersize=3.5,                       # marker size
                #   markerfacecolor='darkorange',      # marker facecolor
                markeredgecolor='black',            # marker edgecolor
                markeredgewidth=0.5,                  # marker edge width
                # linestyle='dashdot',              # line style will be dash line
                linewidth=1.0,                      # line width
                zorder=3,
                label = r"$\kappa_{sim}$")

    line_2 = ax.plot(np.array(dataset[dataset_number][0]), np.array(dataset[dataset_number][2]),                     # data
                # color='orangered',                # linecolor
                marker='o',                         # each marker will be rendered as a circle
                markersize=3.5,                       # marker size
                #  markerfacecolor='cornflowerblue',   # marker facecolor
                markeredgecolor='black',            # marker edgecolor
                markeredgewidth=0.5,                  # marker edge width
                # linestyle='--',                   # line style will be dash line
                linewidth=1.0,                      # line width
                zorder=3,
                alpha=0.8,                           # Change opacity
                label = r"$\kappa_{exp}$")

    # --- Plot order line
    # x = np.linspace(0.01,1/2,100)
    # y = CC_L2[0]*x**2
    # OrderLine = ax.plot(x,y,linestyle='--', label=r"$\mathcal{O}(h)$")



    # Fix_value = 7.674124
    # l3 = plt.axhline(y = Fix_value, color = 'black', linewidth=0.75, linestyle = 'dashed')
    # --------------- Set Axes  -----------------------
    # ax.set_title(r"ratio $r = 0.22$")   # Plot - Title

    # Plot - Title
    match dataset_number:
        case 0:
            ax.set_title(r"Thickness ratio $r = 0.12$") 
        case 1: 
            ax.set_title(r"Thickness ratio $r = 0.17$") 
        case 2:
            ax.set_title(r"Thickness ratio $r = 0.22$") 
        case 3: 
            ax.set_title(r"Thickness ratio $r = 0.34$") 
        case 4:
            ax.set_title(r"Thickness ratio $r = 0.43$") 
        case 5: 
            ax.set_title(r"Thickness ratio $r = 0.49$") 
    
    # plt.xscale('log') # Use Logarithmic-Scale
    # plt.yscale('log')
    ax.set_xlabel(r"Wood moisture content $\omega (\%)$")
    ax.set_ylabel(r"Curvature $\kappa(m^{-1})$")
    # plt.tight_layout()

    # # --- Set Line labels
    # line_labels = [r"$CC_{L_2}$",r"$CC_{H_1}$", r"$\mathcal{O}(h)$"]

    # --- Set Legend
    legend = ax.legend()
    # legend = fig.legend([line_1 , line_2, OrderLine],
    #                     labels = line_labels,
    #                     bbox_to_anchor=[0.97, 0.50],
    #                     # bbox_to_anchor=[0.97, 0.53],
    #                     # loc='center',
    #                     ncol=1,                  # Number of columns used for legend
    #                     # borderaxespad=0.15,    # Small spacing around legend box
    #                     frameon=True,
    #                     prop={'size': 10})


    frame = legend.get_frame()
    frame.set_edgecolor('black')
    frame.set_linewidth(0.5)


    # --- Adjust left/right spacing:
    # plt.subplots_adjust(right=0.81)
    # plt.subplots_adjust(left=0.11)

    # ---------- Output Figure as pdf:
    fig.set_size_inches(width, height)
    fig.savefig('WoodBilayer_expComparison_' + str(dataset_number) + '.pdf')
    # plt.show()


