import numpy as np
import matplotlib.pyplot as plt
import math
import os
import subprocess
import fileinput
import re
import sys
import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.cm as cm
import matplotlib.ticker as ticker
from matplotlib.ticker import MultipleLocator,FormatStrFormatter,MaxNLocator
import seaborn as sns
import matplotlib.colors as mcolors

plt.style.use("seaborn")

mpl.rcParams['text.usetex'] = True
mpl.rcParams["font.family"] = "serif"
mpl.rcParams["font.size"] = "8"
mpl.rcParams['xtick.bottom'] = True
mpl.rcParams['xtick.major.size'] = 2
mpl.rcParams['xtick.minor.size'] = 1.5
mpl.rcParams['xtick.major.width'] = 0.75
mpl.rcParams['xtick.labelsize'] = 8
mpl.rcParams['xtick.major.pad'] = 1

mpl.rcParams['ytick.left'] = True
mpl.rcParams['ytick.major.size'] = 2
mpl.rcParams['ytick.minor.size'] = 1.5
mpl.rcParams['ytick.major.width'] = 0.75
mpl.rcParams['ytick.labelsize'] = 8
mpl.rcParams['ytick.major.pad'] = 1

mpl.rcParams['axes.titlesize'] = 8
mpl.rcParams['axes.titlepad'] = 1
mpl.rcParams['axes.labelsize'] = 8

#Adjust Legend:
mpl.rcParams['legend.frameon'] = True       # Use frame for legend
# mpl.rcParams['legend.framealpha'] = 0.5 
mpl.rcParams['legend.fontsize'] = 8         # fontsize of legend


#Adjust grid:
mpl.rcParams.update({"axes.grid" : True}) # Add grid
mpl.rcParams['axes.labelpad'] = 3
mpl.rcParams['grid.linewidth'] = 0.25
mpl.rcParams['grid.alpha'] = 0.9 # 0.75
mpl.rcParams['grid.linestyle'] = '-'
mpl.rcParams['grid.color']   = 'gray'#'black'
mpl.rcParams['text.latex.preamble'] = r'\usepackage{amsfonts}' # Makes Use of \mathbb possible.
# ----------------------------------------------------------------------------------------
# width = 5.79
# height = width / 1.618 # The golden ratio.
textwidth = 6.26894 #textwidth in inch
width = textwidth * 0.5
height = width/1.618 # The golden ratio.

fig, ax = plt.subplots(figsize=(width,height))
fig.subplots_adjust(left=.15, bottom=.16, right=.95, top=.92)

# ax.tick_params(axis='x',which='major', direction='out',pad=3)

ax.xaxis.set_major_locator(MultipleLocator(1.0))
ax.xaxis.set_minor_locator(MultipleLocator(0.5))    
ax.yaxis.set_major_locator(MultipleLocator(0.5))

data = [   # Dataset Ratio r = 0.49
        [15.30614414,14.49463867,13.46629742,12.78388234,12.23057715,10.21852839,9.341730605],   # input (moisture-content in %)
        [0.60150376, 0.87218045, 1.23308271, 1.5037594 , 1.71428571, 2.46616541,2.79699248],   # curvature kappa from Simulation       
        [0.4008016032064128, 0.5911823647294588, 0.8416833667334669, 1.002004008016032, 1.1322645290581161, 1.6132264529058116, 1.813627254509018], # curvature kappa of local minimizerfrom Simulation       
        [0.357615902,0.376287785,0.851008627,0.904475291,1.039744708,1.346405241,1.566568558]   # curvature kappa from Experiment
        ]

# relative_error = (np.array(data[dataset_number][1]) - np.array(dataset[dataset_number][2])) / np.array(dataset[dataset_number][2])
#print('relative_error:', relative_error)

#--------------- Plot Lines + Scatter -----------------------
line_1 = ax.plot(np.array(data[0]), np.array(data[1]),                    # data
            #  color='forestgreen',              # linecolor
            marker='D',                         # each marker will be rendered as a circle
            markersize=3.5,                       # marker size
            #   markerfacecolor='darkorange',      # marker facecolor
            markeredgecolor='black',            # marker edgecolor
            markeredgewidth=0.5,                  # marker edge width
            # linestyle='dashdot',              # line style will be dash line
            linewidth=1,                      # line width
            zorder=3,
            label = r"$\kappa_{1,sim}$")

line_2 = ax.plot(np.array(data[0]), np.array(data[2]),                    # data
            color='red',                # linecolor
            marker='s',                         # each marker will be rendered as a circle
            markersize=3.5,                       # marker size
            #  markerfacecolor='cornflowerblue',   # marker facecolor
            markeredgecolor='black',            # marker edgecolor
            markeredgewidth=0.5,                  # marker edge width
            # linestyle='--',                   # line style will be dash line
            linewidth=1,                      # line width
            zorder=3,
            alpha=0.8,                           # Change opacity
            label = r"$\kappa_{2,sim}$")

line_3 = ax.plot(np.array(data[0]), np.array(data[3]),                    # data
            # color='orangered',                # linecolor
            marker='o',                         # each marker will be rendered as a circle
            markersize=3.5,                       # marker size
            #  markerfacecolor='cornflowerblue',   # marker facecolor
            markeredgecolor='black',            # marker edgecolor
            markeredgewidth=0.5,                  # marker edge width
            # linestyle='--',                   # line style will be dash line
            linewidth=1,                      # line width
            zorder=3,
            alpha=0.8,                           # Change opacity
            label = r"$\kappa_{exp}$")

    # --- Plot order line
    # x = np.linspace(0.01,1/2,100)
    # y = CC_L2[0]*x**2
    # OrderLine = ax.plot(x,y,linestyle='--', label=r"$\mathcal{O}(h)$")



    # Fix_value = 7.674124
    # l3 = plt.axhline(y = Fix_value, color = 'black', linewidth=0.75, linestyle = 'dashed')
    # --------------- Set Axes  -----------------------
    # ax.set_title(r"ratio $r = 0.22$")   # Plot - Title

# Plot - Titl
ax.set_title(r"ratio $r = 0.49$") 

# plt.xscale('log') # Use Logarithmic-Scale
# plt.yscale('log')
ax.set_xlabel(r"Wood moisture content $\omega (\%)$", labelpad=4)
ax.set_ylabel(r"Curvature $\kappa$($m^{-1}$)", labelpad=4)
plt.tight_layout()

# # --- Set Line labels
# line_labels = [r"$CC_{L_2}$",r"$CC_{H_1}$", r"$\mathcal{O}(h)$"]

# --- Set Legend
legend = ax.legend()
# legend = fig.legend([line_1 , line_2, OrderLine],
#                     labels = line_labels,
#                     bbox_to_anchor=[0.97, 0.50],
#                     # bbox_to_anchor=[0.97, 0.53],
#                     # loc='center',
#                     ncol=1,                  # Number of columns used for legend
#                     # borderaxespad=0.15,    # Small spacing around legend box
#                     frameon=True,
#                     prop={'size': 10})


frame = legend.get_frame()
frame.set_edgecolor('black')
frame.set_linewidth(0.5)


# --- Adjust left/right spacing:
# plt.subplots_adjust(right=0.81)
# plt.subplots_adjust(left=0.11)

# ---------- Output Figure as pdf:
fig.set_size_inches(width, height)
fig.savefig('WoodBilayer_expComparison_local_global.pdf')
plt.show()


