import math
import numpy as np
import ctypes
import os
import sys

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()


#############################################
#  Paths
#############################################
parameterSet.resultPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs_laminate'
parameterSet.baseName= 'laminate'


#############################################
#  Grid parameters
#############################################
nX=7
nY=7

parameterSet.structuredGrid = 'simplex'
# parameterSet.lower = '-5 -2'
# parameterSet.upper = '5 2'
parameterSet.lower = '0 0'
parameterSet.upper = '1 1'
parameterSet.elements = str(nX)+' '+  str(nY)

parameterSet.macroGridLevel = 1
#############################################
#  Options
#############################################
parameterSet.measure_analyticalError = False
parameterSet.measure_isometryError = False
parameterSet.vtkWrite_analyticalSurfaceNormal = False
parameterSet.vtkwrite_analyticalSolution = False


parameterSet.conforming_DiscreteJacobian = 0

#############################################
#  Solver parameters
#############################################
# Tolerance of the multigrid solver
parameterSet.tolerance = 1e-12
# Maximum number of multigrid iterations
parameterSet.maxProximalNewtonSteps = 200
# Initial regularization
parameterSet.initialRegularization = 200
# Measure convergence
parameterSet.instrumented = 0

############################
#   Problem specifications
############################
# Dimension of the domain (only used for numerical-test python files)
parameterSet.dim = 2

# Dimension of the target space
parameterSet.targetDim = 3
parameterSet.targetSpace = 'BendingIsometry'
#############################################
#  VTK/Output
#############################################
# Write discrete solution as .vtk-File
parameterSet.writeVTK = 1
parameterSet.vtkwrite_analyticalSolution = 0
parameterSet.vtkWrite_analyticalSurfaceNormal = 0

# The grid can be refined several times for a higher resolution in the VTK-file.
parameterSet.subsamplingRefinement = 2

# Write Dof-Vector to .txt-file
parameterSet.writeDOFvector = 0

#############################################
#  Dirichlet boundary indicator
#############################################
# def dirichlet_indicator(x) :
#     if( (x[0] <= 0.001) or (x[1]<=0.001)):
#         return True
#     else:
#         return False

# def dirichlet_indicator(x) :
#     if( (x[0] <= -4.99) ):
#         return True
#     else:
#         return False


# # No Boundary condition
def dirichlet_indicator(x) :
        return False


# #Test clamp on other side:
# def dirichlet_indicator(x) :
#     if( (x[0] >=3.999) or (x[1]<=0.001)):
#         return True
#     else:
#         return False


#boundary-values/derivative function
def boundaryValues(x):
    return [x[0], x[1], 0]


# def boundaryValuesDerivative(x):
#     return ((1,0,0),
#             (0,1,0),
#             (0,0,1))



#############################################
#  Initial iterate function
#############################################
def f(x):
    return [x[0], x[1], 0]


def df(x):
    return ((1,0),
            (0,1),
            (0,0))


fdf = (f, df)


#############################################
#  Microstructure
############################################
# parameterSet.prestrainFlag = True
# parameterSet.macroscopically_varying_microstructure = False


# effectivePrestrain= np.array([[0.0, 0.0],
#                               [0.0, 0.5]]);



# effectiveQuadraticForm = np.array([[1.0, 0.0, 0.0],
#                                    [0.0, 2.0, 0.0],
#                                    [0.0, 0.0, 1.0]]);






#############################################
#  MICROSTRUCTURE
############################################
parameterSet.printMicroOutput = False
parameterSet.VTKwriteMacroPhaseIndicator = False
parameterSet.MacroPhaseSubsamplingRefinement = 3

class GlobalMicrostructure:
    """ Class that represents the global microstructure.
    
        The global microstructure can be defined individually 
        for different (finitely many) macroscopic phases. 
        Each macroscopic phase corresponds to a 'LocalMicrostructure_'
        sub-class that defines the necessary data for the local 
        micro-problem. 

        Currently, there are three possibilities for the LocalMicrostructure:
            (1) Read the effective quantities (Qhom,Beff) from this parset.
            (Constant local microstructure)
            (2) Solve the micro-problem once to obtain the effective quantities. 
            (Constant local microstructure)
            (3) Solve for micro-problem for each macroscopic quadrature point.
            (Macroscopocally varying local microstructure)) 
    """
    def __init__(self,macroPoint=[0,0]):
        self.macroPhases = 1

        # define first local microstructure options.
        self.macroPhase1_constantMicrostructure = True
        self.macroPhase1_readEffectiveQuantities = True;

    def macroPhaseIndicator(self,y): #y :macroscopic point
            """ Indicatorfunction that determines the domains 
                i.e. macro-phases of different local microstructures.
            """
            return 1;
                
    # Represents the local microstructure in Phase 1
    class LocalMicrostructure_1:  
        def __init__(self,macroPoint=[0,0]):
            self.gamma = 1.0    #in the future this might change depending on macroPoint.
            self.phases = 2     #in the future this might change depending on macroPoint.

            self.effectivePrestrain= np.array([[0.0, 0.0],
                                               [0.0, 0.5]]);

            self.effectiveQuadraticForm = np.array([[1.0, 0.0, 0.0],
                                            [0.0, 2.0, 0.0],
                                            [0.0, 0.0, 1.0]]);









#TEST NPG1 - 1
# parameterSet.effectivePrestrain = '0.0 0.5 0.0'   # b1 b2 b12

#TEST NPG1 - 2
# parameterSet.effectivePrestrain = '2.0 1.5 0.0'   # b1 b2 b12

#TEST NPG1 - 3 (one parameter family case)
# parameterSet.effectivePrestrain = '0.491 0.347 0.0'   # b1 b2 b12


#Theta = 0.25 , gamma=1.0, theta_mu=theta_rho = 2
# parameterSet.effectivePrestrain = '0.3734012735 -0.3 0.0'   # b1 b2 b12   

#Theta = 0.5 , gamma=1.0, theta_mu=theta_rho = 2
# parameterSet.effectivePrestrain = '-0.75 -1.5 0.0'   # b1 b2 b12   

#Theta = 0.5 , gamma=0.005, theta_mu=theta_rho = 2
# parameterSet.effectivePrestrain = '-1.49863 -1.5 0.0'   # b1 b2 b12   

# #Theta = 0.5 , gamma=0.005, theta_mu=2  theta_rho = 0.5
# parameterSet.effectivePrestrain = '0.000687699 0 0.0'   # b1 b2 b12   


# #Theta = 0.5 , gamma=0.005, theta_mu=2  theta_rho = 0.0
# parameterSet.effectivePrestrain = '0.500459 0.5 0.0'   # b1 b2 b12   

#Theta = 0.5 , gamma=1e6, theta_mu=2  theta_rho = 0.25
# parameterSet.effectivePrestrain = '0.562500 0.25 0.0'   # b1 b2 b12   

# #Theta = 0.5 , gamma=1e6, theta_mu=2  theta_rho = 3.0
# parameterSet.effectivePrestrain = '-1.5 -2.5 0.0'   # b1 b2 b12   


#Theta = 0.5 , gamma=1e6, theta_mu=2  theta_rho = -0.5
# parameterSet.effectivePrestrain = '1.125 1.0 0.0'   # b1 b2 b12   

#############################################
#  Effective elastic moduli
############################################

# parameterSet.effectiveQuadraticForm = '1.0 2.0 3.0 4.0 0.0 0.0'

#Test1
# parameterSet.effectiveQuadraticForm = '1.0 4.0 2.0 1.0 0.0 0.0'

#Test2
# parameterSet.effectiveQuadraticForm = '1.0 0.33 0.5 1.0 0.0 0.0'


#TEST NPG1  - 1
# parameterSet.effectiveQuadraticForm = '1.0 2.0 1.0 0.0 0.0 0.0'

#TEST NPG1  - 2
# parameterSet.effectiveQuadraticForm = '1.0 2.0 1.0 0.0 0.0 0.0'

#TEST NPG1  - 3 (one parameter family case)
# parameterSet.effectiveQuadraticForm = '1.0 2.0 1.164 0.5 0.0 0.0'

# parameterSet.effectiveQuadraticForm = '1.0 1.0 1.0 0.0 0.0 0.0'   #q11 q22 q33 q12 q13 q23 (entries of symmetrix 3x3 matrix)
# parameterSet.effectiveQuadraticForm = '1.0 1.0 1.0 0.0 0.0 0.5'   #q11 q22 q33 q12 q13 q23 (entries of symmetrix 3x3 matrix)
# parameterSet.effectiveQuadraticForm = '0.190612723956301411 0.208333333333333093  0.192304720883782532 0.0 0.0 0.0'   #q11 q22 q33 q12 q13 q23 (entries of symmetrix 3x3 matrix)

#Theta = 0.25 , gamma=1.0, theta_mu=theta_rho = 2
# parameterSet.effectiveQuadraticForm = '0.1905108157 0.2083333333 0.1922697645 0.0 0.0 0.0'   #q11 q22 q33 q12 q13 q23 (entries of symmetrix 3x3 matrix) #Theta = 0.25

 #Theta = 0.5 , gamma=1.0, theta_mu=theta_rho = 2
# parameterSet.effectiveQuadraticForm = '0.222 0.25 0.26994 0.0 0.0 0.0'   #q11 q22 q33 q12 q13 q23 (entries of symmetrix 3x3 matrix)

#Theta = 0.5 , gamma=0.005, theta_mu=theta_rho = 2
# parameterSet.effectiveQuadraticForm = '0.2499429903 0.25 0.2499924856 0.0 0.0 0.0'


#Theta = 0.5 , gamma=0.005, theta_mu=2  theta_rho = 0.5
# parameterSet.effectiveQuadraticForm = '0.2499429903 0.25 0.2499924856 0.0 0.0 0.0'

#Theta = 0.5 , gamma=1e6, theta_mu=2  theta_rho = 0.25
# parameterSet.effectiveQuadraticForm = '0.2222222222 0.25 0.2222222222 0.0 0.0 0.0'

#Theta = 0.5 , gamma=1e6, theta_mu=2  theta_rho = 3.0
# parameterSet.effectiveQuadraticForm = '0.2222222222 0.25 0.2222222222 0.0 0.0 0.0'

#Theta = 0.5 , gamma=1e6, theta_mu=2  theta_rho = -0.5
# parameterSet.effectiveQuadraticForm = '0.2222222222 0.25 0.2222222222 0.0 0.0 0.0'


#############################################
#  Force
############################################
parameterSet.assemble_force_term = False
def force(x):
    # return [0, 0, 0.025]
    return [0, 0, 0]




#############################################
#  Analytical reference Solution
#############################################
# def u(x):
#     return [x[0], x[1], 0]


# def du(x):
#     return ((1,0),
#             (0,1),
#             (0,0))


# udu = (u, du)
