import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from matplotlib.ticker import LogLocator
import codecs
import re
import json
import numpy as np
import matplotlib.pyplot as plt
import math
import os
import subprocess
import fileinput
import re
import sys
import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.cm as cm
import matplotlib.ticker as ticker
from matplotlib.ticker import MultipleLocator,FormatStrFormatter,MaxNLocator
import seaborn as sns
import matplotlib.colors as mcolors
from scipy.optimize import minimize_scalar 


### Choose between different wood material orientations in the upper layer (we used LTR - directions in the beginning)
# orientation = "./orientation_LTR"
# orientation = "/orientation_LRT"

# ### Choose the grid Level used
# gridLevel = "/gridLevel4"
# gridLevel = "./gridLevel5"


dirname = os.path.dirname(__file__)
print('Python file path:', dirname)

# basePath = dirname + orientation + gridLevel 
basePath = dirname + "/wood-bilayer"

# -------------------------------------------------------------------------------
### Helper functions

def energy(kappa,alpha,Q,B)  :
    """
        Energy evaluation for given effective quantities Q and B 
        with input:
        * curvature: kappa 
        * angle: alpha
    """
    #--- TEST (This should have zero energy)
    # kappa = 1
    # alpha = 0
    # Q= np.array([[1,4,0],[0, 2, 2], [0,0,3]])
    # B = np.array([1.0,0.0,0.0])
    #--------------
    # Compute cylindrical minimizer coefficients from angle & curvature
    G = kappa*np.array([np.cos(alpha)**2, np.sin(alpha)**2, np.sqrt(2)*np.cos(alpha)*np.sin(alpha)])
    # Subtract the effective prestrain
    G = G - B;
    # Compute the energy
    energy =  (np.transpose(G)).dot((Q.dot(G)));
    return energy


# Old version (correct if B is transposed)
# def energy(kappa,alpha,Q,B)  :
#     #--- TEST (This should have zero energy)
#     # kappa = 1
#     # alpha = 0
#     # Q= np.array([[1,4,0],[0, 2, 2], [0,0,3]])
#     # B = np.array([1.0,0.0,0.0])
#     #--------------
#     # Transpose B:
#     B = np.transpose([B])

#     # G=kappa*np.array([[np.cos(alpha)**2],[np.sin(alpha)**2],[np.sqrt(2)*np.cos(alpha)*np.sin(alpha)]])
#     G=kappa*np.array([[np.cos(alpha)**2],[np.sin(alpha)**2],[np.sqrt(2)*np.cos(alpha)*np.sin(alpha)]])-B
#     # print('energy: ', np.matmul(np.transpose(G),np.matmul(Q,G))[0,0])
#     # exit(0)
#     return np.matmul(np.transpose(G),np.matmul(Q,G))[0,0]



def xytokappaalpha(x,y):
   
    if y>0:
        return [np.sqrt(x**2+y**2), np.abs(np.arctan2(y,x))]
    else:
        return [-np.sqrt(x**2+y**2), np.abs(np.arctan2(y,x))]

# Read effective quantites
def ReadEffectiveQuantities(QFilePath, BFilePath):
    # Read Output Matrices (effective quantities)
    # From Cell-Problem output Files : ../outputs/Qmatrix.txt , ../outputs/Bmatrix.txt
    # -- Read Matrix Qhom
    X = []
    # with codecs.open(path + '/outputs/QMatrix.txt', encoding='utf-8-sig') as f:
    with codecs.open(QFilePath, encoding='utf-8-sig') as f:
        for line in f:
            s = line.split()
            X.append([float(s[i]) for i in range(len(s))])
    Q = np.array([[X[0][2], X[1][2], X[2][2]],
                  [X[3][2], X[4][2], X[5][2]],
                  [X[6][2], X[7][2], X[8][2]] ])

    # -- Read Beff (as Vector)
    X = []
    # with codecs.open(path + '/outputs/BMatrix.txt', encoding='utf-8-sig') as f:
    with codecs.open(BFilePath, encoding='utf-8-sig') as f:
        for line in f:
            s = line.split()
            X.append([float(s[i]) for i in range(len(s))])
    B = np.array([X[0][2], X[1][2], X[2][2]])
    return Q, B



    #--- Select specific experiment [x, y] with date from results_x/y
def get_Q_B(basePath, index,perforation=False, perforatedLayer='upper'):
    # results_index[0]/index[1]/...
    #DataPath = './experiment/wood-bilayer_PLOS/results_'  + str(data[0]) + '/' +str(data[1])
    # DataPath = './results_'  + str(index[0]) + '/' +str(index[1])
    # DataPath = '.' + dirname + orientation + gridLevel + '/results_'  + str(index[0]) + '/' +str(index[1])
    # DataPath = dirname + orientation + gridLevel + '/results_'  + str(index[0]) + '/' +str(index[1])
    if perforation:
        DataPath = basePath + '/results_'  +  perforatedLayer + '_'   + str(index[0]) + '/' +str(index[1])
    else : 
        DataPath = basePath + '/results_'  + str(index[0]) + '/' +str(index[1])

    QFilePath = DataPath + '/QMatrix.txt'
    BFilePath = DataPath + '/BMatrix.txt'
    # Read Q and B
    Q, B = ReadEffectiveQuantities(QFilePath,BFilePath)
    Q=0.5*(np.transpose(Q)+Q) # symmetrize
    # B=np.transpose([B])      #Transponiert!
    return (Q,B)



#     #--- Select specific experiment [x, y] with date from results_x/y
# def get_Q_B(index):
#     # results_index[0]/index[1]/...
#     #DataPath = './experiment/wood-bilayer_PLOS/results_'  + str(data[0]) + '/' +str(data[1])
#     # DataPath = './results_'  + str(index[0]) + '/' +str(index[1])
#     # DataPath = '.' + dirname + orientation + gridLevel + '/results_'  + str(index[0]) + '/' +str(index[1])
#     DataPath = dirname + orientation + gridLevel + '/results_'  + str(index[0]) + '/' +str(index[1])
#     QFilePath = DataPath + '/QMatrix.txt'
#     BFilePath = DataPath + '/BMatrix.txt'
#     # Read Q and B
#     Q, B = ReadEffectiveQuantities(QFilePath,BFilePath)
#     Q=0.5*(np.transpose(Q)+Q) # symmetrize
#     B=np.transpose([B])
#     return (Q,B)





def get_local_minimizer_on_axes(Q,B):
    invoke_function=lambda kappa: energy(kappa,0,Q,B)
    result_0 = minimize_scalar(invoke_function, method="golden")
    invoke_function=lambda kappa: energy(kappa,np.pi/2,Q,B)
    result_90 = minimize_scalar(invoke_function, method="golden")
    return np.array([[result_0.x,0],[result_90.x,np.pi/2]])


# ----------------------------------------------------------------------------------------
# ------------------------------ EXPERIMENT 1 --------------------------------------------
# ----------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------
### PARAMETERS FROM SIMULATION 

# [r, h, omega_flat, omega_target, theta, Kappa_experimental]
# r = (thickness upper layer)/(thickness)
# h = thickness [meter]
# omega_flat = moisture content in the flat state before drying [%]
# omega_target = moisture content in the target state [%]
# theta = rotation angle (not implemented and used)

materialFunctionParameter=[
[  # Dataset Ratio r = 0.12
[0.12, 0.0047, 17.32986047, 14.70179844, 0, 1.140351217],
[0.12, 0.0047, 17.32986047, 13.6246,     0, 1.691038688],
[0.12, 0.0047, 17.32986047, 12.42994508, 0, 2.243918105],
[0.12, 0.0047, 17.32986047, 11.69773413, 0, 2.595732726],
[0.12, 0.0047, 17.32986047, 11.14159987, 0, 2.945361006],
[0.12, 0.0047, 17.32986047, 9.500670278, 0, 4.001528043],
[0.12, 0.0047, 17.32986047, 9.005046347, 0, 4.312080261]
],
[  # Dataset Ratio r = 0.17
[0.17, 0.0049, 17.28772791 , 14.75453569, 0, 1.02915975],
[0.17, 0.0049, 17.28772791 , 13.71227639,  0, 1.573720805],
[0.17, 0.0049, 17.28772791 , 12.54975012, 0, 2.407706364],
[0.17, 0.0049, 17.28772791 , 11.83455959, 0, 2.790518802],
[0.17, 0.0049, 17.28772791 , 11.29089521, 0, 3.173814476],
[0.17, 0.0049, 17.28772791 , 9.620608917, 0, 4.187433094],
[0.17, 0.0049, 17.28772791 , 9.101671742, 0, 4.511739072]
],
[  # Dataset Ratio r = 0.22
[0.22, 0.0053,  17.17547062, 14.72680026, 0, 1.058078122],
[0.22, 0.0053,  17.17547062, 13.64338887, 0, 1.544624544],
[0.22, 0.0053,  17.17547062, 12.41305478, 0, 2.317033799],
[0.22, 0.0053,  17.17547062, 11.66482931, 0, 2.686043143],
[0.22, 0.0053,  17.17547062, 11.09781471, 0, 2.967694189],
[0.22, 0.0053,  17.17547062, 9.435795985, 0, 3.913528418],
[0.22, 0.0053,  17.17547062, 8.959564147, 0, 4.262750825]
],
[  # Dataset Ratio r = 0.34
[0.34, 0.0063, 17.14061081 , 14.98380876, 0, 0.789078472],
[0.34, 0.0063, 17.14061081 , 13.97154915,  0, 1.1299263],
[0.34, 0.0063, 17.14061081 , 12.77309253, 0, 1.738136936],
[0.34, 0.0063, 17.14061081 , 12.00959929, 0, 2.159520896],
[0.34, 0.0063, 17.14061081 , 11.42001731, 0, 2.370047499],
[0.34, 0.0063, 17.14061081 , 9.561447179, 0, 3.088299431],
[0.34, 0.0063, 17.14061081 , 8.964704969, 0, 3.18097558]
],
[  # Dataset Ratio r = 0.43
[0.43, 0.0073, 17.07559686 , 15.11316339, 0, 0.577989364],
[0.43, 0.0073, 17.07559686 , 14.17997082, 0, 0.829007544],
[0.43, 0.0073, 17.07559686 , 13.05739844, 0, 1.094211707],
[0.43, 0.0073, 17.07559686 , 12.32309209, 0, 1.325332511],
[0.43, 0.0073, 17.07559686 , 11.74608518, 0, 1.400455154],
[0.43, 0.0073, 17.07559686 , 9.812372466, 0, 1.832325697],
[0.43, 0.0073, 17.07559686 , 9.10519385 , 0, 2.047483977]
],
[  # Dataset Ratio r = 0.49
[0.49, 0.008,  17.01520754, 15.30614414, 0, 0.357615902],
[0.49, 0.008,  17.01520754, 14.49463867, 0, 0.376287785],
[0.49, 0.008,  17.01520754, 13.46629742, 0, 0.851008627],
[0.49, 0.008,  17.01520754, 12.78388234, 0, 0.904475291],
[0.49, 0.008,  17.01520754, 12.23057715, 0, 1.039744708],
[0.49, 0.008,  17.01520754, 10.21852839, 0, 1.346405241],
[0.49, 0.008,  17.01520754, 9.341730605, 0, 1.566568558]
]
]


n=len(materialFunctionParameter)
m=len(materialFunctionParameter[0])
kappa_0=np.zeros([n,m])
energy_0=np.zeros([n,m])
kappa_90=np.zeros([n,m])
energy_90=np.zeros([n,m])
energy_global=np.zeros([n,m])
kappa_global_min=np.zeros([n,m])
energy_origin=np.zeros([n,m])
for i in range(0,n):
    for j in range(0,m):
        Q, B=get_Q_B(basePath, [i,j])
        minimizers=get_local_minimizer_on_axes(Q,B)
        kappa_0[i,j]=minimizers[0,0]
        energy_0[i,j]=energy(kappa_0[i,j],0,Q,B)
        kappa_90[i,j]=minimizers[1,0]
        energy_90[i,j]=energy(kappa_90[i,j],np.pi/2,Q,B)
        if energy_0[i,j]<energy_90[i,j]:
            kappa_global_min[i,j]=kappa_0[i,j]
            energy_global[i,j]=energy_0[i,j]
        else:
            kappa_global_min[i,j]=kappa_90[i,j]
            energy_global[i,j]=energy_90[i,j]
        energy_origin[i,j]=energy(0,0,Q,B)


# # Error plot
# for i in range(0,n):
#     plt.plot(np.array(materialFunctionParameter[i])[:,3], np.abs(kappa_0[i,:]-np.array(materialFunctionParameter[i])[:,5])/np.array(materialFunctionParameter[i])[:,5], label=str(i))

# plt.legend()


# -------------------------------------------------------------------------------
### Plot diagrams: comparison with experimental data
        
print('Create plots for comparison with experimental data ...')
        
# Switch for local minimizers
ShowLocal=True

plt.style.use("seaborn")
mpl.rcParams['text.usetex'] = True
mpl.rcParams["font.family"] = "serif"
mpl.rcParams["font.size"] = "8"
mpl.rcParams['xtick.bottom'] = True
mpl.rcParams['xtick.major.size'] = 2
mpl.rcParams['xtick.minor.size'] = 1.5
mpl.rcParams['xtick.major.width'] = 0.75
mpl.rcParams['xtick.labelsize'] = 8
mpl.rcParams['xtick.major.pad'] = 1
mpl.rcParams['ytick.left'] = True
mpl.rcParams['ytick.major.size'] = 2
mpl.rcParams['ytick.minor.size'] = 1.5
mpl.rcParams['ytick.major.width'] = 0.75
mpl.rcParams['ytick.labelsize'] = 8
mpl.rcParams['ytick.major.pad'] = 1
mpl.rcParams['axes.titlesize'] = 8
mpl.rcParams['axes.titlepad'] = 1
mpl.rcParams['axes.labelsize'] = 8
#Adjust Legend:
mpl.rcParams['legend.frameon'] = True       # Use frame for legend
# mpl.rcParams['legend.framealpha'] = 0.5 
mpl.rcParams['legend.fontsize'] = 8         # fontsize of legend
#Adjust grid:
mpl.rcParams.update({"axes.grid" : True}) # Add grid
mpl.rcParams['axes.labelpad'] = 3
mpl.rcParams['grid.linewidth'] = 0.25
mpl.rcParams['grid.alpha'] = 0.9 # 0.75
mpl.rcParams['grid.linestyle'] = '-'
mpl.rcParams['grid.color']   = 'gray'#'black'
mpl.rcParams['text.latex.preamble'] = r'\usepackage{amsfonts}' # Makes Use of \mathbb possible.
# ----------------------------------------------------------------------------------------
# width = 5.79
# height = width / 1.618 # The golden ratio.
textwidth = 6.26894 #textwidth in inch
width = textwidth * 0.5
height = width/1.618 # The golden ratio.

fig, ax = plt.subplots(figsize=(width,height))
fig.subplots_adjust(left=.15, bottom=.16, right=.95, top=.92)

# ax.tick_params(axis='x',which='major', direction='out',pad=3)

for i in range(0,n):
    #ax.set_ylim(0,4.5)
    ax.set_xlim(8.5, 15.5)
    ax.xaxis.set_major_locator(MultipleLocator(1.0))
    ax.xaxis.set_minor_locator(MultipleLocator(0.5))    
    ax.yaxis.set_major_locator(MultipleLocator(0.5))
    data=np.zeros([5,m])
    data[0]=np.array(materialFunctionParameter[i])[:,3] # omega_0
    data[1]=kappa_0[i,:]
    data[2]=kappa_90[i,:]
    data[3]=kappa_global_min[i,:]
    data[4]=np.array(materialFunctionParameter[i])[:,5] # experimental curvature
    # relative_error = (np.array(data[dataset_number][1]) - np.array(dataset[dataset_number][2])) / np.array(dataset[dataset_number][2])
    #print('relative_error:', relative_error)

    #--------------- Plot Lines + Scatter -----------------------
    # Curvature (absolute value) with angle 0
    if ShowLocal:          
        line_1 = ax.plot(np.array(data[0]), np.array(data[1]),                    # data
                    color='forestgreen',              # linecolor
                    marker='o',                         # each marker will be rendered as a circle
                    markersize=1.5,                       # marker size
                    markerfacecolor='forestgreen',      # marker facecolor
                    markeredgecolor='black',            # marker edgecolor
                    markeredgewidth=0.5,                  # marker edge width
                    # linestyle='dashdot',              # line style will be dash line
                    linewidth=1,                      # line width
                    zorder=3,
                    label = r"$0^\circ$")
        # Curvature with angle 90
        line_3 = ax.plot(np.array(data[0]), np.array(data[2]),                    # data
                    color='cornflowerblue',                # linecolor
                    marker='o',                         # each marker will be rendered as a circle
                    markersize=1.5,                       # marker size
                    markerfacecolor='cornflowerblue',   # marker facecolor
                    markeredgecolor='black',            # marker edgecolor
                    markeredgewidth=0.5,                  # marker edge width
                    # linestyle='--',                   # line style will be dash line
                    linewidth=1,                      # line width
                    zorder=3,
                    alpha=0.8,                           # Change opacity
                    label = r"$90^$")
        
        # Global minimizer
        line_4 = ax.plot(np.array(data[0]), np.array(data[3]),                    # data
                    # color='orangered',                # linecolor
                    marker='s',                         # each marker will be rendered as a circle
                    markersize=5,                       # marker size
                    #  markerfacecolor='cornflowerblue',   # marker facecolor
                    markeredgecolor='black',            # marker edgecolor
                    markeredgewidth=0.5,                  # marker edge width
                    # linestyle='--',                   # line style will be dash line
                    linewidth=0,                      # line width
                    zorder=3,
                    alpha=0.2,                           # Change opacity
                    label = r"global min.")
    else:               
        # Global minimizer
        line_4 = ax.plot(np.array(data[0]), np.array(data[3]),                    # data
                    color='forestgreen',              # linecolor
                    marker='o',                         # each marker will be rendered as a circle
                    markersize=1.5,                       # marker size
                    markerfacecolor='forestgreen',      # marker facecolor
                    markeredgecolor='black',            # marker edgecolor
                    markeredgewidth=0.5,                  # marker edge width
                    # linestyle='dashdot',              # line style will be dash line
                    linewidth=1,                      # line width
                    zorder=3,
                    label = r"$\kappa$")
    
    # Experimental value of curvature
    line_2 = ax.plot(np.array(data[0]), np.array(data[4]),                    # data
                color='red',                # linecolor
                marker='o',                         # each marker will be rendered as a circle
                markersize=1.5,                       # marker size
                markerfacecolor='red',   # marker facecolor
                markeredgecolor='black',            # marker edgecolor
                markeredgewidth=0.5,                  # marker edge width
                # linestyle='--',                   # line style will be dash line
                linewidth=1,                      # line width
                zorder=3,
                alpha=0.8,                           # Change opacity
                label = r"\kappa_{\rm exp}$")
    

    # Plot - Titel
    ax.set_title(r"ratio $r = $" +str(materialFunctionParameter[i][0][0]), pad=5.0) 
    ax.set_xlabel(r"Wood moisture content $\omega (\%)$", labelpad=4)
    ax.set_ylabel(r"Curvature $\kappa$($m^{-1}$)", labelpad=4)
    plt.tight_layout()



    # ---------- Output Figure as pdf:
    fig.set_size_inches(width, height)
    if ShowLocal:
        fig.savefig(dirname + '/WoodBilayer_expComparison_local_'+str(i)+'.pdf')
    else:
        fig.savefig(dirname + '/WoodBilayer_expComparison_global_'+str(i)+'.pdf')        
    plt.cla()
    




# -------------------------------------------------------------------------------
### Plot diagrams: Energy landscape (Q - Q_min)
    
print('Create plots of energy landscape ...')
    
# Number of experiments / folders
show_plot = False

fig, (ax1, ax2) = plt.subplots( nrows=2,figsize=(3,7),subplot_kw=dict(projection='polar'), gridspec_kw={'hspace': 0.4})
Emax=31000
levs=np.geomspace(1,Emax,200)



#--- Select specific experiment [x, y] with date from results_x/y
index=[0,6]

# Read Q and B
Q, B = get_Q_B(basePath,index)


print('Qhom: \n', Q)
print('Beff: \n', B)


# Compute local and global minimizer
kappa=0
kappa_pos=0
kappa_neg=0

N=500
length=5
r, theta = np.meshgrid(np.linspace(0,length,N),np.radians(np.linspace(0, 360, N)))
E=np.zeros(np.shape(r))
for i in range(0,N): 
    for j in range(0,N):     
        if theta[i,j]<np.pi:
            E[i,j]=energy(r[i,j],theta[i,j],Q,B)
        else:
            E[i,j]=energy(-r[i,j],theta[i,j],Q,B)
#        
# Compute Minimizer
[imin,jmin]=np.unravel_index(E.argmin(),(N,N))
kappamin=r[imin,jmin]
alphamin=theta[imin,jmin]
# Positiv curvature region
N_mid=int(N/2)
[imin,jmin]=np.unravel_index(E[:N_mid,:].argmin(),(N_mid,N))
kappamin_pos=r[imin,jmin]
alphamin_pos=theta[imin,jmin]
Emin_pos=E[imin,jmin]
# Negative curvature region
[imin,jmin]=np.unravel_index(E[N_mid:,:].argmin(),(N_mid,N))
kappamin_neg=r[imin+N_mid,jmin]
alphamin_neg=theta[imin+N_mid,jmin]
Emin_neg=E[imin+N_mid,jmin]
#
E=E-E.min()


print('Plot energy landscape for dataset ' + str(index))
print('---- r = 0.12------')
print('kappamin_pos: ', kappamin_pos)
print('alphamin_pos: ', alphamin_pos)
print('kappamin_neg: ', kappamin_neg)
print('alphamin_neg: ', alphamin_neg)
print('kappamin: ', kappamin)
print('alphamin: ', alphamin)


# Plot 1. energy landscape

pcm=ax1.contourf(theta, r, E, levs, norm=colors.PowerNorm(gamma=0.4), cmap='brg', zorder=0)
ax1.set_title(r"ratio $r=0.12$", pad=20)
ax1.plot([0,np.pi], [kappamin_pos,kappamin_pos],
            markerfacecolor='blue',
            markeredgecolor='white',            # marker edgecolor
            marker='s',                         # each marker will be rendered as a circle
            markersize=6,                       # marker size
            markeredgewidth=1,                  # marker edge width
            linewidth=0,
            zorder=3,
            alpha=1,                           # Change opacity
            label = r"$\kappa_{0^\circ}(m^{-1})$")        
ax1.plot(-np.pi/2, kappamin_neg,
            markerfacecolor='green',
            markeredgecolor='white',            # marker edgecolor
            marker='D',                         # each marker will be rendered as a circle
            markersize=6,                       # marker size
            markeredgewidth=1,                  # marker edge width
            linewidth=0,                      # line width
            zorder=3,
            alpha=1,
            label = r"$\kappa_{90^\circ}(m^{-1})$")
#
colorbarticks=np.geomspace(0.00001,Emax,10)
cbar = plt.colorbar(pcm, ax=[ax1,ax2], extend='max', ticks=colorbarticks, pad=0.1, orientation="horizontal")
#bounds = ['0','1/80','1/20','1/5','4/5']
cbar.ax.tick_params(labelsize=8)
#cbar.set_ticklabels(bounds)

# rotate radius axis labels.
ax1.set_rlabel_position(135)


#--- Select specific experiment [x, y] with date from results_x/y
#
index=[5,6]
# Read Q and B
Q, B = get_Q_B(basePath,index)
# 
# Compute lokal and global minimizer
kappa=0
kappa_pos=0
kappa_neg=0
#
for i in range(0,N): 
    for j in range(0,N):     
        if theta[i,j]<np.pi:
            E[i,j]=energy(r[i,j],theta[i,j],Q,B)
        else:
            E[i,j]=energy(-r[i,j],theta[i,j],Q,B)
#        
# Compute Minimizer
[imin,jmin]=np.unravel_index(E.argmin(),(N,N))
kappamin=r[imin,jmin]
alphamin=theta[imin,jmin]
# Positiv curvature region
N_mid=int(N/2)
[imin,jmin]=np.unravel_index(E[:N_mid,:].argmin(),(N_mid,N))
kappamin_pos=r[imin,jmin]
alphamin_pos=theta[imin,jmin]
Emin_pos=E[imin,jmin]
# Negative curvature region
[imin,jmin]=np.unravel_index(E[N_mid:,:].argmin(),(N_mid,N))
kappamin_neg=r[imin+N_mid,jmin]
alphamin_neg=theta[imin+N_mid,jmin]
Emin_neg=E[imin+N_mid,jmin]
#
E=E-E.min()

print('Plot energy landscape for dataset ' + str(index))
print('---- r = 0.49 ------')
print('kappamin_pos: ', kappamin_pos)
print('alphamin_pos: ', alphamin_pos)
print('kappamin_neg: ', kappamin_neg)
print('alphamin_neg: ', alphamin_neg)
print('kappamin: ', kappamin)
print('alphamin: ', alphamin)

# Plot 2. energy landscape

pcm2=ax2.contourf(theta, r, E, levs, norm=colors.PowerNorm(gamma=0.4), cmap='brg', zorder=0)
ax2.set_title(r"ratio $r=0.49$", pad=20)
ax2.plot([0,np.pi], [kappamin_pos,kappamin_pos],
            # markerfacecolor='green',
            # markeredgecolor='white',            # marker edgecolor
            markerfacecolor='blue',
            markeredgecolor='white',            # marker edgecolor
            # marker='D',                         # each marker will be rendered as a circle
            marker='s',                         # each marker will be rendered as a circle
            markersize=6,                       # marker size
            markeredgewidth=1,                  # marker edge width
            linewidth=0,
            zorder=3,
            alpha=1,                           # Change opacity
            label = r"$\kappa_{0^\circ}(m^{-1})$")        
ax2.plot(-np.pi/2, kappamin_neg,
            # markerfacecolor='blue',
            # markeredgecolor='white',            # marker edgecolor
            markerfacecolor='green',
            markeredgecolor='white',            # marker edgecolor
            # marker='s',                         # each marker will be rendered as a circle
            marker='D',                         # each marker will be rendered as a circle
            markersize=6,                       # marker size
            markeredgewidth=1,                  # marker edge width
            linewidth=0,                      # line width
            zorder=3,
            alpha=1,                           # Change opacity
            label = r"$\kappa_{90^\circ}(m^{-1})$")


# Plot global minimizer:
# ax2.plot(alphamin, kappamin,
#             markerfacecolor='black',
#             markeredgecolor='white',            # marker edgecolor
#             marker='s',                         # each marker will be rendered as a circle
#             markersize=6,                       # marker size
#             markeredgewidth=1,                  # marker edge width
#             linewidth=0,                      # line width
#             zorder=3,
#             alpha=1                           # Change opacity
#             )


# rotate radius axis labels.
ax2.set_rlabel_position(135)

anglelabel=["0°","45°", "90°", "135°","180°","135°","90°","45°"]
ax1.set_xticks(np.array([.0,1/4,2/4,3/4,1,5/4,6/4,7/4])*np.pi)
ax1.set_xticklabels(anglelabel)
ax1.xaxis.grid(True, color='white')
ax1.set_yticklabels(["1","2","3","4"], zorder=20, color="white")
ax1.yaxis.set_tick_params(color='white')
ax1.yaxis.grid(True, color='white', alpha=0.75)
ax2.set_xticks(np.array([.0,1/4,2/4,3/4,1,5/4,6/4,7/4])*np.pi)
ax2.set_xticklabels(anglelabel)
ax2.xaxis.grid(True, color='white')
ax2.set_yticklabels(["1","2","3","4"], zorder=20, color="white")
ax2.yaxis.set_tick_params(color='white')
ax2.yaxis.grid(True, color='white', alpha=0.75)
# plt.show()
# Save Figure as .pdf
#width = 5.79 
#height = width / 1.618 # The golden ratio.
#fig.set_size_inches(width, height)
fig.savefig(dirname + '/wood-bilayer_PLOS_landscape.png', dpi=300)
# fig.savefig('./wood-bilayer_PLOS_landscape.pdf')



# # ----------------------------------------------------------------------------------------
# # ------------------------------ EXPERIMENT 2 --------------------------------------------
# # ----------------------------------------------------------------------------------------
# # ----------------------------------------------------------------------------------------
# # ### Plot diagrams: Perforated bilayer 
    
print('Create plots for perforated bilayers ...')


# Choose between 'upper' or 'lower' layer
# perforatedLayer = 'lower'
# perforatedLayer = 'upper'

# basePath = dirname + '/perforations' + orientation
basePath = dirname + '/perforated-bilayer'


perforatedLayers = ['lower', 'upper']


# -------------------------------------------------------------------------------
### PARAMETERS FROM SIMULATION 

# [r, h, omega_flat, omega_target, theta, Kappa_experimental]
# r = (thickness upper layer)/(thickness)
# h = thickness [meter]
# omega_flat = moisture content in the flat state before drying [%]
# omega_target = moisture content in the target state [%]
# theta = rotation angle (not implemented and used)

# [r, h, omega_flat, omega_target, theta, Kappa_experimental]
# r = (thickness upper layer)/(thickness)
# h = thickness [meter]
# omega_flat = moisture content in the flat state before drying [%]
# omega_target = moisture content in the target state [%]
# theta = volume fraction

materialFunctionParameter=[
    [  # Dataset Ratio r = 0.12
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.17
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [ # Dataset Ratio r = 0.22
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.05 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.15 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.25 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.34
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.43
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.49
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.05],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.15],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.25],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.3 ]
    ]
]

for perforatedLayer in perforatedLayers:

    # -------------------------------------------------------------------------------
    ### Compute all local, global minimizers and store in arrays


    n=len(materialFunctionParameter)
    m=len(materialFunctionParameter[0])
    kappa_0=np.zeros([n,m])
    energy_0=np.zeros([n,m])
    kappa_90=np.zeros([n,m])
    energy_90=np.zeros([n,m])
    energy_global=np.zeros([n,m])
    kappa_global_min=np.zeros([n,m])
    energy_origin=np.zeros([n,m])
    for i in range(0,n):
        for j in range(0,m):
            Q, B=get_Q_B(basePath, [i,j],True,perforatedLayer)
            minimizers=get_local_minimizer_on_axes(Q,B)
            kappa_0[i,j]=minimizers[0,0]
            energy_0[i,j]=energy(kappa_0[i,j],0,Q,B)
            kappa_90[i,j]=minimizers[1,0]
            energy_90[i,j]=energy(kappa_90[i,j],np.pi/2,Q,B)
            if energy_0[i,j]<energy_90[i,j]:
                kappa_global_min[i,j]=kappa_0[i,j]
                energy_global[i,j]=energy_0[i,j]
            else:
                kappa_global_min[i,j]=kappa_90[i,j]
                energy_global[i,j]=energy_90[i,j]
            energy_origin[i,j]=energy(0,0,Q,B)

    # -------------------------------------------------------------------------------
    ### Plot diagrams: perforated bilayers
            
    print('Create plots for perforated wooden bilayers ...')
            

    # Switch for local minimizers
    ShowLocal=True
    # ShowLocal=False


    plt.style.use("seaborn")
    mpl.rcParams['text.usetex'] = True
    mpl.rcParams["font.family"] = "serif"
    mpl.rcParams["font.size"] = "8"
    mpl.rcParams['xtick.bottom'] = True
    mpl.rcParams['xtick.major.size'] = 2
    mpl.rcParams['xtick.minor.size'] = 1.5
    mpl.rcParams['xtick.major.width'] = 0.75
    mpl.rcParams['xtick.labelsize'] = 8
    mpl.rcParams['xtick.major.pad'] = 1

    mpl.rcParams['ytick.left'] = True
    mpl.rcParams['ytick.major.size'] = 2
    mpl.rcParams['ytick.minor.size'] = 1.5
    mpl.rcParams['ytick.major.width'] = 0.75
    mpl.rcParams['ytick.labelsize'] = 8
    mpl.rcParams['ytick.major.pad'] = 1

    mpl.rcParams['axes.titlesize'] = 8
    mpl.rcParams['axes.titlepad'] = 1
    mpl.rcParams['axes.labelsize'] = 8

    #Adjust Legend:
    mpl.rcParams['legend.frameon'] = True       # Use frame for legend
    # mpl.rcParams['legend.framealpha'] = 0.5 
    mpl.rcParams['legend.fontsize'] = 8         # fontsize of legend


    #Adjust grid:
    mpl.rcParams.update({"axes.grid" : True}) # Add grid
    mpl.rcParams['axes.labelpad'] = 3
    mpl.rcParams['grid.linewidth'] = 0.25
    mpl.rcParams['grid.alpha'] = 0.9 # 0.75
    mpl.rcParams['grid.linestyle'] = '-'
    mpl.rcParams['grid.color']   = 'gray'#'black'
    mpl.rcParams['text.latex.preamble'] = r'\usepackage{amsfonts}' # Makes Use of \mathbb possible.
    # ----------------------------------------------------------------------------------------
    # width = 5.79
    # height = width / 1.618 # The golden ratio.
    textwidth = 6.26894 #textwidth in inch
    width = textwidth * 0.5
    height = width/1.618 # The golden ratio.

    fig, ax = plt.subplots(figsize=(width,height))
    fig.subplots_adjust(left=.15, bottom=.16, right=.95, top=.92)

    # ax.tick_params(axis='x',which='major', direction='out',pad=3)

    # we use the ratios r=0.12 and r=0.49
    dataset_numbers = [0, 5]
    # dataset_numbers = [0, 1]

    # for i in range(0,n):
    for i in dataset_numbers:
        #ax.set_ylim(0,4.5)
        # ax.set_xlim(0.0, 0.35)
        ax.xaxis.set_major_locator(MultipleLocator(0.05))
        ax.xaxis.set_minor_locator(MultipleLocator(0.025))    
        ax.yaxis.set_major_locator(MultipleLocator(1.0))
        # ax.yaxis.set_major_locator(MultipleLocator(0.5))
        # data=np.zeros([7,m])
        data=np.zeros([4,m])
        # data[0]=np.array(materialFunctionParameter[i])[:,3] # omega_0
        # data[0]=np.array(materialFunctionParameter[0])[:,5] # volume ratio (perforation)
        # data[1]=kappa_0[0,:]
        # data[2]=kappa_90[0,:]
        # data[3]=kappa_global_min[0,:]
        data[0]=np.array(materialFunctionParameter[i])[:,5] # volume ratio (perforation)
        data[1]=kappa_0[i,:]
        data[2]=kappa_90[i,:]
        data[3]=kappa_global_min[i,:]

        # if i == 0:
        #     color = 'forestgreen'
        #     label = r"$r = 0.12$ "
        # else :
        #     color = 'cornflowerblue'
        #     label = r"$r = 0.49$ "
        # # data[1]=kappa_0[1,:]
        # data[2]=kappa_90[1,:]
        # data[3]=kappa_global_min[1,:]
        # data[4]=np.array(materialFunctionParameter[i])[:,5] # experimental curvature
        # relative_error = (np.array(data[dataset_number][1]) - np.array(dataset[dataset_number][2])) / np.array(dataset[dataset_number][2])
        #print('relative_error:', relative_error)

        # print('global minimizer curvature:', data[3])

        #--------------- Plot Lines + Scatter -----------------------
        # Curvature (absolute value) with angle 0
        if ShowLocal:          
            line_1 = ax.plot(np.array(data[0]), np.array(data[1]),                    # data
                        color='forestgreen',              # linecolor
                        marker='o',                         # each marker will be rendered as a circle
                        markersize=2.5,                       # marker size
                        markerfacecolor='forestgreen',      # marker facecolor
                        markeredgecolor='black',            # marker edgecolor
                        markeredgewidth=0.5,                  # marker edge width
                        # linestyle='dashdot',              # line style will be dash line
                        linewidth=1,                      # line width
                        zorder=3,
                        label = r"$0^\circ$")
                        # label = label)
            # Curvature with angle 90
            line_2 = ax.plot(np.array(data[0]), np.array(data[2]),                    # data
                        color='cornflowerblue',                # linecolor
                        marker='x',                         # each marker will be rendered as a circle
                        markersize=2.5,                       # marker size
                        markerfacecolor='cornflowerblue',   # marker facecolor
                        markeredgecolor='black',            # marker edgecolor
                        markeredgewidth=0.5,                  # marker edge width
                        # linestyle='--',                   # line style will be dash line
                        linewidth=1,                      # line width
                        zorder=3,
                        alpha=0.8,                    # Change opacity
                        label = r"$90^$")
            
            # Global minimizer
            line_3 = ax.plot(np.array(data[0]), np.array(data[3]),                    # data
                        # color='orangered',                # linecolor
                        marker='s',                         # each marker will be rendered as a circle
                        markersize=5,                       # marker size
                        #  markerfacecolor='cornflowerblue',   # marker facecolor
                        markeredgecolor='black',            # marker edgecolor
                        markeredgewidth=0.5,                  # marker edge width
                        # linestyle='--',                   # line style will be dash line
                        linewidth=0,                      # line width
                        zorder=3,
                        alpha=0.2,                         # Change opacity
                        label = r"global min.")
        else:               
            # Global minimizer
            line_3 = ax.plot(np.array(data[0]), np.array(data[3]),                    # data
                        color='forestgreen',              # linecolor
                        marker='o',                         # each marker will be rendered as a circle
                        markersize=1.5,                       # marker size
                        markerfacecolor='forestgreen',      # marker facecolor
                        markeredgecolor='black',            # marker edgecolor
                        markeredgewidth=0.5,                  # marker edge width
                        # linestyle='dashdot',              # line style will be dash line
                        linewidth=1,                      # line width
                        zorder=3,
                        label = r"$\kappa$")



        # print('global minimizer curvature:', data[3])
        print('----- Compute curvature difference  due to perforations: -----')
        print(perforatedLayer + ' layer perforation.')
        print('---- Dataset:', i)
        print('curvature for (0° - Minimizers) :', data[1])
        print('curvature for (90° - Minimizers) :', data[2])
        print('relative error  - (0° - Minimizers) : ', abs(data[1][0] - data[1][-1])/abs(data[1][-1]) )
        print('relative error  - (90° - Minimizers) : ', abs(data[2][0] - data[2][-1])/abs(data[2][-1]) )



        # Plot - Titel
        if perforatedLayer == 'upper' :
            ax.set_title(r"perforations in upper layer, ratio $r = $" +str(materialFunctionParameter[i][0][0]), pad=5.0) 
        else :
            ax.set_title(r"perforations in lower layer, ratio $r = $" +str(materialFunctionParameter[i][0][0]), pad=5.0) 
        # ax.set_title(perforatedLayer + ' layer perforation ' ) 
        # ax.set_xlabel(r"Wood moisture content $\omega (\%)$", labelpad=4)
        ax.set_xlabel(r"Volume fraction $\beta $", labelpad=4)
        ax.set_ylabel(r"Curvature $\kappa$($m^{-1}$)", labelpad=4)
        plt.tight_layout()


        # ---------- Output Figure as pdf:
        fig.set_size_inches(width, height)
        if ShowLocal:
            fig.savefig(dirname + '/WoodBilayer_perforation' + '_' + perforatedLayer + '_' + str(i)  + '_local_'  + '.pdf')
        else:
            fig.savefig(dirname + '/WoodBilayer_perforation' + '_'+  perforatedLayer + '_' + str(i)  + '_global_' + '.pdf')        
        plt.cla()


# ----------------------------------------------------------------------------------------
# ------------------------------ EXPERIMENT 3 --------------------------------------------
# ----------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------
### Wooden bilayer example with rotated lower layer:
        
# basePath = dirname + '/rotatedLayer'
basePath = dirname + '/wood-bilayer-rotatedLayer'


# -------------------------------------------------------------------------------
### PARAMETERS FROM SIMULATION 

# [r, h, omega_flat, omega_target, theta, Kappa_experimental]
# r = (thickness upper layer)/(thickness)
# h = thickness [meter]
# omega_flat = moisture content in the flat state before drying [%]
# omega_target = moisture content in the target state [%]
# theta = rotation angle (not implemented and used)

# [r, h, omega_flat, omega_target, theta, Kappa_experimental]
# r = (thickness upper layer)/(thickness)
# h = thickness [meter]
# omega_flat = moisture content in the flat state before drying [%]
# omega_target = moisture content in the target state [%]
# theta = volume fraction

materialFunctionParameter=[
[  # Dataset Ratio r = 0.12
[0.12, 0.0047, 17.32986047, 9.005046347, np.pi/12.0, 4.312080261],
[0.12, 0.0047, 17.32986047, 9.005046347, np.pi/6.0, 4.312080261],
[0.12, 0.0047, 17.32986047, 9.005046347, np.pi/4.0, 4.312080261],
[0.12, 0.0047, 17.32986047, 9.005046347, np.pi/3.0, 4.312080261],
[0.12, 0.0047, 17.32986047, 9.005046347, 5.0*np.pi/12.0, 4.312080261],
[0.12, 0.0047, 17.32986047, 9.005046347, np.pi/2.0, 4.312080261],
[0.12, 0.0047, 17.32986047, 9.005046347, 7.0*np.pi/12.0, 4.312080261]
]
]

# -------------------------------------------------------------------------------
### Compute all local, global minimizers and store in arrays


n=len(materialFunctionParameter)
m=len(materialFunctionParameter[0])
kappa_0=np.zeros([n,m])
energy_0=np.zeros([n,m])
kappa_90=np.zeros([n,m])
energy_90=np.zeros([n,m])
energy_global=np.zeros([n,m])
kappa_global_min=np.zeros([n,m])
energy_origin=np.zeros([n,m])
for i in range(0,n):
    for j in range(0,m):
        Q, B=get_Q_B(basePath,[i,j])
        minimizers=get_local_minimizer_on_axes(Q,B)
        kappa_0[i,j]=minimizers[0,0]
        energy_0[i,j]=energy(kappa_0[i,j],0,Q,B)
        kappa_90[i,j]=minimizers[1,0]
        energy_90[i,j]=energy(kappa_90[i,j],np.pi/2,Q,B)
        if energy_0[i,j]<energy_90[i,j]:
            kappa_global_min[i,j]=kappa_0[i,j]
            energy_global[i,j]=energy_0[i,j]
        else:
            kappa_global_min[i,j]=kappa_90[i,j]
            energy_global[i,j]=energy_90[i,j]
        energy_origin[i,j]=energy(0,0,Q,B)


# -------------------------------------------------------------------------------
### Plot diagrams: rotated example - Energy landscape (Q - Q_min)
    
print('Create plots of energy landscape (rotated lower layer example)...')
        
# Number of experiments / folders
show_plot = False

fig, (ax1) = plt.subplots( nrows=1,figsize=(3,3.5),subplot_kw=dict(projection='polar'), gridspec_kw={'hspace': 0.4})
# fig, (ax1, ax2) = plt.subplots( nrows=2,figsize=(3,7),subplot_kw=dict(projection='polar'), gridspec_kw={'hspace': 0.4})
Emax=31000
levs=np.geomspace(1,Emax,200)



#--- Select specific experiment [x, y] with date from results_x/y
#
index=[0,2]
# Read Q and B
Q, B = get_Q_B(basePath,index)
# 
# Compute lokal and global minimizer
kappa=0
kappa_pos=0
kappa_neg=0
#
N=500
length=5
r, theta = np.meshgrid(np.linspace(0,length,N),np.radians(np.linspace(0, 360, N)))
E=np.zeros(np.shape(r))
for i in range(0,N): 
    for j in range(0,N):     
        if theta[i,j]<np.pi:
            E[i,j]=energy(r[i,j],theta[i,j],Q,B)
        else:
            E[i,j]=energy(-r[i,j],theta[i,j],Q,B)
#        
# Compute Minimizer
[imin,jmin]=np.unravel_index(E.argmin(),(N,N))
kappamin=r[imin,jmin]
alphamin=theta[imin,jmin]
# Positiv curvature region
N_mid=int(N/2)
[imin,jmin]=np.unravel_index(E[:N_mid,:].argmin(),(N_mid,N))
kappamin_pos=r[imin,jmin]
alphamin_pos=theta[imin,jmin]
Emin_pos=E[imin,jmin]
# Negative curvature region
[imin,jmin]=np.unravel_index(E[N_mid:,:].argmin(),(N_mid,N))
kappamin_neg=r[imin+N_mid,jmin]
alphamin_neg=theta[imin+N_mid,jmin]
Emin_neg=E[imin+N_mid,jmin]
#
E=E-E.min()

pcm=ax1.contourf(theta, r, E, levs, norm=colors.PowerNorm(gamma=0.4), cmap='brg', zorder=0)
ax1.set_title(r"ratio $r=0.12$",pad=20)


print('kappamin_pos: ', kappamin_pos)
print('alphamin_pos: ', alphamin_pos)
print('kappamin_neg: ', kappamin_neg)
print('alphamin_neg: ', alphamin_neg)

ax1.plot(alphamin_neg, kappamin_neg,
            markerfacecolor='green',
            markeredgecolor='white',            # marker edgecolor
            marker='D',                         # each marker will be rendered as a circle
            markersize=6,                       # marker size
            markeredgewidth=1,                  # marker edge width
            linewidth=0,                      # line width
            zorder=3,
            alpha=1)

ax1.plot(alphamin, kappamin,
            markerfacecolor='blue',
            markeredgecolor='white',            # marker edgecolor
            marker='s',                         # each marker will be rendered as a circle
            markersize=6,                       # marker size
            markeredgewidth=1,                  # marker edge width
            linewidth=0,
            zorder=3,
            alpha=1,                           # Change opacity
            label = r"$\kappa_{0^\circ}(m^{-1})$")        


print('alphamin: ' , alphamin ) 
print('kappamin: ' , kappamin ) 
print('Global Minimizer (alphamin, kappamin):' , '('+ str(np.round(np.rad2deg(alphamin),decimals=1)) + ',' + str(np.round(kappamin,decimals=1))+ ')' ) 
print('Local Minimizer (alphamin, kappamin):' , '('+ str(np.round(360-np.rad2deg(alphamin_neg),decimals=1)) + ',' + str(np.round(-kappamin_neg,decimals=1))+ ')' ) 
# ax1.annotate(alphamin, (alphamin,kappamin))

#--- annotate global minimizer
ax1.annotate( '('+ str(np.round(np.rad2deg(alphamin),decimals=1)) + '°' + ',' + str(np.round(kappamin,decimals=1))+ ')' ,
# ax1.annotate( r'$(\theta_{m}, \kappa_m)=$' + '('+ str(np.round(np.rad2deg(alphamin),decimals=1)) + '°' + ',' + str(np.round(kappamin,decimals=1))+ ')' ,
              xy = (alphamin,kappamin),
              xytext=(0.10, np.round(kappamin,decimals=1)-0.25),
              color='ivory',
              fontsize='8')



#--- annotate local minimizer
ax1.annotate( '('+ str(np.round(360-np.rad2deg(alphamin_neg),decimals=1)) + '°' + ',' + str(np.round(-kappamin_neg,decimals=1))+ ')' ,
              xy = (alphamin_neg,kappamin_neg),
              xytext=(-1.6, np.round(kappamin_neg,decimals=1)+0.9),
              color='ivory',
              # zorder=5,
              fontsize='8')


# rotate radius axis labels.
ax1.set_rlabel_position(135)


colorbarticks=np.geomspace(0.00001,Emax,10)
cbar = plt.colorbar(pcm, ax=[ax1], extend='max', ticks=colorbarticks, pad=0.1, orientation="horizontal")
cbar.ax.tick_params(labelsize=8)

anglelabel=["0°","45°", "90°", "135°","180°","135°","90°","45°"]
ax1.set_xticks(np.array([.0,1/4,2/4,3/4,1,5/4,6/4,7/4])*np.pi)
ax1.set_xticklabels(anglelabel)
ax1.xaxis.grid(True, color='white')
ax1.set_yticklabels(["1","2","3","4"], zorder=20, color="white")
ax1.yaxis.set_tick_params(color='white')
ax1.yaxis.grid(True, color='white', alpha=0.75)
# plt.show()
# Save Figure as .pdf
#width = 5.79 
#height = width / 1.618 # The golden ratio.
#fig.set_size_inches(width, height)
# fig.savefig(dirname + '/wood-bilayer_rotatedLayer_landscape.pdf')
fig.savefig(dirname + '/wood-bilayer_rotatedLayer_landscape.png', dpi=300)