import math

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()

#############################################
#  Paths
#############################################
parameterSet.outputPath = '/home/klaus/Desktop/Dune_release/dune-microstructure/outputs'
parameterSet.baseName= 'material_neukamm'   #(needed for Output-Filename)

#############################################
#  Material Setup
#############################################

# --- Choose scale ratio gamma:
parameterSet.gamma = 1.0

# --- Number of material phases
parameterSet.Phases = 3

#--- Indicator function for material phases
# x[0] : y1-component
# x[1] : y2-component
# x[2] : x3-component
#To indicate phases return either : 1 / 2 / 3
###############
# Cross
###############
def indicatorFunction(x):
    theta=0.25
    factor=1
    if (x[0] <-0.5+theta and x[2]<-0.5+theta):
        return 1    #Phase1
    elif (x[1]<-0.5+theta and x[2]>0.5-theta):
        return 2    #Phase2
    else :
        return 3   #Phase3
    
########### Options for material phases: #################################
#     1. "isotropic"     2. "orthotropic"      3. "transversely_isotropic"   4. "general_anisotropic"
#########################################################################
## Notation - Parameter input :
# isotropic (Lame parameters) : [mu , lambda]
#         orthotropic         : [E1,E2,E3,G12,G23,G31,nu12,nu13,nu23]   # see https://en.wikipedia.org/wiki/Poisson%27s_ratio with x=1,y=2,z=3
# transversely_isotropic      : [E1,E2,G12,nu12,nu23]
# general_anisotropic         : full compliance matrix C
######################################################################

#--- Define different material phases:
#- PHASE 1
parameterSet.phase1_type="isotropic"
materialParameters_phase1 = [80, 80]

#- PHASE 2
parameterSet.phase2_type="isotropic"
materialParameters_phase2 = [80, 80]

#- PHASE 3
parameterSet.phase3_type="isotropic"
materialParameters_phase3 = [60, 25]

#--- Define prestrain function for each phase (also works with non-constant values)
def prestrain_phase1(x):
    return [[1, 0, 0], [0,1,0], [0,0,1]]

def prestrain_phase2(x):
    return [[1, 0, 0], [0,1,0], [0,0,1]]

def prestrain_phase3(x):
    return [[0, 0, 0], [0,0,0], [0,0,0]]



#############################################
#  Grid parameters
#############################################
## numLevels : Number of Levels on which solution is computed. starting with a 2x2x2 cube mesh.
## {start,finish} computes on all grid from 2^(start) to 2^finish refinement
#----------------------------------------------------
parameterSet.numLevels= '3 3'      # computes all levels from first to second entry

#############################################
#  Assembly options
#############################################
parameterSet.set_IntegralZero = 1            #(default = false)
parameterSet.set_oneBasisFunction_Zero = 1   #(default = false)
#parameterSet.arbitraryLocalIndex = 7            #(default = 0)
#parameterSet.arbitraryElementNumber = 3         #(default = 0)

#############################################
#  Solver Options, Type: #1: CG - SOLVER , #2: GMRES - SOLVER, #3: QR - SOLVER (default), #4: UMFPACK - SOLVER
#############################################
parameterSet.Solvertype = 3        # recommended to use iterative solver (e.g GMRES) for finer grid-levels
parameterSet.Solver_verbosity = 0  #(default = 2)  degree of information for solver output


#############################################
#  Write/Output options      #(default=false)
#############################################
# --- (Optional output) write Material / prestrain / Corrector functions to .vtk-Files:
parameterSet.write_materialFunctions = 1   # VTK indicator function for material/prestrain definition
#parameterSet.write_prestrainFunctions = 1  # VTK norm of B (currently not implemented)

# --- (Additional debug output)
parameterSet.print_debug = 0  #(default=false)

# --- Write Correctos to VTK-File:  
parameterSet.write_VTK = 1

# --- (Optional output) L2Error, integral mean: 
#parameterSet.write_L2Error = 1
#parameterSet.write_IntegralMean = 1      

# --- check orthogonality (75) from paper: 
parameterSet.write_checkOrthogonality = 1

# --- Write corrector-coefficients to log-File:
#parameterSet.write_corrector_phi1 = 1
#parameterSet.write_corrector_phi2 = 1
#parameterSet.write_corrector_phi3 = 1

# --- Print Condition number of matrix (can be expensive):
#parameterSet.print_conditionNumber= 1  #(default=false)

# --- write effective quantities to Matlab-folder for symbolic minimization:
#parameterSet.write_toMATLAB = 0  # writes effective quantities to .txt-files QMatrix.txt and BMatrix.txt
