import subprocess
import re
import os
import numpy as np
import matplotlib.pyplot as plt
import math
import fileinput
import time
import matplotlib.ticker as tickers
import matplotlib as mpl
from matplotlib.ticker import MultipleLocator,FormatStrFormatter,MaxNLocator
import codecs
import sys
import threading

# # Schreibe input datei für Parameter
# def SetParametersCellProblem(ParameterSet, ParsetFilePath, outputPath):
#     print('----set Parameters -----')
#     with open(ParsetFilePath, 'r') as file:
#         filedata = file.read()
#         filedata = re.sub('(?m)^materialFunction\s?=.*','materialFunction = '+str(ParameterSet.materialFunction),filedata)
#         filedata = re.sub('(?m)^gamma\s?=.*','gamma='+str(ParameterSet.gamma),filedata)
#         filedata = re.sub('(?m)^numLevels\s?=.*','numLevels='+str(ParameterSet.numLevels)+' '+str(ParameterSet.numLevels) ,filedata)
#         filedata = re.sub('(?m)^outputPath\s?=\s?.*','outputPath='+str(outputPath),filedata)
#         f = open(ParsetFilePath,'w')
#         f.write(filedata)
#         f.close()


# # Ändere Parameter der MaterialFunction
# def SetParameterMaterialFunction(materialFunction, parameterName, parameterValue):
#     with open(Path+"/"+materialFunction+'.py', 'r') as file:
#         filedata = file.read()
#         filedata = re.sub('(?m)^'+str(parameterName)+'\s?=.*',str(parameterName)+' = '+str(parameterValue),filedata)
#         f = open(Path+"/"+materialFunction+'.py','w')
#         f.write(filedata)
#         f.close()

def SetParameterMaterialFunction(inputFunction, parameterName, parameterValue):
    with open(inputFunction+'.py', 'r') as file:
        filedata = file.read()
        filedata = re.sub('(?m)^'+str(parameterName)+'\s?=.*',str(parameterName)+' = '+str(parameterValue),filedata)
        f = open(inputFunction+'.py','w')
        f.write(filedata)
        f.close()



# # Rufe Programm zum Lösen des Cell-Problems auf
# def run_CellProblem(executable, parset,write_LOG):
#     print('----- RUN Cell-Problem ----')
#     processList = []
#     LOGFILE = "Cell-Problem_output.log"
#     print('LOGFILE:',LOGFILE)
#     print('executable:',executable)
#     if write_LOG:
#         p = subprocess.Popen(executable + parset
#                                         + " | tee " + LOGFILE, shell=True)

#     else:
#         p = subprocess.Popen(executable + parset
#                                         + " | tee " + LOGFILE, shell=True)

#     p.wait() # wait
#     processList.append(p)
#     exit_codes = [p.wait() for p in processList]

#     return

# Read effective quantites
def ReadEffectiveQuantities(QFilePath = os.path.dirname(os.getcwd()) + '/outputs/QMatrix.txt', BFilePath = os.path.dirname(os.getcwd())+ '/outputs/BMatrix.txt'):
    # Read Output Matrices (effective quantities)
    # From Cell-Problem output Files : ../outputs/Qmatrix.txt , ../outputs/Bmatrix.txt
    # -- Read Matrix Qhom
    X = []
    # with codecs.open(path + '/outputs/QMatrix.txt', encoding='utf-8-sig') as f:
    with codecs.open(QFilePath, encoding='utf-8-sig') as f:
        for line in f:
            s = line.split()
            X.append([float(s[i]) for i in range(len(s))])
    Q = np.array([[X[0][2], X[1][2], X[2][2]],
                  [X[3][2], X[4][2], X[5][2]],
                  [X[6][2], X[7][2], X[8][2]] ])

    # -- Read Beff (as Vector)
    X = []
    # with codecs.open(path + '/outputs/BMatrix.txt', encoding='utf-8-sig') as f:
    with codecs.open(BFilePath, encoding='utf-8-sig') as f:
        for line in f:
            s = line.split()
            X.append([float(s[i]) for i in range(len(s))])
    B = np.array([X[0][2], X[1][2], X[2][2]])
    return Q, B

# Function for evaluating the energy in terms of kappa, alpha and Q, B
def eval_energy(kappa,alpha,Q,B)  :
    G=kappa*np.array([[np.cos(alpha)**2],[np.sin(alpha)**2],[np.sqrt(2)*np.cos(alpha)*np.sin(alpha)]])-B
    return np.matmul(np.transpose(G),np.matmul(Q,G))[0,0]

#-------------------------------------------------------------------------------------------------------
########################
#### SET PARAMETERS ####
########################
# ----- Setup Paths -----
# write_LOG = True   # writes Cell-Problem output-LOG in "Cell-Problem_output.log"
# path='/home/klaus/Desktop/Dune_release/dune-microstructure/experiment/wood-bilayer/results/'  
# pythonPath = '/home/klaus/Desktop/Dune_release/dune-microstructure/experiment/wood-bilayer'

#--- Choose wether to perforate upper (passive) or lower (active) layer
perforatedLayer = 'upper'
# perforatedLayer = 'lower'

# dataset_numbers = [0, 1, 2, 3, 4, 5]
# dataset_numbers = [0, 5]
dataset_numbers = [1,2,3,4]
# dataset_numbers = [0]

for dataset_number in dataset_numbers:
    print("------------------")
    print(str(dataset_number) + "th data set")
    print("------------------")


    # path = os.getcwd() + '/experiment/perforated-bilayer/results_' +  perforatedLayer + '/'
    path = os.getcwd() + '/experiment/perforated-bilayer/results_' +  perforatedLayer + '_' + str(dataset_number) + '/'
    pythonPath = os.getcwd() + '/experiment/perforated-bilayer'
    pythonModule = "perforated_wood_" + perforatedLayer
    executable = os.getcwd() + '/build-cmake/src/Cell-Problem'
    # ---------------------------------
    # Setup Experiment
    # ---------------------------------
    gamma = 1.0

    # ----- Define Parameters for Material Function  --------------------
    # [r, h, omega_flat, omega_target, theta, beta]
    # r = (thickness upper layer)/(thickness)
    # h = thickness [meter]
    # omega_flat = moisture content in the flat state before drying [%]
    # omega_target = moisture content in the target state [%]
    # theta = rotation angle (not implemented and used)
    # beta = design parameter for perforation = ratio of Volume of (cylindrical) perforation to Volume of active/passive layer 

    #Experiment: Perforate "active"/"passive" bilayer phase 


    materialFunctionParameter=[
    [  # Dataset Ratio r = 0.12
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.12, 0.0047, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.17
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.17, 0.0049, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [ # Dataset Ratio r = 0.22
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.05 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.15 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.25 ],
    [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.34
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.34, 0.0063, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.43
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.05],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.15],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.25],
    [0.43, 0.0073, 17.17547062, 8.959564147, 0.0, 0.3 ]
    ],
    [  # Dataset Ratio r = 0.49
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.0 ],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.05],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.1 ],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.15],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.2 ],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.25],
    [0.49, 0.008,  17.17547062, 8.959564147, 0.0, 0.3 ]
    ]
    ]
    


    #--- Different moisture values for different thicknesses:

    # materialFunctionParameter=[
    # [  # Dataset Ratio r = 0.12
    # [0.12, 0.0047, 17.32986047, 14.70179844, 0.0, 0.0 ],
    # [0.12, 0.0047, 17.32986047, 13.6246,     0.0, 0.05],
    # [0.12, 0.0047, 17.32986047, 12.42994508, 0.0, 0.1 ],
    # [0.12, 0.0047, 17.32986047, 11.69773413, 0.0, 0.15],
    # [0.12, 0.0047, 17.32986047, 11.14159987, 0.0, 0.2 ],
    # [0.12, 0.0047, 17.32986047, 9.500670278, 0.0, 0.25],
    # [0.12, 0.0047, 17.32986047, 9.005046347, 0.0, 0.3 ]
    # ],
    # [  # Dataset Ratio r = 0.17
    # [0.17, 0.0049, 17.28772791 , 14.75453569, 0.0, 0.0 ],
    # [0.17, 0.0049, 17.28772791 , 13.71227639, 0.0, 0.05],
    # [0.17, 0.0049, 17.28772791 , 12.54975012, 0.0, 0.1 ],
    # [0.17, 0.0049, 17.28772791 , 11.83455959, 0.0, 0.15],
    # [0.17, 0.0049, 17.28772791 , 11.29089521, 0.0, 0.2 ],
    # [0.17, 0.0049, 17.28772791 , 9.620608917, 0.0, 0.25],
    # [0.17, 0.0049, 17.28772791 , 9.101671742, 0.0, 0.3 ]
    # ],
    # [ # Dataset Ratio r = 0.22
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.0 ],
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.05 ],
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.1 ],
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.15 ],
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.2 ],
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.25 ],
    # [0.22, 0.0053,  17.17547062, 8.959564147, 0.0, 0.3 ]
    # ],
    # [  # Dataset Ratio r = 0.34
    # [0.34, 0.0063, 17.14061081 , 14.98380876, 0.0, 0.0 ],
    # [0.34, 0.0063, 17.14061081 , 13.97154915, 0.0, 0.05],
    # [0.34, 0.0063, 17.14061081 , 12.77309253, 0.0, 0.1 ],
    # [0.34, 0.0063, 17.14061081 , 12.00959929, 0.0, 0.15],
    # [0.34, 0.0063, 17.14061081 , 11.42001731, 0.0, 0.2 ],
    # [0.34, 0.0063, 17.14061081 , 9.561447179, 0.0, 0.25],
    # [0.34, 0.0063, 17.14061081 , 8.964704969, 0.0, 0.3 ]
    # ],
    # [  # Dataset Ratio r = 0.43
    # [0.43, 0.0073, 17.07559686 , 15.11316339, 0.0, 0.0 ],
    # [0.43, 0.0073, 17.07559686 , 14.17997082, 0.0, 0.05],
    # [0.43, 0.0073, 17.07559686 , 13.05739844, 0.0, 0.1 ],
    # [0.43, 0.0073, 17.07559686 , 12.32309209, 0.0, 0.15],
    # [0.43, 0.0073, 17.07559686 , 11.74608518, 0.0, 0.2 ],
    # [0.43, 0.0073, 17.07559686 , 9.812372466, 0.0, 0.25],
    # [0.43, 0.0073, 17.07559686 , 9.10519385 , 0.0, 0.3 ]
    # ],
    # [  # Dataset Ratio r = 0.49
    # [0.49, 0.008,  17.01520754, 15.30614414, 0.0, 0.0 ],
    # [0.49, 0.008,  17.01520754, 14.49463867, 0.0, 0.05],
    # [0.49, 0.008,  17.01520754, 13.46629742, 0.0, 0.1 ],
    # [0.49, 0.008,  17.01520754, 12.78388234, 0.0, 0.15],
    # [0.49, 0.008,  17.01520754, 12.23057715, 0.0, 0.2 ],
    # [0.49, 0.008,  17.01520754, 10.21852839, 0.0, 0.25],
    # [0.49, 0.008,  17.01520754, 9.341730605, 0.0, 0.3 ]
    # ]
    # ]



    # ------ Loops through Parameters for Material Function -----------
    for i in range(0,np.shape(materialFunctionParameter)[1]):
        print("------------------")
        print("New Loop")
        print("------------------")
    # Check output directory
        outputPath = path + str(i)
        isExist = os.path.exists(outputPath)
        if not isExist:
            # Create a new directory because it does not exist
            os.makedirs(outputPath)
            print("The new directory " + outputPath + " is created!")

        # thread = threading.Thread(target=run_CellProblem(executable, pythonModule, pythonPath, LOGFILE))
        # thread.start()

        #TODO: apperently its not possible to pass a variable via subprocess and "calculate" another input value inside the python file.
        #      Therefore we use this instead.
        SetParameterMaterialFunction(pythonPath + "/" + pythonModule, "param_r",materialFunctionParameter[dataset_number][i][0])
        SetParameterMaterialFunction(pythonPath + "/" + pythonModule, "param_h",materialFunctionParameter[dataset_number][i][1])
        SetParameterMaterialFunction(pythonPath + "/" + pythonModule, "param_omega_flat",materialFunctionParameter[dataset_number][i][2])
        SetParameterMaterialFunction(pythonPath + "/" + pythonModule, "param_omega_target",materialFunctionParameter[dataset_number][i][3])
        SetParameterMaterialFunction(pythonPath + "/" + pythonModule, "param_theta",materialFunctionParameter[dataset_number][i][4])   
        SetParameterMaterialFunction(pythonPath + "/" + pythonModule, "param_beta",materialFunctionParameter[dataset_number][i][5])    

        LOGFILE = outputPath + "/" + pythonModule + "_output" + "_" + str(i) + ".log"

        processList = []
        p = subprocess.Popen(executable + " " + pythonPath + " " + pythonModule
                                        + " -outputPath " + outputPath
                                        + " -gamma " + str(gamma) 
                                        + " | tee " + LOGFILE, shell=True)

        # p = subprocess.Popen(executable + " " + pythonPath + " " + pythonModule
        #                                 + " -outputPath " + outputPath
        #                                 + " -gamma " + str(gamma) 
        #                                 + " -param_r " + str(materialFunctionParameter[i][0])
        #                                 + " -param_h " + str(materialFunctionParameter[i][1])
        #                                 + " -param_omega_flat " + str(materialFunctionParameter[i][2])
        #                                 + " -param_omega_target " + str(materialFunctionParameter[i][3])
        #                                 + " -phase2_angle " + str(materialFunctionParameter[i][4])
        #                                 + " | tee " + LOGFILE, shell=True)

        p.wait() # wait
        processList.append(p)
        exit_codes = [p.wait() for p in processList]
        # ---------------------------------------------------
        # wait here for the result to be available before continuing
        # thread.join()
        f = open(outputPath+"/parameter.txt", "w")
        f.write("r = "+str(materialFunctionParameter[dataset_number][i][0])+"\n")
        f.write("h = "+str(materialFunctionParameter[dataset_number][i][1])+"\n")
        f.write("omega_flat = "+str(materialFunctionParameter[dataset_number][i][2])+"\n")        
        f.write("omega_target = "+str(materialFunctionParameter[dataset_number][i][3])+"\n")     
        f.write("param_beta = "+str(materialFunctionParameter[dataset_number][i][5])+"\n")         
        f.close()   
        #
