import math

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()


#############################################
#  Paths
#############################################
# parameterSet.problemsPath = '/home/klaus/Desktop/harmonicmapBenchmark/dune-gfe/problems'
parameterSet.resultPath = '/home/klaus/Desktop/Dune_release/dune-gfe/outputs_rotation-test'
parameterSet.instrumentedPath = '/home/klaus/Desktop/Dune_release/dune-gfe/outputs_rotation-test/instrumented'
parameterSet.executablePath = '/home/klaus/Desktop/Dune_release/dune-gfe/build-cmake/src'
parameterSet.baseName= 'rotation-test'


#############################################
#  Grid parameters
#############################################
nX=1
nY=1

# parameterSet.structuredGrid = 'simplex'
parameterSet.structuredGrid = 'false'
parameterSet.lower = '0 0'
parameterSet.upper = '4 4'
# parameterSet.upper = '6.28318530718 6.28318530718'     # 2* pi
parameterSet.elements = str(nX)+' '+  str(nY)

parameterSet.numLevels = 1


parameterSet.gridPath = '/home/klaus/Desktop/Dune_release/dune-gfe/problems'
parameterSet.gridFile = 'Geometry2.msh'



#############################################
#  GFE parameters
#############################################

parameterSet.order = 1

# parameterSet.interpolationMethod = 'projected'
# parameterSet.interpolationMethod = 'embedded'
# parameterSet.interpolationMethod = 'geodesic'



parameterSet.conforming_DiscreteJacobian = 1

#############################################
#  Solver parameters
#############################################

# Tolerance of the multigrid solver
parameterSet.tolerance = 1e-12
# Maximum number of multigrid iterations
parameterSet.maxProximalNewtonSteps = 100
# Initial regularization
parameterSet.initialRegularization = 100
# Measure convergence
parameterSet.instrumented = 1

############################
#   Problem specifications
############################

# Dimension of the domain (only used for numerical-test python files)
parameterSet.dim = 2


# Dimension of the target space
parameterSet.targetDim = 3

parameterSet.targetSpace = 'BendingIsometry'

#############################################
#  Options
#############################################

# parameterSet.readConfiguration = 0
# parameterSet.configurationFile = /home/klaus/Desktop/DUNE/dune-harmonicmaps/problems/2_2_deg4_6.txt
# parameterSet.perturbation = 0
# parameterSet.perturbationRadius = 0.3
# parameterSet.epsilon = 0.25

# Write discrete solution as .vtk-File
parameterSet.writeVTK = 1

# Write Dof-Vector to .txt-file
parameterSet.writeDOFvector = 0

#############################################
#  Dirichlet boundary indicator
#############################################
# def dirichlet_indicator(x) :
#     if( (x[0] <= 0.001) or (x[1]<=0.001)):
#         return True
#     else:
#         return False

def dirichlet_indicator(x) :
    if( (x[0] <= 0.0001) ):
        return True
    else:
        return False

# def dirichlet_indicator(x) :
#     if( (x[0] <= 0.0001) and abs(x[1]-math.pi)<1 ):
#         return True
#     else:
#         return False



# #Test clamp on other side:
# def dirichlet_indicator(x) :
#     if( (x[0] >=3.999) or (x[1]<=0.001)):
#         return True
#     else:
#         return False


#boundary-values/derivative function
# def boundaryValues(x):
#     return [x[0], x[1], 0]


# def boundaryValuesDerivative(x):
#     return ((1,0,0),
#             (0,1,0),
#             (0,0,1))



#############################################
#  Initial iterate function
#############################################
# def f(x):
#     return [math.sin(x[0]), x[1], 1.0-math.cos(x[0])]
#
#
# def df(x):
#     return [[math.cos(x[0]),0],
#             [0,1],
#             [math.sin(x[0]),0]]

# def f(x):
#     return [x[0], x[1], 2*x[0] ]

def f(x):
    angle = -math.pi/4.0;
    return [x[0]*math.cos(angle), x[1], -x[0]*math.sin(angle)]


#rotation around y-axis with angle = pi/4
def df(x):
    angle = -math.pi/4.0;
    return [[math.cos(angle),0],
            [0,1],
            [-math.sin(angle),0]]


# def f(x):
#     return [math.sin(x[0]), x[1], math.cos(x[0])-1.0]
#
#
# def df(x):
#     return [[math.cos(x[0]),0],
#             [0,1],
#             [-math.sin(x[0]),0]]

# def f(x):
#     return [x[0], x[1], 0]
#
#
# def df(x):
#     return ((1,0),
#             (0,1),
#             (0,0))


fdf = (f, df)


#############################################
#  Force
############################################
def force(x):
    # return [0, 0, 0.025]
    return [0, 0, 0]



#############################################
#  Effective prestrain
############################################
parameterSet.prestrainFlag = True
# parameterSet.effectivePrestrain = [2.5, -1.0, 0.0]   #unfortunately this does not work (yet)
# effectivePrestrain = [2.5, -1.0, 0.0]

# effectivePrestrain = [ [2.5, -1.0] , [-1.0 ,  0.0]]
# parameterSet.effectivePrestrain = '1.0 0.5 0.0'   # b1 b2 b12
# parameterSet.effectivePrestrain = '0.0 1.0 0.0'   # b1 b2 b12
# parameterSet.effectivePrestrain = '1.0 0.0 0.0'   # b1 b2 b12
parameterSet.effectivePrestrain = '0.0 0.0 0.0'   # b1 b2 b12

#############################################
#  Discretization error parameters
#############################################
parameterSet.computeDiscError = ParameterSet()

parameterSet.computeDiscError.numSimLevels = 4
parameterSet.computeDiscError.simulationData = '/home/klaus/Desktop/harmonicmapBenchmark/dune-gfe/outputs_experiment1/experiment1_order1_projected_level3.data'
# parameterSet.computeDiscError.numRefLevels = 1  # Also grid level for computation of analytical solution
# parameterSet.computeDiscError.referenceData = '/home/klaus/Desktop/Dune_release/dune-gfe/outputs_L-clamped-Plate/L-clamped-Plate_level1_NC.data'
parameterSet.computeDiscError.numRefLevels = 4  # Also grid level for computation of analytical solution
parameterSet.computeDiscError.referenceData = '/home/klaus/Desktop/Dune_release/dune-gfe/outputs_L-clamped-Plate/L-clamped-Plate_level4_NC.data'
# parameterSet.computeDiscError.targetSpace = 'BendingIsometry'
# parameterSet.computeDiscError.discretizationErrorMode = 'discrete'
parameterSet.computeDiscError.discretizationErrorMode = 'analytical'
parameterSet.computeDiscError.discretizationErrorMode = 'discreteIntermediateSteps'
parameterSet.computeDiscError.compute_ConstraintError = 0


# Amount of steps used by Riemannain trust-region solver (= amount of intermediateSolutions)
# parameterSet.computeDiscError.solverSteps = 14
parameterSet.computeDiscError.solverSteps = 19
# parameterSet.computeDiscError.solverSteps = 1

#############################################
#  Analytical reference Solution
#############################################
# def f(x):
#     return [x[0], x[1], 0]
#
#
# def df(x):
#     return ((1,0),
#             (0,1),
#             (0,0))
#
#
# fdf = (f, df)
