import math
#from python_matrix_operations import *
import ctypes
import os
import sys
import numpy as np
import elasticity_toolbox as elast

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()
#---------------------------------------------------------------
#############################################
#  Paths
#############################################
# Path for results and logfile
parameterSet.outputPath='/home/klaus/Desktop/Dune_release/dune-microstructure/experiment/rotation-test/results'
parameterSet.baseName= 'isotrop_orthotrop_rotation'   #(needed for Output-Filename)

# Path for material description
# parameterSet.geometryFunctionPath =experiment/wood-bilayer/

#---------------------------------------------------------------
# Komposit beschreibt ein Material mit 
# oberer Schicht: orthotrop (Holz), in der Ebene gedreht
# orientierung y_1-direction: L, y_2-direction: T, x_3-direction: R
compliance_wood=np.array([[ 7.70543562e-05, -1.56584619e-05, -1.96144058e-05,
         0.00000000e+00,  0.00000000e+00,  0.00000000e+00],
       [-1.56584619e-05,  1.84913679e-03, -5.14793170e-04,
         0.00000000e+00,  0.00000000e+00,  0.00000000e+00],
       [-1.96144058e-05, -5.14793170e-04,  5.92975602e-04,
         0.00000000e+00,  0.00000000e+00,  0.00000000e+00],
       [ 0.00000000e+00,  0.00000000e+00,  0.00000000e+00,
         2.25174559e-03,  0.00000000e+00,  0.00000000e+00],
       [ 0.00000000e+00,  0.00000000e+00,  0.00000000e+00,
         0.00000000e+00,  7.95374719e-04,  0.00000000e+00],
       [ 0.00000000e+00,  0.00000000e+00,  0.00000000e+00,
         0.00000000e+00,  0.00000000e+00,  1.19183673e-03]])
prestrain_wood=np.array([[-0.050821460301886785, 0, 0],                    
 [0, -2.134501332679245, 0],
 [0, 0, -0.8824453561509432]])
# rotation angle
# param_theta = '1.5707963267948966'


# untere Schicht: isotrop, ohne prestrain

# --- define geometry

def indicatorFunction(x):
    factor=1
    if (x[2]>=0):
        return 1  #Phase1
    else :
        return 2   #Phase2

# --- Number of material phases
parameterSet.Phases=2

# --- PHASE 1
parameterSet.phase1_type="general_anisotropic"
# Drehung um theta um Achse 2 = x_3-Achse
# N=elast.rotation_matrix_compliance(2,param_theta)
# materialParameters_phase1 = np.dot(np.dot(N,compliance_wood),N.T)
# materialParameters_phase1 = (0.5*(materialParameters_phase1.T+materialParameters_phase1)).tolist()



# Drehung um theta um Achse 2 = x_3-Achse
parameterSet.phase1_axis = 2

parameterSet.phase1_angle = 0.0
# parameterSet.phase1_angle = param_theta
materialParameters_phase1 = compliance_wood


# rotation of strain
# def prestrain_phase1(x):
#     R=elast.rotation_matrix(2,param_theta)
#     return np.dot(R,np.dot(np.array(prestrain_wood),R.T)).tolist()
def prestrain_phase1(x):
    return prestrain_wood

# --- PHASE 2
parameterSet.phase2_type="isotropic"
# extract E and nu from wood
E = 1/compliance_wood[0,0]
nu = -compliance_wood[0,1]*E
# [mu, lambda]
materialParameters_phase2 = [E/(2*(1+nu)), (E*nu)/((1+nu)*(1-2*nu))]
def prestrain_phase2(x):
    return [[0,0,0],[0,0,0],[0,0,0]]

# --- Choose scale ratio gamma:
parameterSet.gamma=1.0




#############################################
#  Grid parameters
#############################################
## numLevels : Number of Levels on which solution is computed. starting with a 2x2x2 cube mesh.
## {start,finish} computes on all grid from 2^(start) to 2^finish refinement
#----------------------------------------------------
parameterSet.numLevels= '3 3'      # computes all levels from first to second entry


#############################################
#  Assembly options
#############################################
parameterSet.set_IntegralZero = 1            #(default = false)
parameterSet.set_oneBasisFunction_Zero = 1   #(default = false)
#parameterSet.arbitraryLocalIndex = 7            #(default = 0)
#parameterSet.arbitraryElementNumber = 3         #(default = 0)

#############################################
#  Solver Options, Type: #1: CG - SOLVER , #2: GMRES - SOLVER, #3: QR - SOLVER (default), #4: UMFPACK - SOLVER
#############################################
parameterSet.Solvertype = 3        # recommended to use iterative solver (e.g GMRES) for finer grid-levels
parameterSet.Solver_verbosity = 0  #(default = 2)  degree of information for solver output


#############################################
#  Write/Output options      #(default=false)
#############################################
# --- (Optional output) write Material / prestrain / Corrector functions to .vtk-Files:
parameterSet.write_materialFunctions = 1   # VTK indicator function for material/prestrain definition
#parameterSet.write_prestrainFunctions = 1  # VTK norm of B (currently not implemented)

# --- (Additional debug output)
parameterSet.print_debug = 0  #(default=false)

# --- Write Correctos to VTK-File:  
parameterSet.write_VTK = 1

# The grid can be refined several times for a higher resolution in the VTK-file.
parameterSet.subsamplingRefinement = 2

# --- (Optional output) L2Error, integral mean: 
#parameterSet.write_L2Error = 1
#parameterSet.write_IntegralMean = 1      

# --- check orthogonality (75) from paper: 
parameterSet.write_checkOrthogonality = 0

# --- Write corrector-coefficients to log-File:
#parameterSet.write_corrector_phi1 = 1
#parameterSet.write_corrector_phi2 = 1
#parameterSet.write_corrector_phi3 = 1

# --- Print Condition number of matrix (can be expensive):
#parameterSet.print_conditionNumber= 1  #(default=false)

# --- write effective quantities to Matlab-folder for symbolic minimization:
parameterSet.write_toMATLAB = 1  # writes effective quantities to .txt-files QMatrix.txt and BMatrix.txt
