import math
import numpy as np

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()



""""
    Experiment: Three-phase composite consiting of a matrx material (phase 3) 
                together with a prestrained fibre in the top layer aligned 
                with e2-direction (phase 1) and a prestrained fibre in the 
                bottom layer aligned with the e1-direction (phase 2).

    rho: ratio between the prestrain of the fibres.

"""


#############################################
#  Paths
#############################################
parameterSet.resultPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs_thee-phase-composite'
parameterSet.baseName= 'thee-phase-composite'

##################### MICROSCALE PROBLEM ####################

class Microstructure:
    def __init__(self):
        self.gamma = 1.0    #in the future this might change depending on macroPoint.
        self.phases = 3     #in the future this might change depending on macroPoint.
        
        #--- Define different material phases:
        #- PHASE 1
        self.phase1_type="isotropic"
        self.materialParameters_phase1 = [200, 1.0]   
        #- PHASE 2
        self.phase2_type="isotropic"
        self.materialParameters_phase2 = [200, 1.0]    
        #- PHASE 3
        self.phase3_type="isotropic"
        self.materialParameters_phase3 = [100, 1.0]    #(Matrix-material)
        # self.materialParameters_phase3 = [100, 0.1]    #(Matrix-material)

    #--- Three-phase composite phase indicator
    def indicatorFunction(self,x):
        # l = 1.0/4.0 # center point of fibre with quadratic cross section of area r**2
        # # l = 3.0/8.0 # center point of fibre with quadratic cross section of area r**2
        # r = 1.0/4.0
        # if (np.max([abs(x[2]-l), abs(x[1]-0.5)]) < r):
        fibreRadius = 1.0/4.0
        if (abs(x[0]) < fibreRadius   and x[2] > 0 ):
            return 1    #Phase1   
        elif (abs(x[1]) < fibreRadius and x[2] < 0 ):
            return 2    #Phase2
        else :
            return 3    #Phase3
        
    #---Two-phase composite:
    # def indicatorFunction(self,x):
    #     if (abs(x[0]) < (1.0/2.0)   and x[2] >= 0 ):
    #         return 1    #Phase1   
    #     elif (abs(x[1]) < (1.0/2.0) and x[2] < 0 ):
    #         return 2    #Phase2
    #     else :
    #         return 3    #Phase3
        


    # #TEST: smaller fibres
    # def indicatorFunction(self,x):
    #     # l = 1.0/4.0 # center point of fibre with quadratic cross section of area r**2
    #     # # l = 3.0/8.0 # center point of fibre with quadratic cross section of area r**2
    #     # r = 1.0/4.0
    #     # if (np.max([abs(x[2]-l), abs(x[1]-0.5)]) < r):
    #     if (abs(x[0]) < (1.0/8.0)   and x[2] > (1.0/4.0) ):
    #         return 1    #Phase1   
    #     elif (abs(x[1]) < (1.0/8.0) and x[2] < -(1.0/4.0) ):
    #         return 2    #Phase2
    #     else :
    #         return 3    #Phase3
        
    # prestrained fibre in top layer , e2-aligned
    def prestrain_phase1(self,x):
        return [[1.0, 0, 0], [0,1.0,0], [0,0,1.0]]

    # prestrained fibre in bottom layer , e1-aligned
    def prestrain_phase2(self,x):
        #prestrain ratio
        rho = 1.0
        return [[rho*1.0, 0, 0], [0,rho*1.0,0], [0,0,rho*1.0]]

    # no prestrain in matrix-material
    def prestrain_phase3(self,x):
        return [[0, 0, 0], [0,0,0], [0,0,0]]

#############################################
#  Grid parameters
#############################################
parameterSet.microGridLevel = 3

#############################################
#  Assembly options
#############################################
parameterSet.set_IntegralZero = 1            #(default = false)
parameterSet.set_oneBasisFunction_Zero = 1   #(default = false)
#parameterSet.arbitraryLocalIndex = 7            #(default = 0)
#parameterSet.arbitraryElementNumber = 3         #(default = 0)

#############################################
#  Solver Options, Type: #1: CG - SOLVER , #2: GMRES - SOLVER, #3: QR - SOLVER, #4: UMFPACK - SOLVER (default)
#############################################
parameterSet.Solvertype = 4        # recommended to use iterative solver (e.g GMRES) for finer grid-levels
parameterSet.Solver_verbosity = 0  #(default = 2)  degree of information for solver output


#############################################
#  Write/Output options      #(default=false)
#############################################
# --- (Optional output) write Material / prestrain / Corrector functions to .vtk-Files:
parameterSet.write_materialFunctions = 1   # VTK indicator function for material/prestrain definition
#parameterSet.write_prestrainFunctions = 1  # VTK norm of B (currently not implemented)
parameterSet.MaterialSubsamplingRefinement= 2

# --- (Additional debug output)
parameterSet.print_debug = 0  #(default=false)

parameterSet.print_corrector_matrices = 0

# --- Write Correctos to VTK-File:  
parameterSet.writeCorrectorsVTK = 1

# --- Use caching of element matrices:  
parameterSet.cacheElementMatrices = 1

# --- check orthogonality (75) from paper: 
parameterSet.write_checkOrthogonality = 0

# --- write effective quantities (Qhom.Beff) to .txt-files
parameterSet.write_EffectiveQuantitiesToTxt = True
