#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jul  6 13:17:28 2022

@author: stefan
"""
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from matplotlib.ticker import LogLocator
import codecs
import re
import json

def energy(kappa,alpha,Q,B)  :
    G=kappa*np.array([[np.cos(alpha)**2],[np.sin(alpha)**2],[np.sqrt(2)*np.cos(alpha)*np.sin(alpha)]])-B
    return np.matmul(np.transpose(G),np.matmul(Q,G))[0,0]

def xytokappaalpha(x,y):
   
    if y>0:
        return [np.sqrt(x**2+y**2), np.abs(np.arctan2(y,x))]
    else:
        return [-np.sqrt(x**2+y**2), np.abs(np.arctan2(y,x))]

# Read effective quantites
def ReadEffectiveQuantities(QFilePath, BFilePath):
    # Read Output Matrices (effective quantities)
    # From Cell-Problem output Files : ../outputs/Qmatrix.txt , ../outputs/Bmatrix.txt
    # -- Read Matrix Qhom
    X = []
    # with codecs.open(path + '/outputs/QMatrix.txt', encoding='utf-8-sig') as f:
    with codecs.open(QFilePath, encoding='utf-8-sig') as f:
        for line in f:
            s = line.split()
            X.append([float(s[i]) for i in range(len(s))])
    Q = np.array([[X[0][2], X[1][2], X[2][2]],
                  [X[3][2], X[4][2], X[5][2]],
                  [X[6][2], X[7][2], X[8][2]] ])

    # -- Read Beff (as Vector)
    X = []
    # with codecs.open(path + '/outputs/BMatrix.txt', encoding='utf-8-sig') as f:
    with codecs.open(BFilePath, encoding='utf-8-sig') as f:
        for line in f:
            s = line.split()
            X.append([float(s[i]) for i in range(len(s))])
    B = np.array([X[0][2], X[1][2], X[2][2]])
    return Q, B

# Number of experiments / folders
show_plot = True

#--- Select specific experiment [x, y] with date from results_x/y
data=[5,6]
#DataPath = './experiment/wood-bilayer_PLOS/results_'  + str(data[0]) + '/' +str(data[1])
DataPath = './results_'  + str(data[0]) + '/' +str(data[1])
QFilePath = DataPath + '/QMatrix.txt'
BFilePath = DataPath + '/BMatrix.txt'
ParameterPath = DataPath + '/parameter.txt'
#
# Read Thickness from parameter file (needed for energy scaling)
with open(ParameterPath , 'r') as file:
    parameterFile  = file.read()
thickness = float(re.findall(r'(?m)h = (\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',parameterFile)[0])
energyscalingfactor = thickness**2
# Read Q and B
Q, B = ReadEffectiveQuantities(QFilePath,BFilePath)
Q=0.5*(np.transpose(Q)+Q) # symmetrize
B=np.transpose([B])
# 
# Compute lokal and global minimizer
kappa=0
kappa_pos=0
kappa_neg=0
#
N=500
length=4
r, theta = np.meshgrid(np.linspace(0,length,N),np.radians(np.linspace(0, 360, N)))
E=np.zeros(np.shape(r))
for i in range(0,N): 
    for j in range(0,N):     
        if theta[i,j]<np.pi:
            E[i,j]=energy(r[i,j],theta[i,j],Q,B)  * energyscalingfactor
        else:
            E[i,j]=energy(-r[i,j],theta[i,j],Q,B) * energyscalingfactor
#        
# Compute Minimizer
[imin,jmin]=np.unravel_index(E.argmin(),(N,N))
kappamin=r[imin,jmin]
alphamin=theta[imin,jmin]
# Positiv curvature region
N_mid=int(N/2)
[imin,jmin]=np.unravel_index(E[:N_mid,:].argmin(),(N_mid,N))
kappamin_pos=r[imin,jmin]
alphamin_pos=theta[imin,jmin]
Emin_pos=E[imin,jmin]
# Negative curvature region
[imin,jmin]=np.unravel_index(E[N_mid:,:].argmin(),(N_mid,N))
kappamin_neg=r[imin+N_mid,jmin]
alphamin_neg=theta[imin+N_mid,jmin]
Emin_neg=E[imin+N_mid,jmin]
#
E=E/E.min()
print(Emin_pos/Emin_neg)
fig, ax = plt.subplots(figsize=(6,6),subplot_kw=dict(projection='polar'))
levs=np.geomspace(1,E.max(),1000)
pcm=ax.contourf(theta, r, E, levs, norm=colors.PowerNorm(gamma=0.4), cmap='brg')
ax.set_xticks(np.array([.0,1/4,2/4,3/4,1,5/4,6/4,7/4])*np.pi)
anglelabel=["0°","45°", "90°", "135°","180°","135°","90°","45°"]
ax.set_xticklabels(anglelabel)
ax.set_yticks([1,2,3,4])
#ax.set_yticklabels(["1$m^{-1}$","2$m^{-1}$","3$m^{-1}$","4$m^{-1}$"])
#
ax.plot([alphamin_pos,alphamin_pos+np.pi], [kappamin_pos,kappamin_pos],
            markerfacecolor='red',
            markeredgecolor='black',            # marker edgecolor
            marker='s',                         # each marker will be rendered as a circle
            markersize=5,                       # marker size
            markeredgewidth=0.5,                  # marker edge width
            linewidth=0,
            zorder=3,
            alpha=1,                           # Change opacity
            label = r"$\kappa_{2,sim}(m^{-1})$")        
ax.plot(alphamin_neg, kappamin_neg,
            markerfacecolor='blue',
            markeredgecolor='black',            # marker edgecolor
            marker='D',                         # each marker will be rendered as a circle
            markersize=5,                       # marker size
            markeredgewidth=0.5,                  # marker edge width
            linewidth=0,                      # line width
            zorder=3,
            alpha=1,                           # Change opacity
            label = r"$\kappa_{1,sim}(m^{-1})$")
colorbarticks=np.linspace(1,15,10)
cbar = plt.colorbar(pcm, extend='max', ticks=colorbarticks, pad=0.1)
#bounds = ['0','1/80','1/20','1/5','4/5']
cbar.ax.tick_params(labelsize=8)
#cbar.set_ticklabels(bounds)
fig.legend(loc="upper left")
if (show_plot):
    plt.show()
# Save Figure as .pdf
width = 5.79 
height = width / 1.618 # The golden ratio.
fig.set_size_inches(width, height)
#fig.savefig('./experiment/wood-bilayer_PLOS/wood-bilayer_PLOS_dataset_' +str(data[0]) + '_exp' + str(data[1]) + '.pdf', dpi=300)