#include <config.h>
#include <signal.h>
#include <memory>

#include <fenv.h>
#include <array>

#include <math.h>

// Includes for the ADOL-C automatic differentiation library
// Need to come before (almost) all others.
#include <adolc/adouble.h>
#include <dune/fufem/utilities/adolcnamespaceinjections.hh>

#include <dune/common/typetraits.hh>

#include <dune/common/bitsetvector.hh>
#include <dune/common/parametertree.hh>
#include <dune/common/parametertreeparser.hh>

#include <dune/grid/uggrid.hh>
#include <dune/grid/utility/structuredgridfactory.hh>

#include <dune/grid/io/file/gmshreader.hh>
#include <dune/grid/io/file/vtk.hh>

#include <dune/functions/gridfunctions/discreteglobalbasisfunction.hh>
#include <dune/functions/gridfunctions/composedgridfunction.hh>
#include <dune/functions/functionspacebases/lagrangebasis.hh>
#include <dune/functions/functionspacebases/powerbasis.hh>
#include <dune/functions/functionspacebases/interpolate.hh>
#include <dune/functions/functionspacebases/reducedcubichermitetrianglebasis.hh>

#include <dune/fufem/boundarypatch.hh>
#include <dune/fufem/functiontools/boundarydofs.hh>
#include <dune/fufem/dunepython.hh>

#include <dune/fufem/discretizationerror.hh>

#include <dune/solvers/solvers/iterativesolver.hh>
#include <dune/solvers/norms/energynorm.hh>

#include <dune/gfe/spaces/productmanifold.hh>
#include <dune/gfe/spaces/realtuple.hh>
#include <dune/gfe/spaces/rotation.hh>
#include <dune/gfe/localdiscretekirchhoffbendingisometry.hh>
#include <dune/gfe/assemblers/localgeodesicfeadolcstiffness.hh>
#include <dune/gfe/assemblers/harmonicenergy.hh>
#include <dune/gfe/assemblers/geodesicfeassembler.hh>
#include <dune/gfe/bendingisometryhelper.hh>
#include <dune/gfe/riemannianpnsolver.hh>
#include <dune/gfe/embeddedglobalgfefunction.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergy.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergyconforming.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergyzienkiewicz.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergyzienkiewiczprojected.hh>

#include <dune/microstructure/energies/discretekirchhoffbendingenergyprestrained.hh>
#include <dune/microstructure/microproblem.hh>

#include <dune/gfe/spaces/stiefelmanifold.hh>
// #include <dune/gfe/spaces/stiefelmatrix.hh>

#include <dune/gmsh4/gmsh4reader.hh>
#include <dune/gmsh4/gridcreators/lagrangegridcreator.hh>

//TEST 
#include <dune/gfe/riemanniantrsolver.hh>


//TEST dune-vtk
// #include <dune/vtk/function.hh>
#include <dune/vtk/vtkwriter.hh>
// #include <dune/vtk/vtkwriterbase.hh>
#include <dune/vtk/writers/unstructuredgridwriter.hh>
#include <dune/vtk/datacollectors/continuousdatacollector.hh>
#include <dune/vtk/datacollectors/discontinuousdatacollector.hh>
#include <dune/vtk/datacollectors/quadraticdatacollector.hh>
#include <dune/vtk/datacollectors/lagrangedatacollector.hh>


const int dim = 2;

using namespace Dune;


  // template <class T>
  // constexpr std::string_view type_name()
  // {
  //     using namespace std;
  // #ifdef __clang__
  //     string_view p = __PRETTY_FUNCTION__;
  //     return string_view(p.data() + 34, p.size() - 34 - 1);
  // #elif defined(__GNUC__)
  //     string_view p = __PRETTY_FUNCTION__;
  // #  if __cplusplus < 201402
  //     return string_view(p.data() + 36, p.size() - 36 - 1);
  // #  else
  //     return string_view(p.data() + 49, p.find(';', 49) - 49);
  // #  endif
  // #elif defined(_MSC_VER)
  //     string_view p = __FUNCSIG__;
  //     return string_view(p.data() + 84, p.size() - 84 - 7);
  // #endif
  // }

// struct Difference2
// {

//   double operator()(Dune::FieldMatrix<double, 3, 2> x, Dune::FieldMatrix<double, 2, 2> I) const
//   {
//     return ((x.transposed()*x)-I).frobenius_norm();
//   }
// };


// // Wrapper for global-coordinate functions F
// template <class GridView, class F>
// class GlobalKirchhoffFunction
// {
//   using Element = typename GridView::template Codim<0>::Entity;
//   using Geometry = typename Element::Geometry;

// public:
//   GlobalKirchhoffFunction (GridView const& gridView, F const& f)
//     : gridView_(gridView)
//     , f_(f)
//   {}

//   void bind(Element const& element) { 
//     geometry_.emplace(element.geometry());
//     f_.bind(element);  // we need to bind kirchhoff function to the element
//   }
//   void unbind() { geometry_.reset(); }

//   auto operator() (typename Geometry::LocalCoordinate const& local) const
//   {
//     assert(!!geometry_);
//     return f_(geometry_->global(local));
//   }

// private:
//   GridView gridView_;
//   F f_;
//   std::optional<Geometry> geometry_;
// };

// // Wrapper for global-coordinate functions F
// template <class GridView, class F>
// class GlobalFunction
// {
//   using Element = typename GridView::template Codim<0>::Entity;
//   using Geometry = typename Element::Geometry;

// public:
//   GlobalFunction (GridView const& gridView, F const& f)
//     : gridView_(gridView)
//     , f_(f)
//   {}

//   void bind(Element const& element) { geometry_.emplace(element.geometry()); }
//   void unbind() { geometry_.reset(); }

//   auto operator() (typename Geometry::LocalCoordinate const& local) const
//   {
//     assert(!!geometry_);
//     return f_(geometry_->global(local));
//   }

// private:
//   GridView gridView_;
//   F f_;
//   std::optional<Geometry> geometry_;
// };



int main(int argc, char *argv[])
{

  /**
   * @brief We use this to catch a 'Floating point exception' caused by the 'innerSolver' 
   *        in RiemannianProximalNewton
   */
  //   std::shared_ptr<void(int)> handler(
  //       signal(SIGFPE, [](int signum) {throw std::logic_error("FPE"); }),
  //       [](__sighandler_t f) { signal(SIGFPE, f); });



  // feenableexcept(FE_INVALID);

  MPIHelper::instance(argc, argv);
  Dune::Timer globalTimer;

  // if (argc < 3)
  //   DUNE_THROW(Exception, "Usage: ./bending-isometries <python path> <python module without extension>");

  // Start Python interpreter
  Python::start();
  auto pyMain = Python::main();
  pyMain.runStream()
      << std::endl << "import math"
      << std::endl << "import sys"
      << std::endl << "sys.path.append('" << argv[1] << "')"  << std::endl;
  auto pyModule = pyMain.import(argv[2]);


  std::cout << "Current path is " << std::filesystem::current_path() << '\n';
  std::filesystem::path file_path = (__FILE__);
  std::cout<< "File path: " << file_path<<std::endl;
  std::cout << "dir_path: " << file_path.parent_path() << std::endl;
  std::string dir_path  = file_path.parent_path();

  // Start Python interpreter
  // Python::start();
  // auto pyMain = Python::main();
  // pyMain.runStream()
  //     << std::endl << "import math"
  //     << std::endl << "import sys"
  //     << std::endl << "sys.path.append('" << dir_path << "')"  << std::endl;
  // auto pyModule = pyMain.import("properties-shape-test-2");

  // parse data file
  ParameterTree parameterSet;
  pyModule.get("parameterSet").toC(parameterSet);

  // read possible further parameters from the command line
  ParameterTreeParser::readOptions(argc, argv, parameterSet);

  // Print all parameters, to make them appear in the log file
  std::cout << "Executable: bending-isometries, with parameters:" << std::endl;
  parameterSet.report();

  /////////////////////////////////////////
  //   Create the grid
  /////////////////////////////////////////
  using GridType = UGGrid<dim>;

  std::shared_ptr<GridType> grid;
  FieldVector<double,dim> lower(0), upper(1);
  std::array<unsigned int,dim> elementsArray;

  std::string structuredGridType = parameterSet["structuredGrid"];
  if (structuredGridType != "false" )
  {
    lower = parameterSet.get<FieldVector<double,dim> >("lower");
    upper = parameterSet.get<FieldVector<double,dim> >("upper");
    elementsArray = parameterSet.get<std::array<unsigned int,dim> >("elements");

    if (structuredGridType == "simplex")
      grid = StructuredGridFactory<GridType>::createSimplexGrid(lower, upper, elementsArray);
    else if (structuredGridType == "cube")
      grid = StructuredGridFactory<GridType>::createCubeGrid(lower, upper, elementsArray);
    else
      DUNE_THROW(Exception, "Unknown structured grid type '" << structuredGridType << "' found!");
  } else {
    std::cout << "Read GMSH grid." << std::endl;
    std::string gridPath = parameterSet.get<std::string>("gridPath");
    std::string gridFile = parameterSet.get<std::string>("gridFile");
    GridFactory<GridType> factory;
    Gmsh4::LagrangeGridCreator creator{factory};
    Gmsh4Reader reader{creator};
    reader.read(gridPath + "/" + gridFile);
    grid = factory.createGrid();
  }

  const int macroGridLevel = parameterSet.get<int>("macroGridLevel");
  grid->globalRefine(macroGridLevel-1);

  using GridView = typename GridType::LeafGridView;
  GridView gridView = grid->leafGridView();

  ///////////////////////////////////////////////////////
  //  Set up the function spaces
  ///////////////////////////////////////////////////////

  // General coefficient vector of a Discrete Kirchhoff deformation function
  using VectorSpaceCoefficients = BlockVector<FieldVector<double,3>>;

  /**
   * @brief Coefficient vector of a Discrete Kirchhoff deformation function that is constrained to be an isometry.
   *        we need both 'double' and 'adouble' versions.
   */
  using Coefficient = GFE::ProductManifold<RealTuple<double,3>, Rotation<double,3> >;
  using IsometryCoefficients = std::vector<Coefficient>;
  using ACoefficient = typename Coefficient::template rebind<adouble>::other;
  using AIsometryCoefficients = std::vector<ACoefficient>;

  using namespace Functions::BasisFactory;
  auto deformationBasis = makeBasis(gridView,
                                    power<3>(reducedCubicHermiteTriangle(),
                                    blockedInterleaved()));

  using DeformationBasis = decltype(deformationBasis);

  /**
   * @brief The next basis is used to assign (nonlinear) degrees of freedom to the grid vertices.
   *        The actual basis function values are never used.
   */
  using CoefficientBasis = Functions::LagrangeBasis<GridView, 1>;
  CoefficientBasis coefficientBasis(gridView);

  // A basis for the tangent space (used to set DirichletNodes)
  auto tangentBasis = makeBasis(gridView,
                                power<Coefficient::TangentVector::dimension>(
                                lagrange<1>(),
                                blockedInterleaved()));


  //TEST
  // auto tangentBasis = makeBasis(gridView,
  //                               power<Coefficient::TangentVector::dimension>(  //dim = 6
  //                               lagrange<1>(),
  //                               flatInterleaved()));


  std::cout << "Coefficient::TangentVector::dimension: " << Coefficient::TangentVector::dimension<< std::endl;

  // Print some information on the grid and degrees of freedom.                        
  std::cout << "Number of Elements in the grid: " << gridView.size(0)<< std::endl;
  std::cout << "Number of Nodes in the grid: "    << gridView.size(dim)<< std::endl;
  std::cout << "deformationBasis.size(): "        << deformationBasis.size() << std::endl;
  std::cout << "deformationBasis.dimension(): "   << deformationBasis.dimension() << std::endl;
  std::cout << "Degrees of Freedom: "             << deformationBasis.dimension() << std::endl;

  bool PRINT_DEBUG = parameterSet.get<bool>("print_debug", 0);

  ///////////////////////////////////////////
  //   Read Dirichlet values 
  ///////////////////////////////////////////
  BitSetVector<1> dirichletVertices(gridView.size(dim), false);
  const typename GridView::IndexSet &indexSet = gridView.indexSet();


  BitSetVector<Coefficient::TangentVector::dimension> dirichletNodes(tangentBasis.size(), false); //tangentBasis.size()=coefficientBasis.size()

  /**
   * @brief Make Python function that computes which vertices are on the Dirichlet boundary,
   *        based on the vertex positions.
   */
  auto dirichletIndicatorFunction = Python::make_function<bool>(pyModule.get("dirichlet_indicator"));



  /**
   * @brief If we want to clamp DOFs inside the domain, we connot use 'BoundaryPatch' 
   *        and 'constructBoundaryDofs'. This is a workaround for now.
   *        
   *
   */
  for (auto &&vertex : vertices(gridView))
  {
    dirichletVertices[indexSet.index(vertex)] = dirichletIndicatorFunction(vertex.geometry().corner(0));

      if(dirichletIndicatorFunction(vertex.geometry().corner(0)))
      { 
        dirichletNodes[indexSet.index(vertex)] = true;
        if(PRINT_DEBUG)
          std::cout << "Dirichlet Vertex with coordinates:" << vertex.geometry().corner(0) << std::endl;
      }
  
  }
    
  // std::cout << "tangentBasis.size():" << tangentBasis.size() << std::endl;
  // std::cout << "coefficientBasis.size():" << coefficientBasis.size() << std::endl;

  //deprecated: 
  // BoundaryPatch<GridView> dirichletBoundary(gridView, dirichletVertices);
  // constructBoundaryDofs(dirichletBoundary, tangentBasis, dirichletNodes);






  // --------------------------------------




  // markBoundaryPatchDofs(dirichletBoundary, tangentBasis, dirichletNodes);

  //TEST with blocksize?
  // constructBoundaryDofs<GridView,decltype(tangentBasis),Coefficient::TangentVector::dimension>(dirichletBoundary, tangentBasis, dirichletNodes);
  // constructBoundaryDofs<GridView,decltype(tangentBasis),Coefficient::TangentVector::dimension>(dirichletBoundary, tangentBasis, dirichletNodes);



    //TEST: print dirichletNodes 
  // std::cout << "print dirichletVertices:" << std::endl;
  // std::cout <<  dirichletVertices << std::endl;

  // std::cout << "print dirichletNodes:" << std::endl;
  // std::cout <<  dirichletNodes << std::endl;


  ///////////////////////////////////////////
  //   Get initial Iterate
  ///////////////////////////////////////////
  auto pythonInitialIterate = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("f"), pyModule.get("df"));

  VectorSpaceCoefficients x(deformationBasis.size());
  interpolate(deformationBasis, x, pythonInitialIterate); 



   /**
   * @brief TEST: Add boundary-value function
   * 
   */
  // auto boundaryValueFunction = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,3>)>(pyModule.get("boundaryValues"), pyModule.get("boundaryValuesDerivative"));
  // auto boundaryValueFunction = Python::make_function<FieldVector<double,3>>(pyModule.get("boundaryValues"));
  // using namespace Dune::Indices;
  // for (auto &&vertex : vertices(gridView))
  // {
  //   if(dirichletIndicatorFunction(vertex.geometry().corner(0)))
  //   {
  //   isometryCoefficients[indexSet.index(vertex)][_0] = (RealTuple<double, 3>)boundaryValueFunction(vertex.geometry().corner(0));
  //   // isometryCoefficients[indexSet.index(vertex)][_1].set(derivative(boundaryValueFunction)(vertex.geometry().corner(0)));
  //   isometryCoefficients_adouble[indexSet.index(vertex)][_0] = (RealTuple<adouble, 3>)boundaryValueFunction(vertex.geometry().corner(0));
  //   // isometryCoefficients_adouble[indexSet.index(vertex)][_1].set(derivative(boundaryValueFunction)(vertex.geometry().corner(0)));
  //   }
  // }
    

  /**
   * @brief We need to setup LocalDiscreteKirchhoffBendingIsometry with a coefficient 
   * vector of ctype 'adouble' while the solver gets a coefficient vector 
   * of ctype 'double'.
   */
  IsometryCoefficients isometryCoefficients(coefficientBasis.size());
  AIsometryCoefficients isometryCoefficients_adouble(coefficientBasis.size());


  /**
   * @brief Copy the current iterate into a data type that encapsulates the isometry constraint
   *        i. e. convert coefficient data structure from 'VectorSpaceCoefficients' to 'IsometryCoefficients'
   */
  vectorToIsometryCoefficientMap(deformationBasis,coefficientBasis,x,isometryCoefficients);
  vectorToIsometryCoefficientMap(deformationBasis,coefficientBasis,x,isometryCoefficients_adouble);

  /**
   * @brief TEST: conversion of coefficient vectors (back & forth)
   */
  coefficientConversionTest<VectorSpaceCoefficients, DeformationBasis, CoefficientBasis, IsometryCoefficients>(x, deformationBasis, coefficientBasis);


    

  using LocalDKFunction = GFE::LocalDiscreteKirchhoffBendingIsometry<DeformationBasis, CoefficientBasis, AIsometryCoefficients>;
  LocalDKFunction localDKFunction(deformationBasis, coefficientBasis, isometryCoefficients_adouble);


  /**
   * @brief TEST: check isometry condition on the nodes of the grid for initial deformation.
   */
    // auto initialDeformationFunction = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, x);
    // nodewiseIsometryTest(initialDeformationFunction);
  nodewiseIsometryTest(localDKFunction, gridView);


  /**
   * @brief Read force term.
   */
  auto pythonForce = Python::make_function<FieldVector<double,3>>(pyModule.get("force"));
  auto forceGVF  = Dune::Functions::makeGridViewFunction(pythonForce, gridView);
  auto localForce = localFunction(forceGVF);

  // /**
  //  * @brief Read effective prestrain Tensor
  //  */
  // Dune::FieldVector<adouble,3> Beffvec = parameterSet.get<Dune::FieldVector<adouble,3>>("effectivePrestrain", {0.0, 0.0, 0.0});
  // Dune::FieldMatrix<adouble,2,2> effectivePrestrain = {{(adouble)Beffvec[0], (adouble)Beffvec[2]}, {(adouble)Beffvec[2],(adouble)Beffvec[1]} };
  // printmatrix(std::cout, effectivePrestrain, "effective prestrain (Beff): ", "--");


  // /**
  //  * @brief Get effective quadratic form Qhom
  //  * 
  //  *  input-vector: [q_1,q_2,q_3,q_12,q_13,q_23]
  //  *  is assembled into a matrix where the off-diagonal entries are divided by 2 (compare with definition in the paper)
  //  *            (   q_1    , 0.5*q_12  ,  0.5*q_13 )
  //  *     Q =    ( 0.5*q_12 ,   q_2     ,  0.5*q_23 )
  //  *            ( 0.5*q_13 , 0.5*q_23  ,    q_3    )
  //  */
  // Dune::FieldVector<adouble,6> Qhomvec = parameterSet.get<Dune::FieldVector<adouble,6>>("effectiveQuadraticForm", {1.0, 1.0, 1.0, 0.0, 0.0, 0.0});
  // Dune::FieldMatrix<adouble,3,3> Qhom = {{(adouble)Qhomvec[0], (adouble)0.5*Qhomvec[3], (adouble)0.5*Qhomvec[4]}, 
  //                                       {(adouble)0.5*Qhomvec[3], (adouble)Qhomvec[1], (adouble)0.5*Qhomvec[5]},
  //                                       {(adouble)0.5*Qhomvec[4], (adouble)0.5*Qhomvec[5], (adouble)Qhomvec[2]}};
  // printmatrix(std::cout, Qhom, "effective quadratic form (Qhom): ", "--");

  ////////////////////////////////////////////////////////////////////////
  //   Create an assembler for the Discrete Kirchhoff Energy Functional  (using ADOL-C)
  ////////////////////////////////////////////////////////////////////////

  /**
   * @brief Setup nonconforming energy
   */
  // auto localEnergy_nonconforming = std::make_shared<GFE::DiscreteKirchhoffBendingEnergy<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce);
  /**
   * @brief Setup conforming energy (WIP)
   */
  // auto localEnergy_conforming = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyZienkiewicz<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce);
  // auto localEnergy_conforming = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyZienkiewiczProjected<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce);
  /**
   * @brief Setup energy featuring prestrain
   */
  //  auto localEnergy_prestrain = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyPrestrained<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce, Qhom, effectivePrestrain,  parameterSet);
  auto localEnergy_prestrain = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyPrestrained<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce, parameterSet, pyModule);

  // LocalGeodesicFEADOLCStiffness<CoefficientBasis, Coefficient> localGFEADOLCStiffness_conforming(localEnergy_conforming.get());
  // LocalGeodesicFEADOLCStiffness<CoefficientBasis, Coefficient> localGFEADOLCStiffness_nonconforming(localEnergy_nonconforming.get());
  LocalGeodesicFEADOLCStiffness<CoefficientBasis, Coefficient> localGFEADOLCStiffness_prestrain(localEnergy_prestrain.get());
  // GeodesicFEAssembler<CoefficientBasis, Coefficient> assembler_conforming(coefficientBasis, localGFEADOLCStiffness_conforming);
  // GeodesicFEAssembler<CoefficientBasis, Coefficient> assembler_nonconforming(coefficientBasis, localGFEADOLCStiffness_nonconforming);
  GeodesicFEAssembler<CoefficientBasis, Coefficient> assembler_prestrain(coefficientBasis, localGFEADOLCStiffness_prestrain);







  /**
   * @brief Create a solver: 
   *        - Riemannian Newton with Hessian modification 
   *        - Riemannian Trust-region 
   */
  RiemannianProximalNewtonSolver<CoefficientBasis, Coefficient> RNHMsolver;
  RiemannianTrustRegionSolver<CoefficientBasis, Coefficient> RTRsolver;


  std::string Solver_name = parameterSet.get<std::string>("Solver", "RNHM");
  double numerical_energy; //final discrete energy

  if(Solver_name == "RNHM")
  {

    RNHMsolver.setup(*grid,
          &assembler_prestrain,
          isometryCoefficients,
          dirichletNodes,
          parameterSet);

    RNHMsolver.solve();
    isometryCoefficients = RNHMsolver.getSol();
    numerical_energy = RNHMsolver.getStatistics().finalEnergy;
  } else if (Solver_name =="RiemannianTR")
  {
    std::cout << "Using Riemannian Trust-region method for energy minimization." << std::endl;
    RTRsolver.setup(*grid,
            &assembler_prestrain,
            isometryCoefficients,
            dirichletNodes,
            parameterSet);
    RTRsolver.solve();
    isometryCoefficients = RTRsolver.getSol();
    numerical_energy = RTRsolver.getStatistics().finalEnergy;
  } else 
      DUNE_THROW(Dune::Exception, "Unknown Solver type for bending isometries.");



  // TEST
  // RiemannianTrustRegionSolver<CoefficientBasis, Coefficient> solver;
  // solver.setup(*grid,
  //               &assembler_prestrain,
  //               isometryCoefficients,
  //               dirichletNodes,
  //               parameterSet);
  // RNHMsolver.setup(*grid,
  //           &assembler_prestrain,
  //           isometryCoefficients,
  //           dirichletNodes,
  //           parameterSet);



//deprecated:
  // if (parameterSet.get<bool>("prestrainFlag", 0))  
  //     solver.setup(*grid,
  //               &assembler_prestrain,
  //               isometryCoefficients,
  //               dirichletNodes,
  //               parameterSet);
  // else if (parameterSet.get<bool>("conforming_DiscreteJacobian", 1))
  //     solver.setup(*grid,
  //                 &assembler_conforming,
  //                 isometryCoefficients,
  //                 dirichletNodes,
  //                 parameterSet);
  // else 
  //     solver.setup(*grid,
  //               &assembler_nonconforming,
  //               isometryCoefficients,
  //               dirichletNodes,
  //               parameterSet);

  ///////////////////////////////////////////////////////
  //  Solve!
  ///////////////////////////////////////////////////////
  // solver.setInitialIterate(isometryCoefficients);



  // Convert coefficient data structure from 'IsometryCoefficients' to 'VectorSpaceCoefficients'
  VectorSpaceCoefficients x_out(deformationBasis.size());
  isometryToVectorCoefficientMap(deformationBasis,coefficientBasis,x_out,isometryCoefficients);

  ////////////////////////////////
  //   Output result
  ////////////////////////////////
  std::string baseNameDefault = "bending-isometries-";
  std::string baseName = parameterSet.get("baseName", baseNameDefault); 
  std::string resultFileName =  parameterSet.get("resultPath", "")
                                + "/" + baseName
                                + "_level" + std::to_string(parameterSet.get<int>("macroGridLevel"));

  if (parameterSet.get<bool>("conforming_DiscreteJacobian", 1))
    resultFileName = resultFileName + "_C";
  else 
    resultFileName = resultFileName + "_NC";


  // Create a deformation function but this time with double-types.
  using LocalDKFunctionD = GFE::LocalDiscreteKirchhoffBendingIsometry<DeformationBasis, CoefficientBasis, IsometryCoefficients>;
  LocalDKFunctionD localDKFunctionDouble(deformationBasis, coefficientBasis, isometryCoefficients);




  if (parameterSet.get<bool>("writeVTK", 1))
  {
    std::cout << "write VTK..." << std::endl;
    std::cout << "to Filename: " << resultFileName << std::endl;

    /**
     * @brief Compute the displacement from the deformation.
     *        interpolate the identity and substract from the coefficient vector.
     */
    VectorSpaceCoefficients identity(deformationBasis.size());
    IdentityGridEmbedding<double> identityGridEmbedding;
    interpolate(deformationBasis, identity, identityGridEmbedding);

    // auto identity_tmp = identity;

    // Compute the displacement
    auto displacement = x_out;
    displacement -= identity;

    // std::cout << "displacement.size():" << displacement.size() << std::endl;
    auto deformationFunction = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, x_out);
    auto displacementFunction = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, displacement);

    /**
     * @brief  We need to subsample, because VTK cannot natively display real third-order functions
     */
    int subsamplingRefinement = parameterSet.get<int>("subsamplingRefinement", 2);
    SubsamplingVTKWriter<GridView> vtkWriter(gridView, Dune::refinementLevels(subsamplingRefinement));
    // SubsamplingVTKWriter<GridView> vtkWriter(gridView, Dune::refinementLevels(2));


    // Use VTK writer from dune-vtk
    // Setup a DataCollector of order 4.
    Dune::Vtk::LagrangeDataCollector<GridView, 4> lagrangeDataCollector(gridView);
    Dune::Vtk::DiscontinuousDataCollector<GridView> discontinuousDataCollector(gridView);
    Dune::Vtk::UnstructuredGridWriter duneVTKwriter(lagrangeDataCollector, Dune::Vtk::FormatTypes::ASCII, Vtk::DataTypes::FLOAT32);
    // Dune::Vtk::UnstructuredGridWriter writer(discontinuousDataCollector, Dune::Vtk::FormatTypes::ASCII, Vtk::DataTypes::FLOAT32);


    /**
     * @brief Basis used to represent normal vector fields (deprecated)
     * 
     */
    // auto normalBasis = makeBasis(gridView,
    //                   power<3>(
    //                   lagrange<1>(),
    //                   flatLexicographic()));


    // Write discrete displacement
    duneVTKwriter.addPointData(displacementFunction, Dune::Vtk::FieldInfo{"Displacement dune-VTK",3, Vtk::RangeTypes::VECTOR});
    vtkWriter.addVertexData(displacementFunction, VTK::FieldInfo("Displacement", VTK::FieldInfo::Type::vector, 3));



    // ComposedGridFunction - approach:
    auto deformationGradientFunction = derivative(deformationFunction);
    auto deformationGradientLocalFunction = localFunction(deformationGradientFunction);



    FieldMatrix<double,2,2> I = ScaledIdentityMatrix<double,2>(1);

    auto isometryErrorFunction =[&](auto M)-> double {
        return ((M.transposed()*M) - I).frobenius_norm();
    };

    auto isometryErrorGridFunction = makeComposedGridFunction(isometryErrorFunction,deformationGradientFunction);

    duneVTKwriter.addPointData(isometryErrorGridFunction, "IsometryErrorFunction");
    vtkWriter.addVertexData(isometryErrorGridFunction , VTK::FieldInfo("IsometryErrorFunction", VTK::FieldInfo::Type::scalar, 1));
    




    /**
     * @brief Interpolate and VTK-write analytical solution "u" from ParamterFile.
     */
    if (parameterSet.get<bool>("vtkwrite_analyticalSolution", 0))
    {
      // auto pythonAnalyticalSolution = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("u"), pyModule.get("du"));
      auto pythonAnalyticalSolution = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("displacement"), pyModule.get("displacementGradient"));
      vtkWriter.addVertexData((pythonAnalyticalSolution), VTK::FieldInfo("Displacement_analytical", VTK::FieldInfo::Type::vector, 3));

      //dune-vtk:
      auto pythonAnalyticalSolutionGVF = Dune::Functions::makeAnalyticGridViewFunction(pythonAnalyticalSolution ,gridView);
      duneVTKwriter.addPointData(pythonAnalyticalSolutionGVF, Dune::Vtk::FieldInfo{"Displacement_analytical dune-VTK",3, Dune::Vtk::RangeTypes::VECTOR});



    

      /**
       * @brief Get the normal vector field of the surface parametrized 
       *        by the analytical solution.
       *        - We represent the normal vector in a first order Lagrange-Power basis ('normalBasis').
       */
      if (parameterSet.get<bool>("vtkWrite_analyticalSurfaceNormal", 0))
      {
      // Get the surface normal function.
      auto pythonSurfaceNormal = Python::make_function<FieldVector<double,3>>(pyModule.get("surfaceNormal"));
      vtkWriter.addVertexData(pythonSurfaceNormal, VTK::FieldInfo("SurfaceNormal_analytical", VTK::FieldInfo::Type::vector, 3));


      //dune-vtk 
      auto pythonSurfaceNormalGVF = Dune::Functions::makeAnalyticGridViewFunction(pythonSurfaceNormal ,gridView);
      duneVTKwriter.addPointData(pythonSurfaceNormalGVF, Dune::Vtk::FieldInfo{"SurfaceNormal_analytical dune-VTK",3, Dune::Vtk::RangeTypes::VECTOR});

      }
    }
    //-------------------------------------------------------------------------------------    
    // /**
    //  * @brief: Compute the discrete normal vector of the surface parametrized 
    //  *              by the discrete solution.
    //  */
    auto discreteSurfaceNormalField = [](auto M) -> Dune::FieldVector<double,3> {
      return {M[1][0]*M[2][1] - M[1][1]*M[2][0],
              M[2][0]*M[0][1] - M[0][0]*M[2][1], 
              M[0][0]*M[1][1] - M[1][0]*M[0][1]};
    };

    auto discreteSurfaceNormalFieldGVF = makeComposedGridFunction(discreteSurfaceNormalField ,deformationGradientFunction);

    vtkWriter.addVertexData(discreteSurfaceNormalFieldGVF, VTK::FieldInfo("SurfaceNormalDiscrete", VTK::FieldInfo::Type::vector, 3));
    duneVTKwriter.addPointData(discreteSurfaceNormalFieldGVF, Dune::Vtk::FieldInfo{"SurfaceNormalDiscrete",3, Dune::Vtk::RangeTypes::VECTOR});



    /**
     * @brief deprecated version using interpolation:
     * 
     */
    //  auto surfaceNormalDiscreteCoefficients = computeDiscreteSurfaceNormal(localDKFunctionDouble, normalBasis); (deprecated)
    // 

    // // Create DiscreteGlobalBasisFunctions.
    // auto surfaceNormalDiscrete   = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(normalBasis, surfaceNormalDiscreteCoefficients);
    
    // vtkWriter.addVertexData(surfaceNormalDiscrete , VTK::FieldInfo("surfaceNormal_discrete", VTK::FieldInfo::Type::vector, 3));
    // vtkWriter.write(resultFileName);
    


    
    // duneVTKwriter.addPointData(surfaceNormalDiscrete, Dune::Vtk::FieldInfo{"surfaceNormal_discrete dune-VTK",3, Vtk::RangeTypes::VECTOR});

    // std::cout << "written discrete Surface Normal Old" << std::endl;
    //-------------------------------------------------------------------------------------
    vtkWriter.write(resultFileName);
    duneVTKwriter.write(resultFileName + "_DuneVTK");
  }



    /**
     * @brief Measure errors.   
     *        1.  Compute the Isometry-Error
     *        2.  Compute the L2-Error and H1-SemiError as well as the energy difference 
     *            between the discrete deformation u_h and the analytical deformation u .
     */
    if (parameterSet.get<bool>("measure_isometryError", 0))
        auto isometry_errors = isometryError(localDKFunctionDouble, gridView);
    if (parameterSet.get<bool>("measure_analyticalError", 0))
    {
        auto discretization_errors = measureAnalyticalError(localDKFunctionDouble,gridView,pyModule);


        /**
         * @brief Check Quantities
         * 
         */
        // L2 - Discretization error
        // if(discretization_errors[0] > 1.0)
        // {
        //     std::cerr << std::setprecision(9);
        //     std::cerr << "L2-error is to large! " << std::endl;
        //     return 1;
        // }
        // // H1(semi) - Discretization error
        // if(discretization_errors[1] > 1.0)
        // {
        //     std::cerr << std::setprecision(9);
        //     std::cerr << "H1-error is to large! " << std::endl;
        //     return 1;
        // }
    }
        


    /**
     * @brief Write Energy values.
     * 
     */
    
    std::cout << "(Final) Energy of discrete solution: " << numerical_energy<< std::endl;
    if (parameterSet.get<bool>("compare_EnergyValues", 0))
    {
      auto analytical_energy = parameterSet.get<double>("analytical_energy");
      auto energy_difference = analytical_energy - numerical_energy;
      std::cout << "Analytical energy: " << analytical_energy<< std::endl;  
      std::cout << "Energy difference: " << energy_difference << std::endl;

      // if(energy_difference > 1.0)
      // {
      //   std::cerr << std::setprecision(9);
      //   std::cerr << "Energy difference: " << energy_difference  << " is to large! " << std::endl;
      //   return 1;
      // }
    }

  // // Write the corresponding coefficient vector: verbatim in binary, to be completely lossless
  //   char iFilename[200];
  //   sprintf(iFilename, (resultFileName + ".data").c_str());

  //   FILE* fpIterate = fopen(iFilename, "wb");
  //   if (!fpIterate)
  //       DUNE_THROW(SolverError, "Couldn't open file " << iFilename << " for writing");

  //   for (size_t j=0; j<isometryCoefficients.size(); j++)
  //       fwrite(&isometryCoefficients[j], sizeof(Coefficient), 1, fpIterate);

  //   fclose(fpIterate);


  #if 0
    // Write the corresponding coefficient vector: verbatim in binary, to be completely lossless
    typedef BlockVector<typename Coefficient::CoordinateType> EmbeddedVectorType;
    EmbeddedVectorType xEmbedded(isometryCoefficients.size());
    for (size_t i = 0; i<isometryCoefficients.size(); i++)
    {
      xEmbedded[i] = isometryCoefficients[i].globalCoordinates();
      std::cout << "isometryCoefficients[i].globalCoordinates():" << isometryCoefficients[i].globalCoordinates() << std::endl;

    }
    std::ofstream outFile(resultFileName + ".data", std::ios_base::binary);
    MatrixVector::Generic::writeBinary(outFile, xEmbedded);
    outFile.close();
    ///////////////////////////////////////////
    //  (Option) write Solution(DOF)-vector and vertex coordinates to .txt-File
    ///////////////////////////////////////////
    if (parameterSet.get<bool>("writeDOFvector", 0))
      writeDOFVector(gridView,targetDim, x, "DOFVectorR" + std::to_string(dim) + "_R" + std::to_string(targetDim) + ".txt");
  #endif




  std::cout << "Total time elapsed: " << globalTimer.elapsed() << std::endl;
  return 0;
  }