#include <config.h>
#include <signal.h>
#include <memory>

#include <fenv.h>
#include <array>

#include <math.h>

// Includes for the ADOL-C automatic differentiation library
// Need to come before (almost) all others.
#include <adolc/adouble.h>
#include <dune/fufem/utilities/adolcnamespaceinjections.hh>

#include <dune/common/typetraits.hh>

#include <dune/common/bitsetvector.hh>
#include <dune/common/parametertree.hh>
#include <dune/common/parametertreeparser.hh>

#include <dune/grid/uggrid.hh>
#include <dune/grid/utility/structuredgridfactory.hh>

#include <dune/grid/io/file/gmshreader.hh>
#include <dune/grid/io/file/vtk.hh>

#include <dune/functions/gridfunctions/discreteglobalbasisfunction.hh>
#include <dune/functions/functionspacebases/lagrangebasis.hh>
#include <dune/functions/functionspacebases/powerbasis.hh>
#include <dune/functions/functionspacebases/interpolate.hh>
#include <dune/functions/functionspacebases/reducedcubichermitetrianglebasis.hh>

#include <dune/fufem/boundarypatch.hh>
#include <dune/fufem/functiontools/boundarydofs.hh>
#include <dune/fufem/dunepython.hh>

#include <dune/fufem/discretizationerror.hh>

#include <dune/solvers/solvers/iterativesolver.hh>
#include <dune/solvers/norms/energynorm.hh>

#include <dune/gfe/spaces/productmanifold.hh>
#include <dune/gfe/spaces/realtuple.hh>
#include <dune/gfe/spaces/rotation.hh>
#include <dune/gfe/localdiscretekirchhoffbendingisometry.hh>
#include <dune/gfe/assemblers/localgeodesicfeadolcstiffness.hh>
#include <dune/gfe/assemblers/harmonicenergy.hh>
#include <dune/gfe/assemblers/geodesicfeassembler.hh>
#include <dune/gfe/riemannianpnsolver.hh>
#include <dune/gfe/embeddedglobalgfefunction.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergy.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergyconforming.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergyzienkiewicz.hh>
#include <dune/gfe/energies/discretekirchhoffbendingenergyzienkiewiczprojected.hh>
// #include <dune/gfe/energies/discretekirchhoffbendingenergyprestrained.hh>

#include <dune/microstructure/energies/discretekirchhoffbendingenergyprestrained.hh>

#include <dune/gfe/spaces/stiefelmanifold.hh>
// #include <dune/gfe/spaces/stiefelmatrix.hh>


#include <dune/gmsh4/gmsh4reader.hh>
#include <dune/gmsh4/gridcreators/lagrangegridcreator.hh>

const int dim = 2;

using namespace Dune;


/**
 * @brief Test the interpolation in deformation space.
 *        A linear isometry can be represented exactly in that space.
 *        compare energy values / L2-error / H1-error 
 *        for die difference | I_h[u] - u | where u is a given analytic linear isometry
 */


// /**
//  * @brief Comparison of different function definitions for debugging purposes.
//  * TODO: remove later
//  */
// template<class LocalDiscreteKirchhoffFunction,class DeformationFunction, class GridView>
// auto compare(LocalDiscreteKirchhoffFunction& kirchhoffFunction,
//              DeformationFunction&  deformationFunction,
//              GridView& gridView)
// {
//     std::cout << "COMPARE:" << std::endl;
//     auto localDeformation = localFunction(deformationFunction);
//     auto rotation = derivative(deformationFunction);
//     auto localRotation = localFunction(rotation);
//     auto localRotation_2 = derivative(localDeformation);
//     for (const auto &element :  elements(gridView))
//     {
//       auto geometry = element.geometry();
      
//       kirchhoffFunction.bind(element);
//       localDeformation.bind(element);
//       localRotation.bind(element); 
//       localRotation_2.bind(element); 

//       //evaluate at nodes:
//       for (int i=0; i<geometry.corners(); i++)
//       {
//          auto kirchhoffOut = kirchhoffFunction.evaluate(geometry.local(geometry.corner(i))).globalCoordinates();
//          auto KirchhoffRot = kirchhoffFunction.evaluateDerivative(geometry.local(geometry.corner(i)));
//         //Test
//         const auto& jacobian = geometry.jacobian(geometry.corner(i));
//         printmatrix(std::cout, jacobian, "jacobianInverse:", "--");

//         auto defOut = localDeformation(geometry.local(geometry.corner(i)));
//         auto defRot = localRotation(geometry.local(geometry.corner(i)));
//         auto defRot_2 = localRotation_2(geometry.local(geometry.corner(i)));

//         printvector(std::cout, kirchhoffOut, "kirchhoffOut:", "--");
//         printvector(std::cout, defOut, "defOut:", "--");
//         printmatrix(std::cout, KirchhoffRot, "KirchhoffRot:", "--");
//         printmatrix(std::cout, defRot, "defRot:", "--");
//         // printmatrix(std::cout, defRot*jacobian, "defRot*jacobian:", "--");   // deprecated: when ReducedCubicHermite returned values in globalCoordinates, this was actually neccessary to get the right values. 
//       }
//     }
// }


// /**
//  * @brief Test that checks the isometry constraint at the nodes of the Triangulation
//  */
// template<class LocalDiscreteKirchhoffFunction, class GridView>
// auto nodewiseIsometryTest(LocalDiscreteKirchhoffFunction& deformationFunction,
//                   GridView& gridView)
// {
//     for (const auto &element :  elements(gridView))
//     {
//       auto geometry = element.geometry();
//       deformationFunction.bind(element);
//       // evaluate at nodes:
//       for (int i=0; i<geometry.corners(); i++)
//       {
//         //  auto out = deformationFunction.evaluate(geometry.local(geometry.corner(i))).globalCoordinates();
//          auto rot = deformationFunction.evaluateDerivative(geometry.local(geometry.corner(i)));
//         FieldMatrix<double,2,2> I = ScaledIdentityMatrix<double,2>(1);
//         auto diff = (rot.transposed()*rot) - I;
//         // std::cout << "diff.frobenius_norm():" << diff.frobenius_norm() << std::endl;
//         if(diff.frobenius_norm()>1e-8)
//           DUNE_THROW(Exception, "Nodewise Isometry Test failed! with error"<< diff.frobenius_norm());
//       }
//     }
//     std::cout << "nodewiseIsometryTest passed." << std::endl;
// }


// template<class DeformationFunction>
// auto nodewiseIsometryTest(DeformationFunction& deformationGridViewFunction)
// {
//     auto localDeformation = localFunction(deformationGridViewFunction);
//     auto localRotation = derivative(localDeformation);
//     auto gridView = deformationGridViewFunction.basis().gridView();
//     for (const auto &element :  elements(gridView))
//     {
//       auto geometry = element.geometry();
//       localDeformation.bind(element);
//       localRotation.bind(element);

//       // evaluate at nodes:
//       for (int i=0; i<geometry.corners(); i++)
//       {
//          auto rot = localRotation(geometry.local(geometry.corner(i)));
//         auto product = rot.transposed()*rot;

//         FieldMatrix<double,2,2> I = ScaledIdentityMatrix<double,2>(1);
//         auto diff = (rot.transposed()*rot) - I;

//         if(diff.frobenius_norm()>1e-8)
//           DUNE_THROW(Exception, "Nodewise Isometry Test failed! with error"<< diff.frobenius_norm());
//       }
//     }
//     std::cout << "nodewiseIsometryTest passed." << std::endl;
// }



// /** \brief The identity function in R^d, and its derivative
//  *
//  * This is needed to compute the displacement from the deformation (for visualization).
//  * Since we want to represent it in a Hermite finite element space, we need
//  * the derivative as well.
//  *
//  * Is there no shorter way to implement this?
//  */
// template<class K>
// class IdentityGridEmbedding
// {
// public:
//   //! Evaluate identity
//   FieldVector<K,3> operator() (const FieldVector<K,2>& x) const
//   {
//     return {x[0], x[1], 0.0};
//   }

//   /**
//    * \brief Obtain derivative of identity function
//    */
//   friend auto derivative(const IdentityGridEmbedding& p)
//   {
//     return [](const FieldVector<K,dim>& x) { return FieldMatrix<K,3,2>({{1,0}, {0,1}, {0,0}}); };
//   }
// };

// /** \brief Project a vector field onto a product of ReducedHermiteTriangle spaces
//  *
//  * \todo This functionality should be provided by dune-functions.  It currently isn't,
//  * because ReducedHermiteTriangleBasis is not an affine finite element.
//  */
// template <class Basis, class Coefficients, class Function>
// void interpolate(const Basis& basis, Coefficients& coefficients, Function& f)
// {
//   coefficients.resize(basis.size());
//   // coefficients.resize(basis.dimension());
//   auto localView = basis.localView();

//   for (auto&& e : elements(basis.gridView()))
//   {
//     localView.bind(e);

//     // Loop over the corners
//     for (std::size_t i=0; i<3; i++)
//     {
//       auto pos = e.geometry().corner(i);
//       auto value = f(pos);
//       auto der = derivative(f)(pos);

//       // printvector(std::cout, value, "value:", "--");
//       // printmatrix(std::cout, der, "der:", "--");

//       // std::cout << "sin(x):" << sin(pos[0]) << std::endl;
//       // std::cout << "cos(x):" << cos(pos[0]) << std::endl;
//       // printvector(std::cout, pos, "pos: ", "--");

//       // Loop over components of power basis
//       for(std::size_t k=0; k<3 ; k++)
//       {
//         auto localIdx_0 = localView.tree().child(k).localIndex(i);    // index for values
//         auto localIdx_1 = localView.tree().child(k).localIndex(i+3);  // index for partial derivatives w.r.t. x
//         auto localIdx_2 = localView.tree().child(k).localIndex(i+6);  // index for partial derivatives w.r.t. y
//         // std::cout << "localIdx_0: " << localIdx_0 << std::endl;
//         // std::cout << "localIdx_1: " << localIdx_1 << std::endl;
//         // std::cout << "localIdx_2: " << localIdx_2 << std::endl;

//         auto globalIdx_0 = localView.index(localIdx_0);
//         auto globalIdx_1 = localView.index(localIdx_1);
//         auto globalIdx_2 = localView.index(localIdx_2);
//         // std::cout << "globalIdx_0: " << globalIdx_0 << std::endl;
//         // std::cout << "globalIdx_1: " << globalIdx_1 << std::endl;
//         // std::cout << "globalIdx_2: " << globalIdx_2 << std::endl;

//         coefficients[globalIdx_0[0]][globalIdx_0[1]] = value[k];
//         coefficients[globalIdx_1[0]][globalIdx_1[1]] = der[k][0];
//         coefficients[globalIdx_2[0]][globalIdx_2[1]] = der[k][1];
//       }
//     }

//   }
// }

// /**
//  * @brief Data structure conversion from 'VectorSpaceCoefficients' to 'IsometryCoefficients'.
//  *  TODO: Throw an error if rotation-part is not orthogonal?  (expensive to check)
//  */
// template<class DiscreteKirchhoffBasis, class CoefficientBasis, class Coefficients, class IsometryCoefficients>
// void vectorToIsometryCoefficientMap(const DiscreteKirchhoffBasis& discreteKirchhoffBasis,
//                              const CoefficientBasis& coefficientBasis,
//                              const Coefficients& vectorCoefficients,
//                              IsometryCoefficients& isometryCoefficients)
// {
//   std::cout << "calling vectorToIsometryCoefficientMap" << std::endl;
//   auto localView = discreteKirchhoffBasis.localView();
//   auto localViewCoefficients = coefficientBasis.localView();

//   using RT = typename IsometryCoefficients::value_type::ctype;

//   for (const auto &element :  elements(coefficientBasis.gridView()))
//   {
//     localView.bind(element);
//     localViewCoefficients.bind(element);

//     // Loop over the corners
//     for (std::size_t c=0; c<3; c++)
//     {
//         size_t localIdxOut = localViewCoefficients.tree().localIndex(c);
//         size_t globalIdxOut = localViewCoefficients.index(localIdxOut);

//         FieldVector<RT,3> currentDeformation(0);
//         FieldVector<RT,3> currentDerivative_x(0);
//         FieldVector<RT,3> currentDerivative_y(0);

//       // Loop over components of power basis
//       for(std::size_t k=0; k<3 ; k++)
//       {
//         auto localIdx_0 = localView.tree().child(k).localIndex(c);    // index for values
//         auto localIdx_1 = localView.tree().child(k).localIndex(c+3);  // index for partial derivatives w.r.t. x
//         auto localIdx_2 = localView.tree().child(k).localIndex(c+6);  // index for partial derivatives w.r.t. y
//         // std::cout << "localIdx_0: " << localIdx_0 << std::endl;
//         // std::cout << "localIdx_1: " << localIdx_1 << std::endl;
//         // std::cout << "localIdx_2: " << localIdx_2 << std::endl;

//         auto globalIdx_0 = localView.index(localIdx_0);
//         auto globalIdx_1 = localView.index(localIdx_1);
//         auto globalIdx_2 = localView.index(localIdx_2);

//         // blockedInterleaved -- Version
//         currentDeformation[k]  = vectorCoefficients[globalIdx_0[0]][globalIdx_0[1]];
//         currentDerivative_x[k] = vectorCoefficients[globalIdx_1[0]][globalIdx_1[1]];
//         currentDerivative_y[k] = vectorCoefficients[globalIdx_2[0]][globalIdx_2[1]];

//       }

//       //cross product to get last column of rotation matrix
//       FieldVector<RT,3> cross = {currentDerivative_x[1]*currentDerivative_y[2] - currentDerivative_x[2]*currentDerivative_y[1],
//                                  currentDerivative_x[2]*currentDerivative_y[0] - currentDerivative_x[0]*currentDerivative_y[2], 
//                                  currentDerivative_x[0]*currentDerivative_y[1] - currentDerivative_x[1]*currentDerivative_y[0]};

//       FieldMatrix<RT,3,3> rotMatrix(0);
//       for(std::size_t k=0; k<3 ; k++)
//       {
//         rotMatrix[k][0] = currentDerivative_x[k];
//         rotMatrix[k][1] = currentDerivative_y[k];
//         rotMatrix[k][2] = cross[k];
//       }
//       // printmatrix(std::cout, rotMatrix, "rotMatrix:", "--");
//       using namespace Dune::Indices;
//       isometryCoefficients[globalIdxOut][_1].set(rotMatrix);                          //TODO: Throw if matrix is not orthogonal?
//       isometryCoefficients[globalIdxOut][_0] = (RealTuple<RT, 3>)currentDeformation;
//     }
//   }
// }


// /**
//  * @brief Data structure conversion from 'IsometryCoefficients' to 'VectorSpaceCoefficients'.
//  */
// template<class DiscreteKirchhoffBasis, class CoefficientBasis, class Coefficients, class IsometryCoefficients>
// void isometryToVectorCoefficientMap(const DiscreteKirchhoffBasis& discreteKirchhoffBasis,
//                              const CoefficientBasis& coefficientBasis,
//                              Coefficients& coefficients,
//                              const IsometryCoefficients& isometryCoefficients)
// {
//   std::cout << "calling isometryToVectorCoefficientMap" << std::endl;
//   auto localView = discreteKirchhoffBasis.localView();
//   auto localViewCoefficients = coefficientBasis.localView();

//   for (const auto &element :  elements(coefficientBasis.gridView()))
//   {
//     localView.bind(element);
//     localViewCoefficients.bind(element);

//     // Loop over the corners
//     for (std::size_t c=0; c<3; c++)
//     {
//       size_t localIdx = localViewCoefficients.tree().localIndex(c);
//       size_t globalIdx = localViewCoefficients.index(localIdx);

//       // Loop over components of power basis
//       for(std::size_t k=0; k<3 ; k++)
//       {
//         auto localIdx_0 = localView.tree().child(k).localIndex(c);    // index for values
//         auto localIdx_1 = localView.tree().child(k).localIndex(c+3);  // index for partial derivatives w.r.t. x
//         auto localIdx_2 = localView.tree().child(k).localIndex(c+6);  // index for partial derivatives w.r.t. y
//         // std::cout << "localIdx_0: " << localIdx_0 << std::endl;
//         // std::cout << "localIdx_1: " << localIdx_1 << std::endl;
//         // std::cout << "localIdx_2: " << localIdx_2 << std::endl;

//         auto globalIdxOut_0 = localView.index(localIdx_0);
//         auto globalIdxOut_1 = localView.index(localIdx_1);
//         auto globalIdxOut_2 = localView.index(localIdx_2);

//         using namespace Dune::Indices;
//         auto deformation = isometryCoefficients[globalIdx][_0].globalCoordinates();
//         auto rotation1 = isometryCoefficients[globalIdx][_1].director(0);       
//         auto rotation2 = isometryCoefficients[globalIdx][_1].director(1);
      
//         coefficients[globalIdxOut_0[0]][globalIdxOut_0[1]] = deformation[k];
//         coefficients[globalIdxOut_1[0]][globalIdxOut_1[1]] = rotation1[k];
//         coefficients[globalIdxOut_2[0]][globalIdxOut_2[1]] = rotation2[k];
//       }
//     }
//   }
// }


int main(int argc, char *argv[])
{

  /**
   * @brief We use this to catch a 'Floating point exception' caused by the 'innerSolver' 
   *        in RiemannianProximalNewton
   */
    std::shared_ptr<void(int)> handler(
        signal(SIGFPE, [](int signum) {throw std::logic_error("FPE"); }),
        [](__sighandler_t f) { signal(SIGFPE, f); });



  feenableexcept(FE_INVALID);

  MPIHelper::instance(argc, argv);
  Dune::Timer globalTimer;

  std::cout << "Current path is " << std::filesystem::current_path() << '\n';
  std::filesystem::path file_path = (__FILE__);
  std::cout<< "File path: " << file_path<<std::endl;
  std::cout << "dir_path: " << file_path.parent_path() << std::endl;
  std::string dir_path  = file_path.parent_path();

  // Start Python interpreter
  Python::start();
  auto pyMain = Python::main();
  pyMain.runStream()
      << std::endl << "import math"
      << std::endl << "import sys"
      << std::endl << "sys.path.append('" << dir_path << "')"  << std::endl;
  auto pyModule = pyMain.import("linearisometrytest");

  // parse data file
  ParameterTree parameterSet;
  pyModule.get("parameterSet").toC(parameterSet);

  // read possible further parameters from the command line
  ParameterTreeParser::readOptions(argc, argv, parameterSet);

  // Print all parameters, to make them appear in the log file
  std::cout << "Executable: bending-isometries, with parameters:" << std::endl;
  parameterSet.report();

  /////////////////////////////////////////
  //   Create the grid
  /////////////////////////////////////////
  using GridType = UGGrid<dim>;
  std::shared_ptr<GridType> grid;
//   FieldVector<double,dim> lower(0), upper(1);
//   std::array<unsigned int,dim> elementsArray;

  // create unstructured grid.
  std::cout << "Read GMSH grid." << std::endl;
  GridFactory<GridType> factory;
  Gmsh4::LagrangeGridCreator creator{factory};
  Gmsh4Reader reader{creator};
  reader.read(dir_path + "/" + std::string("UnstructuredGrid.msh"));
  grid = factory.createGrid();


  const int numLevels = parameterSet.get<int>("numLevels");
  grid->globalRefine(numLevels-1);

  using GridView = typename GridType::LeafGridView;
  GridView gridView = grid->leafGridView();

  ///////////////////////////////////////////////////////
  //  Set up the function space and the energy
  ///////////////////////////////////////////////////////

  // General coefficient vector of a Discrete Kirchhoff deformation function
  using VectorSpaceCoefficients = BlockVector<FieldVector<double,3>>;

  // Coefficient vector of a Discrete Kirchhoff deformation function that is constrained to be an isometry
  using Coefficient = GFE::ProductManifold<RealTuple<double,3>, Rotation<double,3> >;
  using IsometryCoefficients = std::vector<Coefficient>;

  using namespace Functions::BasisFactory;

  auto deformationBasis = makeBasis(gridView,
                                    power<3>(reducedCubicHermiteTriangle(),
                                             blockedInterleaved()));


  // The next basis is used to assign (nonlinear) degrees of freedom to the grid vertices.
  // The actual basis function values are never used.
  using CoefficientBasis = Functions::LagrangeBasis<GridView, 1>;
  CoefficientBasis coefficientBasis(gridView);

  // A basis for the tangent space (used to set DirichletNodes)
  auto tangentBasis = makeBasis(gridView,
                                power<Coefficient::TangentVector::dimension>(
                                    lagrange<1>(),
                                    blockedInterleaved()));


  ///////////////////////////////////////////
  //   Read Dirichlet values 
  ///////////////////////////////////////////
  BitSetVector<1> dirichletVertices(gridView.size(dim), false);
  const typename GridView::IndexSet &indexSet = gridView.indexSet();

  // Make Python function that computes which vertices are on the Dirichlet boundary,
  // based on the vertex positions.
  auto dirichletIndicatorFunction = Python::make_function<bool>(pyModule.get("dirichlet_indicator"));

  for (auto &&vertex : vertices(gridView))
    dirichletVertices[indexSet.index(vertex)] = dirichletIndicatorFunction(vertex.geometry().corner(0));

  BoundaryPatch<GridView> dirichletBoundary(gridView, dirichletVertices);
  BitSetVector<Coefficient::TangentVector::dimension> dirichletNodes(tangentBasis.size(), false);
  constructBoundaryDofs(dirichletBoundary, tangentBasis, dirichletNodes);

  ///////////////////////////////////////////
  //   Get initial Iterate
  ///////////////////////////////////////////
  auto pythonInitialIterate = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("f"), pyModule.get("df"));

  VectorSpaceCoefficients x(deformationBasis.size());

  IdentityGridEmbedding<double> identityGridEmbedding;
  // interpolate(deformationBasis, x, identityGridEmbedding);
  interpolate(deformationBasis, x, pythonInitialIterate);   // Interpolation on a python function

  ////////////////////////////////////////////////////////////////////////
  //   Create an assembler for the Discrete Kirchhoff Energy Functional
  ////////////////////////////////////////////////////////////////////////

  // Copy the current iterate into a data type that encapsulates the isometry constraint
  using ACoefficient = typename Coefficient::template rebind<adouble>::other;
  using AIsometryCoefficients = std::vector<ACoefficient>;


  /**
   * @brief We need to setup LocalDiscreteKirchhoffBendingIsometry with a coefficient 
   * vector of ctype 'adouble' while the solver gets a coefficient vector 
   * of ctype 'double'.
   */
  IsometryCoefficients isometryCoefficients(coefficientBasis.size());
  AIsometryCoefficients isometryCoefficients_adouble(coefficientBasis.size());

  std::cout << "Number of Elements in the grid:" << gridView.size(0)<< std::endl;
  std::cout << "Number of Nodes in the grid:" << gridView.size(dim)<< std::endl;
  std::cout << "deformationBasis.size():" << deformationBasis.size() << std::endl;
  std::cout << "deformationBasis.dimension():" << deformationBasis.dimension() << std::endl;


    /**
     * @brief TEST: compute isometry error
     */
    auto initialDeformationFunction = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, x);
    // nodewiseIsometryTest(initialDeformationFunction);




  // Convert coefficient data structure from 'VectorSpaceCoefficients' to 'IsometryCoefficients'
  vectorToIsometryCoefficientMap(deformationBasis,coefficientBasis,x,isometryCoefficients);
  vectorToIsometryCoefficientMap(deformationBasis,coefficientBasis,x,isometryCoefficients_adouble);


    /**
     * @brief TEST: conversion of coefficient vectors (back & forth)
     */
    VectorSpaceCoefficients x_convert = x;
    IsometryCoefficients isometryCoefficientsTMP(coefficientBasis.size());
    vectorToIsometryCoefficientMap(deformationBasis,coefficientBasis,x_convert,isometryCoefficientsTMP);
    isometryToVectorCoefficientMap(deformationBasis,coefficientBasis,x_convert,isometryCoefficientsTMP);
    //Check difference
    for(int i=0; i<x.size(); i++)
    {
      if((x[i]-x_convert[i]).two_norm() > 1e-4)
        std::cout << "Coefficient conversion failed! with x[i]-x_convert[i]:" << x[i]-x_convert[i] << std::endl;
    }
      




  using DeformationBasis = decltype(deformationBasis);
  using LocalDKFunction = GFE::LocalDiscreteKirchhoffBendingIsometry<DeformationBasis, CoefficientBasis, AIsometryCoefficients>;
  LocalDKFunction localDKFunction(deformationBasis, coefficientBasis, isometryCoefficients_adouble);
  nodewiseIsometryTest(localDKFunction, gridView);



  /**
   * @brief Get force term
   */
  auto pythonForce = Python::make_function<FieldVector<double,3>>(pyModule.get("force"));
  // auto forceGVF  = Dune::Functions::makeGridViewFunction(dummyForce, gridView);
  auto forceGVF  = Dune::Functions::makeGridViewFunction(pythonForce, gridView);
  auto localForce = localFunction(forceGVF);


  /**
   * @brief Get effective prestrain Tensor
   */
  Dune::FieldVector<adouble,3> Beffvec = parameterSet.get<Dune::FieldVector<adouble,3>>("effectivePrestrain", {0.0, 0.0, 0.0});
  Dune::FieldMatrix<adouble,2,2> effectivePrestrain = {{(adouble)Beffvec[0], (adouble)Beffvec[2]}, {(adouble)Beffvec[2],(adouble)Beffvec[1]} };
  printmatrix(std::cout, effectivePrestrain, "effective prestrain (Beff): ", "--");


  /**
   * @brief Get effective quadratic form
   */
  Dune::FieldVector<adouble,6> Qhomvec = parameterSet.get<Dune::FieldVector<adouble,6>>("effectiveQuadraticForm", {1.0, 1.0, 1.0, 0.0, 0.0, 0.0});
  Dune::FieldMatrix<adouble,3,3> Qhom = {{(adouble)Qhomvec[0], (adouble)Qhomvec[3], (adouble)Qhomvec[4]}, 
                                         {(adouble)Qhomvec[3], (adouble)Qhomvec[1], (adouble)Qhomvec[5]},
                                         {(adouble)Qhomvec[4], (adouble)Qhomvec[5], (adouble)Qhomvec[2]}};
  printmatrix(std::cout, Qhom, "effective quadratic form (Qhom): ", "--");


  // Assembler using ADOL-C
  /**
   * @brief Setup nonconforming energy
   */
  auto localEnergy_nonconforming = std::make_shared<GFE::DiscreteKirchhoffBendingEnergy<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce);
  /**
   * @brief Setup conforming energy (WIP)
   */
  auto localEnergy_conforming = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyZienkiewicz<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce);
  // auto localEnergy_conforming = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyZienkiewiczProjected<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce);
  /**
   * @brief Setup energy featuring prestrain
   */
  //  auto localEnergy_prestrain = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyPrestrained<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce, Qhom, effectivePrestrain);
  auto localEnergy_prestrain = std::make_shared<GFE::DiscreteKirchhoffBendingEnergyPrestrained<CoefficientBasis, LocalDKFunction, decltype(localForce), ACoefficient>>(localDKFunction, localForce, parameterSet, pyModule);


  LocalGeodesicFEADOLCStiffness<CoefficientBasis, Coefficient> localGFEADOLCStiffness_conforming(localEnergy_conforming.get());
  LocalGeodesicFEADOLCStiffness<CoefficientBasis, Coefficient> localGFEADOLCStiffness_nonconforming(localEnergy_nonconforming.get());
  LocalGeodesicFEADOLCStiffness<CoefficientBasis, Coefficient> localGFEADOLCStiffness_prestrain(localEnergy_prestrain.get());
  GeodesicFEAssembler<CoefficientBasis, Coefficient> assembler_conforming(coefficientBasis, localGFEADOLCStiffness_conforming);
  GeodesicFEAssembler<CoefficientBasis, Coefficient> assembler_nonconforming(coefficientBasis, localGFEADOLCStiffness_nonconforming);
  GeodesicFEAssembler<CoefficientBasis, Coefficient> assembler_prestrain(coefficientBasis, localGFEADOLCStiffness_prestrain);

  // /////////////////////////////////////////////////
  //   Create a Riemannian trust-region solver
  // /////////////////////////////////////////////////
  RiemannianProximalNewtonSolver<CoefficientBasis, Coefficient> solver;



  if (parameterSet.get<bool>("prestrainFlag", 0))
      solver.setup(*grid,
                &assembler_prestrain,
                isometryCoefficients,
                dirichletNodes,
                parameterSet);
  else if (parameterSet.get<bool>("conforming_DiscreteJacobian", 1))
      solver.setup(*grid,
                  &assembler_conforming,
                  isometryCoefficients,
                  dirichletNodes,
                  parameterSet);
  else 
      solver.setup(*grid,
                &assembler_nonconforming,
                isometryCoefficients,
                dirichletNodes,
                parameterSet);

  // /////////////////////////////////////////////////////
  //   Solve!
  // /////////////////////////////////////////////////////
  // solver.setInitialIterate(isometryCoefficients);
  solver.solve();
  isometryCoefficients = solver.getSol();

  // Convert coefficient data structure from 'IsometryCoefficients' to 'VectorSpaceCoefficients'
  VectorSpaceCoefficients x_out(deformationBasis.size());
  isometryToVectorCoefficientMap(deformationBasis,coefficientBasis,x_out,isometryCoefficients);

  ////////////////////////////////
  //   Output result
  ////////////////////////////////
  std::string baseNameDefault = "bending-isometries-";
  std::string baseName = parameterSet.get("baseName", baseNameDefault);
  std::string resultFileName =  parameterSet.get("resultPath", "")
                                + "/" + baseName
                                + "_level" + std::to_string(parameterSet.get<int>("numLevels"));

  if (parameterSet.get<bool>("conforming_DiscreteJacobian", 1))
    resultFileName = resultFileName + "_C";
  else 
    resultFileName = resultFileName + "_NC";

    // Create a deformation function but this time with double-types.
  using LocalDKFunctionD = GFE::LocalDiscreteKirchhoffBendingIsometry<DeformationBasis, CoefficientBasis, IsometryCoefficients>;
  LocalDKFunctionD localDKFunctionDouble(deformationBasis, coefficientBasis, isometryCoefficients);


  if (parameterSet.get<bool>("writeVTK", 1))
  {
    std::cout << "write VTK..." << std::endl;

    /**
     * @brief Compute the displacement from the deformation.
     *        interpolate the identity and substract from the coefficient vector.
     */
    VectorSpaceCoefficients identity(deformationBasis.size());
    IdentityGridEmbedding<double> identityGridEmbedding;
    interpolate(deformationBasis, identity, identityGridEmbedding);

    // auto identity_tmp = identity;

    // Compute the displacement
    auto displacement = x_out;
    displacement -= identity;

    // std::cout << "displacement.size():" << displacement.size() << std::endl;
    auto deformationFunction = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, x_out);
    auto displacementFunction = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, displacement);

    /**
     * @brief  We need to subsample, because VTK cannot natively display real third-order functions
     */
    int subsamplingRefinement = parameterSet.get<int>("subsamplingRefinement", 2);
    SubsamplingVTKWriter<GridView> vtkWriter(gridView, Dune::refinementLevels(subsamplingRefinement));
    // SubsamplingVTKWriter<GridView> vtkWriter(gridView, Dune::refinementLevels(2));



    /**
     * @brief Basis used to represent normal vector fields
     * 
     */
    auto normalBasis = makeBasis(gridView,
                      power<3>(
                      lagrange<1>(),
                      flatLexicographic()));
    // auto normalBasis = makeBasis(gridView,
    //                               power<3>(
    //                               lagrange<1>(),
    //                               blockedInterleaved()));


    // TEST Compute 


    // auto normalLambda = [deformationFunction](const FieldVector<double,2>& x)
    // {
    //   // deformationfunction.derivative() ... //needs binding?!
    

    // }

    


    /**
     * @brief Interpolate and VTK-write analytical solution "u" from ParamterFile.
     */
    if (parameterSet.get<bool>("vtkwrite_analyticalSolution", 0))
    {
      // auto pythonAnalyticalSolution = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("u"), pyModule.get("du"));
      auto pythonAnalyticalSolution = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("displacement"), pyModule.get("displacementGradient"));


      // deprecated: interpolate ...
      // VectorSpaceCoefficients y(deformationBasis.size());
      // interpolate(deformationBasis, y, pythonAnalyticalSolution);
      // // Compute the displacement
      // auto displacementAnalytical = y;
      // // displacementAnalytical -= identity_tmp;
      // displacementAnalytical -= identity;


      // auto pythonIdentity =   Python::make_function<FieldVector<double,3>>(IdentityGridEmbedding<double>::operator())


      // auto displacementFunctionAnalytical = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, displacementAnalytical);
      // auto deformationFunctionAnalytical = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(deformationBasis, y);
      // vtkWriter.addVertexData(displacementFunctionAnalytical, VTK::FieldInfo("displacement_analytical", VTK::FieldInfo::Type::vector, 3));
      // vtkWriter.addVertexData(deformationFunctionAnalytical, VTK::FieldInfo("deformation_analytical", VTK::FieldInfo::Type::vector, 3));
    
      vtkWriter.addVertexData((pythonAnalyticalSolution), VTK::FieldInfo("displacement_analytical", VTK::FieldInfo::Type::vector, 3));
      /**
       * @brief Get the normal vector field of the surface parametrized 
       *        by the analytical solution.
       *        - We represent the normal vector in a first order Lagrange-Power basis ('normalBasis').
       */
      if (parameterSet.get<bool>("vtkWrite_analyticalSurfaceNormal", 0))
      {
      // Get the surface normal function.
      auto pythonSurfaceNormal = Python::make_function<FieldVector<double,3>>(pyModule.get("surfaceNormal"));


      // deprecated: interpolate ...
      // std::vector<FieldVector<double,3>> normalVectorCoefficients(normalBasis.size());
      // Dune::Functions::interpolate(normalBasis, normalVectorCoefficients, pythonSurfaceNormal); 
      // auto surfaceNormalAnalytical = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(normalBasis, normalVectorCoefficients);
      // vtkWriter.addVertexData(surfaceNormalAnalytical, VTK::FieldInfo("surfaceNormal_analytical", VTK::FieldInfo::Type::vector, 3));

      vtkWriter.addVertexData(pythonSurfaceNormal, VTK::FieldInfo("surfaceNormal_analytical", VTK::FieldInfo::Type::vector, 3));
      }
    }
    //-------------------------------------------------------------------------------------    TODO: OUTSOURCE
    /**
     * @brief TEST: Compute the discrete normal vector of the surface parametrized 
     *              by the discrete solution.
     */
     auto surfaceNormalDiscreteCoefficients = computeDiscreteSurfaceNormal(localDKFunctionDouble, normalBasis);
    //-------------------------------------------------------------------------------------

    // Create DiscreteGlobalBasisFunctions.
    auto surfaceNormalDiscrete   = Functions::makeDiscreteGlobalBasisFunction<FieldVector<double,3>>(normalBasis, surfaceNormalDiscreteCoefficients);
    vtkWriter.addVertexData(displacementFunction, VTK::FieldInfo("displacement", VTK::FieldInfo::Type::vector, 3));
    vtkWriter.addVertexData(surfaceNormalDiscrete , VTK::FieldInfo("surfaceNormal_discrete", VTK::FieldInfo::Type::vector, 3));
    vtkWriter.write(resultFileName);
  }

  /**
   * @brief Compute the L2-Error and H1-SemiError between the discrete deformation u_h and the analytical deformation u 
   * 
   */
    // Read reference solution and its derivative into a Python function
    // auto referenceSolution = Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("u"), pyModule.get("du"));
    // auto referenceSolution =  Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("displacement"), pyModule.get("displacementGradient"));
    auto referenceSolution =  Python::makeDifferentiableFunction<FieldVector<double,3>(FieldVector<double,2>)>(pyModule.get("deformation"), pyModule.get("deformationGradient"));
    auto referenceDerivative = derivative(referenceSolution);

    using LocalDKFunctionD = GFE::LocalDiscreteKirchhoffBendingIsometry<DeformationBasis, CoefficientBasis, IsometryCoefficients>;
    LocalDKFunctionD localDKFunctionD(deformationBasis, coefficientBasis, isometryCoefficients);


    // QuadratureRule for the integral of the L^2 error
    QuadratureRuleKey quadKey(dim,6);

    // The error to be computed
    double l2ErrorSquared = 0;
    double h1ErrorSquared = 0;

    for (auto&& element : elements(gridView))
    {

    //   localNumericalSolution.bind(element);

      localDKFunctionD.bind(element);

      // Get quadrature formula
      quadKey.setGeometryType(element.type());
      const auto& quad = QuadratureRuleCache<double, dim>::rule(quadKey);

      for (auto quadPoint : quad)
      {
        auto quadPos = quadPoint.position();
        const auto integrationElement = element.geometry().integrationElement(quadPos);
        const auto weight = quadPoint.weight();

        // auto numValue = localNumericalSolution(quadPos);
        auto refValue = referenceSolution(element.geometry().global(quadPos));
        auto numValue = localDKFunctionD.evaluate(quadPos).globalCoordinates();

        auto numDir = localDKFunctionD.evaluateDerivative(quadPos);
        auto refDir = referenceDerivative(element.geometry().global(quadPos));
        auto diff = numDir - refDir;


        // integrate error
        l2ErrorSquared += (numValue - refValue)* (numValue - refValue) * weight * integrationElement;
        h1ErrorSquared += diff.frobenius_norm2() * weight * integrationElement;
      }
    }
    std::cout << "elements: " << gridView.size(0)
              << "      "
              << "L^2 error: " << std::sqrt(l2ErrorSquared)
              << "      ";
    std::cout << "h^1 error: " << std::sqrt(h1ErrorSquared) << std::endl;

   
    //Check Quantities
    if( std::sqrt(l2ErrorSquared) > 1e-12)
    {
        std::cerr << std::setprecision(9);
        std::cerr << "L2-error is to large! " << std::endl;
        return 1;
    }
    if( std::sqrt(h1ErrorSquared) > 1e-12)
    {
        std::cerr << std::setprecision(9);
        std::cerr << "H1-error is to large! " << std::endl;
        return 1;
    }
    // Compare energy values.
    auto numerical_energy = solver.getStatistics().finalEnergy;
    auto analytical_energy = 0.0;
    std::cout << "Energy difference: " << std::abs(analytical_energy - numerical_energy)  << std::endl;
    if( std::abs(analytical_energy - numerical_energy) > 1e-12)
    {
        std::cerr << std::setprecision(9);
        std::cerr << "Energy difference: " << std::abs(analytical_energy - numerical_energy)  << " is to large! " << std::endl;
        return 1;
    }

  std::cout << "Total time elapsed: " << globalTimer.elapsed() << std::endl;
  std::cout << "Test: linearisometrytest passed." << std::endl;
  return 0;
  }