import math
import numpy as np

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()


#############################################
#  Paths
#############################################
parameterSet.resultPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs_linearisometrytest'
# parameterSet.instrumentedPath = '/home/klaus/Desktop/Dune_release/dune-gfe/outputs_rotation-test/instrumented'
# parameterSet.executablePath = '/home/klaus/Desktop/Dune_release/dune-gfe/build-cmake/src'
parameterSet.baseName= 'linearisometrytest'


#############################################
#  Grid parameters
#############################################
# nX=1
# nY=1
# # parameterSet.structuredGrid = 'simplex'
# parameterSet.structuredGrid = 'false'
# parameterSet.lower = '0 0'
# parameterSet.upper = '4 4'
# # parameterSet.upper = '6.28318530718 6.28318530718'     # 2* pi
# parameterSet.elements = str(nX)+' '+  str(nY)

parameterSet.numLevels = 3

# parameterSet.gridPath = '/home/klaus/Desktop/Dune_release/dune-gfe/problems'
# parameterSet.gridFile = 'Geometry2.msh'



#############################################
#  GFE parameters
#############################################
parameterSet.order = 1

parameterSet.conforming_DiscreteJacobian = 1

#############################################
#  Solver parameters
#############################################

# Tolerance of the multigrid solver
parameterSet.tolerance = 1e-12
# Maximum number of multigrid iterations
parameterSet.maxProximalNewtonSteps = 0
# Initial regularization
parameterSet.initialRegularization = 1
# Measure convergence
parameterSet.instrumented = 1

############################
#   Problem specifications
############################
# Dimension of the domain (only used for numerical-test python files)
parameterSet.dim = 2


# Dimension of the target space
parameterSet.targetDim = 3
parameterSet.targetSpace = 'BendingIsometry'

#############################################
#  Options
#############################################
# Write discrete solution as .vtk-File
parameterSet.writeVTK = 1
parameterSet.vtkwrite_analyticalSolution = 1
parameterSet.vtkWrite_analyticalSurfaceNormal = 0

# Write Dof-Vector to .txt-file
parameterSet.writeDOFvector = 0




#############################################
#  MICROSTRUCTURE
############################################
# parameterSet.printMicroOutput = False
# parameterSet.VTKwriteMacroPhaseIndicator = False
# parameterSet.MacroPhaseSubsamplingRefinement = 3

class GlobalMicrostructure:
    """ Class that represents the global microstructure.
    
        The global microstructure can be defined individually 
        for different (finitely many) macroscopic phases. 
        Each macroscopic phase corresponds to a 'LocalMicrostructure_'
        sub-class that defines the necessary data for the local 
        micro-problem. 

        Currently, there are three possibilities for the LocalMicrostructure:
            (1) Read the effective quantities (Qhom,Beff) from this parset.
            (Constant local microstructure)
            (2) Solve the micro-problem once to obtain the effective quantities. 
            (Constant local microstructure)
            (3) Solve for micro-problem for each macroscopic quadrature point.
            (Macroscopocally varying local microstructure)) 
    """
    def __init__(self,macroPoint=[0,0]):
        self.macroPhases = 1

        # define first local microstructure options.
        self.macroPhase1_constantMicrostructure = True
        self.macroPhase1_readEffectiveQuantities = True;

    def macroPhaseIndicator(self,y): #y :macroscopic point
            """ Indicatorfunction that determines the domains 
                i.e. macro-phases of different local microstructures.
            """
            return 1;
                
    # Represents the local microstructure in Phase 1
    class LocalMicrostructure_1:  
        def __init__(self,macroPoint=[0,0]):
            self.gamma = 1.0    #in the future this might change depending on macroPoint.
            self.phases = 2     #in the future this might change depending on macroPoint.

            self.effectivePrestrain= np.array([[0.0,  0.0],
                                               [ 0.0, 0.0]]);

            self.effectiveQuadraticForm = np.array([[1.0, 0.0, 0.0],
                                                    [0.0, 1.0, 0.0],
                                                    [0.0, 0.0, 1.0]]);


#############################################
#  Dirichlet boundary indicator
#############################################
def dirichlet_indicator(x) :
    if( (x[0] <= 0.0001) ):
        return True
    else:
        return False


#############################################
#  Initial iterate function
#############################################
def f(x):
    angle = -math.pi/4.0;
    return [x[0]*math.cos(angle), x[1], -x[0]*math.sin(angle)]


#rotation around y-axis with angle = pi/4
def df(x):
    angle = -math.pi/4.0;
    return [[math.cos(angle),0],
            [0,1],
            [-math.sin(angle),0]]


fdf = (f, df)


#############################################
#  Force
############################################
def force(x):
    return [0, 0, 0]

# #############################################
# #  Effective prestrain
# ############################################
# parameterSet.prestrainFlag = True
# parameterSet.macroscopically_varying_microstructure = False
# parameterSet.read_effectiveQuantities_from_Parset = True # Otherwise the Micro/Cell-Problem is solved once to obtain the quantities.

# # parameterSet.effectivePrestrain = '0.0 0.0 0.0'   # b1 b2 b12

# # parameterSet.read_effectiveQuantities_from_Parset = True
# effectivePrestrain= np.array([[0.0, 0.0],
#                               [0.0, 0.0]]);

# effectiveQuadraticForm = np.array([[1.0, 0.0, 0.0],
#                                    [0.0, 1.0, 0.0],
#                                    [0.0, 0.0, 1.0]]);


# class Microstructure:  #Dummy class. Just nedded for initialization
#     def __init__(self):
#         self.gamma = 1 
#         self.phases = 1
#         self.phase1_type="isotropic"
#         self.materialParameters_phase1 = [200, 1.0]   

#     #--- Indicator function for material phases
#     # def indicatorFunction(self,x):
#     #     return 1 

#     #--- Define prestrain function for each phase (also works with non-constant values)
#     def prestrain_phase1(self,x):
#         return [[0, 0, 0], [0,0,0], [0,0,0]]



#############################################
#  Analytical reference Solution
#############################################
def deformation(x):
    angle = -math.pi/4.0;
    return np.array([x[0]*math.cos(angle), x[1], -x[0]*math.sin(angle)])

#rotation around y-axis with angle = pi/4
def deformationGradient(x):
    angle = -math.pi/4.0;
    return np.array([[math.cos(angle),0],
                     [0,1],
                     [-math.sin(angle),0]])


def displacement(x):
    return deformation(x) - [x[0], x[1], 0]

def displacementGradient(x):
    return deformationGradient(x) - np.array([[1.0, 0.0], [0.0,1.0], [0.0,0.0]])


def surfaceNormal(x):
    dmat = deformationGradient(x);

    normal = np.array([(dmat[1][0]*dmat[2][1])-(dmat[1][1]*dmat[2][0]), (dmat[2][0]*dmat[0][1]) - (dmat[0][0]*dmat[2][1]), (dmat[0][0]*dmat[1][1]) - (dmat[0][1]*dmat[1][0]) ])
    
    return normal


# udu = (u,du)
analyticalSol = (displacement, displacementGradient)