import math
import numpy as np

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()

#############################################
#  Paths
#############################################
parameterSet.resultPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs_properties-shape-test-2'
parameterSet.baseName= 'properties-shape-test-2'

#############################################
#  Grid parameters
#############################################
nX=8
nY=8

parameterSet.structuredGrid = 'simplex'
parameterSet.lower = '0 0'
parameterSet.upper = '3.14159265359 3.14159265359'     #  pi
parameterSet.elements = str(nX)+' '+  str(nY)

parameterSet.macroGridLevel = 3
############################################
#  GFE parameters
#############################################
parameterSet.order = 1
parameterSet.conforming_DiscreteJacobian = 1
parameterSet.measure_analyticalError = True
parameterSet.measure_isometryError = True

# Compare energy values.
parameterSet.compare_EnergyValues = True
parameterSet.analytical_energy = 2.467

#############################################
#  Solver parameters
#############################################
# Tolerance of the multigrid solver
parameterSet.tolerance = 1e-12
# Maximum number of multigrid iterations
parameterSet.maxProximalNewtonSteps = 200
# Initial regularization
parameterSet.initialRegularization = 200
# Measure convergence
parameterSet.instrumented = 0

############################
#   Problem specifications
############################

# Dimension of the domain (only used for numerical-test python files)
parameterSet.dim = 2

# # Dimension of the target space
# parameterSet.targetDim = 3
# parameterSet.targetSpace = 'BendingIsometry'

# #############################################
# #  Effective prestrain
# ############################################
# parameterSet.prestrainFlag = True
# parameterSet.macroscopically_varying_microstructure = False
# parameterSet.read_effectiveQuantities_from_Parset = True # Otherwise the Micro/Cell-Problem is solved once to obtain the quantities.

# # parameterSet.effectivePrestrain = '1.0 1.0 0.0'   # b1 b2 b12  #deprecated

# effectivePrestrain= np.array([[1.0, 0.0],
#                               [0.0, 1.0]]);



# #############################################
# #  Effective elastic moduli
# ############################################
# # input-vector: [q_1,q_2,q_3,q_12,q_13,q_23]
# # is assembled into a matrix where the off-diagonal entries are divided by 2 (compare with definition in the paper)
# #  (   q_1    , 0.5*q_12  ,  0.5*q_13 )
# #  ( 0.5*q_12 ,   q_2     ,  0.5*q_23 )
# #  ( 0.5*q_13 , 0.5*q_23  ,    q_3    )

# # parameterSet.effectiveQuadraticForm = '3.0 0.33 1.0 1.0 0.0 0.0'   #deprecated


# effectiveQuadraticForm = np.array([[3.0, 0.5, 0.0],
#                                    [0.5, 0.33, 0.0],
#                                    [0.0, 0.0, 1.0]]);



#############################################
#  MICROSTRUCTURE
############################################
parameterSet.printMicroOutput = False
parameterSet.VTKwriteMacroPhaseIndicator = False
parameterSet.MacroPhaseSubsamplingRefinement = 3

# Microstructure used:  Isotropic matrix material (phase 2) with prestrained fibers (phase 1) in the top layer aligned with the e2-direction.
class GlobalMicrostructure:
    """ Class that represents the global microstructure.
    
        The global microstructure can be defined individually 
        for different (finitely many) macroscopic phases. 
        Each macroscopic phase corresponds to a 'LocalMicrostructure_'
        sub-class that defines the necessary data for the local 
        micro-problem. 

        Currently, there are three possibilities for the LocalMicrostructure:
            (1) Read the effective quantities (Qhom,Beff) from this parset.
            (Constant local microstructure)
            (2) Solve the micro-problem once to obtain the effective quantities. 
            (Constant local microstructure)
            (3) Solve for micro-problem for each macroscopic quadrature point.
            (Macroscopocally varying local microstructure)) 
    """
    def __init__(self,macroPoint=[0,0]):
        self.macroPhases = 1

        # define first local microstructure options.
        self.macroPhase1_constantMicrostructure = True
        self.macroPhase1_readEffectiveQuantities = True;


    def macroPhaseIndicator(self,y): #y :macroscopic point
            """ Indicatorfunction that determines the domains 
                i.e. macro-phases of different local microstructures.
            """
            return 1;
                
    # Represents the local microstructure in Phase 1
    class LocalMicrostructure_1:  
        def __init__(self,macroPoint=[0,0]):
            # gamma = 1.0
            # self.macroPoint = macroPoint
            self.gamma = 1.0    #in the future this might change depending on macroPoint.
            self.phases = 2     #in the future this might change depending on macroPoint.

            self.effectivePrestrain= np.array([[1.0, 0.0],
                                               [0.0, 1.0]]);
            self.effectiveQuadraticForm = np.array([[3.0, 0.5, 0.0],
                                            [0.5, 0.33, 0.0],
                                            [0.0, 0.0, 1.0]]);


#############################################
#  Options
#############################################
# Write discrete solution as .vtk-File
parameterSet.writeVTK = 0
parameterSet.vtkwrite_analyticalSolution = 0
parameterSet.vtkWrite_analyticalSurfaceNormal = 0

# Write Dof-Vector to .txt-file
parameterSet.writeDOFvector = 0

# The grid can be refined several times for a higher resolution in the VTK-file.
parameterSet.subsamplingRefinement = 2
#############################################
#  Dirichlet boundary indicator
#############################################
# Parameterset II: "x-axial Minimizer"
def dirichlet_indicator(x) :
    if((x[0] <= 0.01)  ):
        return True
    else:
        return False


#############################################
#  Initial iterate function
#############################################
# Identity 
def f(x):
    return [x[0], x[1], 0]


def df(x):
    return ((1,0),
            (0,1),
            (0,0))

#start with analytical solution:
# def f(x):
#     return u(x)

# def df(x):
#     return du(x)

fdf = (f, df)

#############################################
#  Force
############################################
parameterSet.assemble_force_term = False

def force(x):
    return [0, 0, 0]
#############################################
#  Analytical reference Solution
#############################################

#Rotation around z-axis:
def R(alpha):
    return np.array([[math.cos(alpha), -math.sin(alpha), 0],
                    [math.sin(alpha), math.cos(alpha), 0],
                    [0, 0, 1]
                    ])
#Rotation around z-axis:
def R_z(alpha):
    return np.array([[math.cos(alpha), -math.sin(alpha), 0],
                    [math.sin(alpha), math.cos(alpha), 0],
                    [0, 0, 1]
                    ])

#Rotation around x-axis:
def R_x(alpha):
    return np.array([[1,0,0],
                    [0 , math.cos(alpha), -math.sin(alpha)],
                    [0 , math.sin(alpha), math.cos(alpha)]
                    ])



#Reflection of z-component:
S_z = np.array([[1,0,0],
                [0,1,0],
                [0,0,-1]
                ])

#Rotate solution 
alpha= 0.0

kappa = 7.0/6.0


def deformation(x):
    mat = np.array([-(1.0/kappa)*math.sin(-kappa*x[0]), x[1], ((1.0/kappa)*math.cos(-kappa*x[0])-(1.0/kappa))] )

    # out = np.dot(R_z(alpha), mat)

    return mat


def deformationGradient(x):
    dmat = np.array([[math.cos(-kappa*x[0]), 0],
            [0, 1.0],
            [1.0*(math.sin(-kappa*x[0])) , 0 ]])
    
    # out = np.dot(R_z(alpha),dmat)

    # return out
    return dmat

def displacement(x):
    return deformation(x) - np.array([x[0], x[1], 0])

def displacementGradient(x):
    return deformationGradient(x) - np.array([[1.0, 0.0], [0.0,1.0], [0.0,0.0]])



def surfaceNormal(x):
    
    # dmat = du(x);
    dmat = deformationGradient(x);

    normal = np.array([(dmat[1][0]*dmat[2][1])-(dmat[1][1]*dmat[2][0]), (dmat[2][0]*dmat[0][1]) - (dmat[0][0]*dmat[2][1]), (dmat[0][0]*dmat[1][1]) - (dmat[0][1]*dmat[1][0]) ]) 
    
    return normal


# udu = (u, du)
analyticalSol = (displacement, displacementGradient)