import math
import numpy as np

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()


#############################################
#  Paths
#############################################
parameterSet.resultPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs_properties-shape-test'

# parameterSet.instrumentedPath = '/home/klaus/Desktop/Dune_release/dune-microstructure/outputs_bartels-analytical-example/instrumented'
# parameterSet.executablePath = '/home/klaus/Desktop/Dune_release/dune-microstructure/build-cmake/src'
parameterSet.baseName= 'properties-shape-test'


#############################################
#  Grid parameters
#############################################
nX=7
nY=7

parameterSet.structuredGrid = 'simplex'
# parameterSet.lower = '-5 -2'
# parameterSet.upper = '5 2'
parameterSet.lower = '0 0'
# parameterSet.upper = '1 1'
# parameterSet.upper = '6.28318530718 6.28318530718'     # 2* pi
parameterSet.upper = '3.14159265359 3.14159265359'     #  pi
# parameterSet.lower = '-0.5 -0.5'
# parameterSet.upper = '0.5 0.5'
parameterSet.elements = str(nX)+' '+  str(nY)

parameterSet.numLevels = 2

#############################################
#  GFE parameters
#############################################

parameterSet.order = 1
# parameterSet.interpolationMethod = 'projected'
# parameterSet.interpolationMethod = 'embedded'
# parameterSet.interpolationMethod = 'geodesic'


parameterSet.conforming_DiscreteJacobian = 1

#############################################
#  Solver parameters
#############################################

# Tolerance of the multigrid solver
parameterSet.tolerance = 1e-12
# Maximum number of multigrid iterations
parameterSet.maxProximalNewtonSteps = 200
# Initial regularization
parameterSet.initialRegularization = 200
# Measure convergence
parameterSet.instrumented = 0

############################
#   Problem specifications
############################

# Dimension of the domain (only used for numerical-test python files)
parameterSet.dim = 2


# Dimension of the target space
parameterSet.targetDim = 3
parameterSet.targetSpace = 'BendingIsometry'

#############################################
#  Effective prestrain
############################################
parameterSet.prestrainFlag = True


#TEST 1 & 2
# parameterSet.effectivePrestrain = '1.0 2.0 0.0'   # b1 b2 b12


#Test1 - Neu
parameterSet.effectivePrestrain = '1.0 1.0 0.0'   # b1 b2 b12

#TEST NPG1 - 1
# parameterSet.effectivePrestrain = '0.0 0.5 0.0'   # b1 b2 b12

#TEST NPG1 - 2
# parameterSet.effectivePrestrain = '2.0 1.5 0.0'   # b1 b2 b12

#TEST NPG1 - 3 (one parameter family case)
# parameterSet.effectivePrestrain = '0.491 0.347 0.0'   # b1 b2 b12


#############################################
#  Effective elastic moduli
############################################
# input-vector: [q_1,q_2,q_3,q_12,q_13,q_23]
# is assembled into a matrix where the off-diagonal entries are divided by 2 (compare with definition in the paper)
#  (   q_1    , 0.5*q_12  ,  0.5*q_13 )
#  ( 0.5*q_12 ,   q_2     ,  0.5*q_23 )
#  ( 0.5*q_13 , 0.5*q_23  ,    q_3    )

# parameterSet.effectiveQuadraticForm = '1.0 1.0 1.0 0.0 0.0 0.0'   #q11 q22 q33 q12 q13 q23 (entries of symmetrix 3x3 matrix)

#Test1
# parameterSet.effectiveQuadraticForm = '1.0 4.0 2.0 1.0 0.0 0.0'


#Test1 - Neu
parameterSet.effectiveQuadraticForm = '1.0 4.0 2.0 1.0 0.0 0.0'
# parameterSet.effectiveQuadraticForm = '1.0 4.0 2.0 1.0 0.0 0.0'

#Test2
# parameterSet.effectiveQuadraticForm = '1.0 0.33 0.5 1.0 0.0 0.0'

# -----------------------------
#TEST NPG1  - 1
# parameterSet.effectiveQuadraticForm = '1.0 2.0 1.0 0.0 0.0 0.0'

#TEST NPG1  - 2
# parameterSet.effectiveQuadraticForm = '1.0 2.0 1.0 0.0 0.0 0.0'

#TEST NPG1  - 3 (one parameter family case)
# parameterSet.effectiveQuadraticForm = '1.0 2.0 1.164 0.5 0.0 0.0'

#############################################
#  Options
#############################################
# Write discrete solution as .vtk-File
parameterSet.writeVTK = 1

# Write Dof-Vector to .txt-file
parameterSet.writeDOFvector = 0
#############################################
#  Dirichlet boundary indicator
#############################################
# def dirichlet_indicator(x) :
#     if( (x[0] <= 0.001) or (x[1]<=0.001)):
#         return True
#     else:
#         return False

# def dirichlet_indicator(x) :
#     if( (x[0] <= 0.0001) ):
#         return True
#     else:
#         return False
    

# # No Boundary condition
# def dirichlet_indicator(x) :
#         return False
    
epsilon=0.01
# constraint stripe 
# def dirichlet_indicator(x) :
#     # epsilon=0.1
#     if((x[0] <= epsilon)  ):
#         return True
#     else:
#         return False

def dirichlet_indicator(x) :
    # epsilon=0.1
    if((x[1] <= epsilon)  ):
        return True
    else:
        return False


# constraint only corner nodes
# def dirichlet_indicator(x) :
#     epsilon=0.2
#     if((x[0] <= epsilon) and (x[1] <=epsilon) ):
#         return True
#     elif ((x[0] <= epsilon) and (x[1] >=(1.0-epsilon))):
#         return True
#     else:
#         return False



# def dirichlet_indicator(x):
#     epsilon=0.2
#     if((x[0] >= (1.0-epsilon)) and (x[1] <=epsilon) ):
#         return True
#     elif ((x[0] >= (1.0-epsilon)) and (x[1] >=(1.0-epsilon))):
#         return True
#     else:
#         return False
    



#############################################
#  Initial iterate function
#############################################
# def f(x):
#     return [math.sin(x[0]), x[1], 1.0-math.cos(x[0])]


# def df(x):
#     return [[math.cos(x[0]),0],
#             [0,1],
#             [math.sin(x[0]),0]]


# def f(x):
#     return [x[0], x[1], 0]


# def df(x):
#     return ((1,0),
#             (0,1),
#             (0,0))


#start with analytical solution:
def f(x):
    return u(x)

def df(x):
    return du(x)


#---------------- Constraint Stripe 
# def f(x):
#     # epsilon=0.1
#     if((x[0] <= epsilon)):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]
#     else:
#         return [x[0], x[1], 0]



# def df(x):
#     # epsilon=0.1
#     if((x[0] <= epsilon)):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [math.sin(-2.0*x[0]),0]]
#     else:
#         return ((1,0),
#                 (0,1),
#                 (0,0))
    
# reverse z-direction
# def f(x):
#     # epsilon=0.1
#     if((x[0] <= epsilon)):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], -1.0*(0.5*math.cos(-2.0*x[0])-0.5)]
#     else:
#         return [x[0], x[1], 0]



# def df(x):
#     # epsilon=0.1
#     if((x[0] <= epsilon)):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [-1.0*math.sin(-2.0*x[0]),0]]
#     else:
#         return ((1,0),
#                 (0,1),
#                 (0,0))

# def f(x):
#     # epsilon=0.1
#     if((x[0] <= epsilon)):
#         return u(x)
#     else:
#         return [x[0], x[1], 0]

# def df(x):
#     # epsilon=0.1
#     if((x[0] <= epsilon)):
#         return du(x)
#     else:
#         return ((1,0),
#                 (0,1),
#                 (0,0))
    
# def f(x):
#     # epsilon=0.1
#     if((x[1] <= epsilon)):
#         return u(x)
#     else:
#         return [x[0], x[1], 0]

# def df(x):
#     # epsilon=0.1
#     if((x[1] <= epsilon)):
#         return du(x)
#     else:
#         return ((1,0),
#                 (0,1),
#                 (0,0))
#---------------------------------------------------------


# def f(x):
#     if( (x[0] <= 0.0001) ):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]
#     else:
#         return [x[0], x[1], 0]



# def df(x):
#     if( (x[0] <= 0.0001) ):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [math.sin(-2.0*x[0]),0]]
#     else:
#         return ((1,0),
#                 (0,1),
                # (0,0))



# def f(x):
#     if( (x[0] <= 0.1) and (x[1] <=0.1) ):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]
#     elif ((x[0] <= 0.1) and (x[1] >=0.9) ):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]
#     else:
#         return [x[0], x[1], 0]



# def df(x):
#     if( (x[0] <= 0.1) and (x[1] <=0.1) ):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [math.sin(-2.0*x[0]),0]]
#     elif ((x[0] <= 0.1) and (x[1] >=0.9) ):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [math.sin(-2.0*x[0]),0]]
#     else:
#         return ((1,0),
#                 (0,1),
#                 (0,0))

# def f(x):
#     epsilon=0.2
#     if((x[0] >= (1.0-epsilon)) and (x[1] <=epsilon) ):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]
#     elif ((x[0] >= (1.0-epsilon)) and (x[1] >=(1.0-epsilon))):
#         return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]
#     else:
#         return [x[0], x[1], 0]


# def df(x):
#     epsilon=0.2
#     if((x[0] >= (1.0-epsilon)) and (x[1] <=epsilon) ):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [math.sin(-2.0*x[0]),0]]
#     elif ((x[0] >= (1.0-epsilon)) and (x[1] >=(1.0-epsilon))):
#         return [[math.cos(-2.0*x[0]),0],
#                 [0,1.0],
#                 [math.sin(-2.0*x[0]),0]]
#     else:
#         return ((1,0),
#                 (0,1),
#                 (0,0))




    

# def f(x):
#     return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]


# def df(x):
#     return [[math.cos(-2.0*x[0]),0],
#             [0,1],
#             [math.sin(-2.0*x[0]),0]]


fdf = (f, df)


#############################################
#  Force
############################################
def force(x):
    return [0, 0, 0]

#############################################
#  Analytical reference Solution
#############################################

# ParameterSet1 ---------------------
# def u(x):
#     return [-0.5*math.sin(-2.0*x[0]), x[1], 0.5*math.cos(-2.0*x[0])-0.5]


# def du(x):
#     return [[math.cos(-2.0*x[0]),0],
#             [0,1.0],
#             [math.sin(-2.0*x[0]),0]]


# reverse z-direction:
# def u(x):
#     return [-0.5*math.sin(-2.0*x[0]), x[1], -1.0*(0.5*math.cos(-2.0*x[0])-0.5)]


# def du(x):
#     return [[math.cos(-2.0*x[0]),0],
#             [0,1.0],
#             [-math.sin(-2.0*x[0]),0]]
# --------------------------------------------------------
# ParameterSet 2 - NEU  --------------------
# def u(x):
#     return [-(4.0/3.0)*math.sin(-(3.0/4.0)*x[1]), -x[0], (4.0/3.0)*math.cos(-(3.0/4.0)*x[1])-(4.0/3.0)]


# def du(x):
#     return [[0, math.cos(-(3.0/4.0)*x[1])],
#             [1.0, 0],
#             [0, math.sin(-(3.0/4.0)*x[1])]]


# reverse z-direction:
# def u(x):
    # return [-(4.0/3.0)*math.sin(-(3.0/4.0)*x[1]), -x[0], -1.0*((4.0/3.0)*math.cos(-(3.0/4.0)*x[1])-(4.0/3.0))]



# def u(x):
#     return [-(4.0/3.0)*math.sin(-(3.0/4.0)*x[1]), -x[0], -1.0*((4.0/3.0)*math.cos(-(3.0/4.0)*x[1])-(4.0/3.0))] 


# def du(x):
#     return [[0, math.cos(-(3.0/4.0)*x[1])],
#             [-1.0, 0],
#             [0, -1.0*(math.sin(-(3.0/4.0)*x[1])) ]]


#Rotation around z-axis:
def R(alpha):
    return np.array([[math.cos(alpha), -math.sin(alpha), 0],
             [math.sin(alpha), math.cos(alpha), 0],
             [0, 0, 1]
             ])

#replace y and y coordinates
# def u(x):
#     return [-(4.0/3.0)*math.sin(-(3.0/4.0)*x[0]), -x[1], -1.0*((4.0/3.0)*math.cos(-(3.0/4.0)*x[0])-(4.0/3.0))] 


# def du(x):
#     return [[math.cos(-(3.0/4.0)*x[0]),0],
#             [0, -1.0],
#             [-1.0*(math.sin(-(3.0/4.0)*x[0])), 0 ]]


#Rotate solution 
alpha= math.pi/2.0
# alpha= 0.0

# kappa = 5/4 # dahin scheint diskrete Minimierer zu laufen!?
# kappa = 3/4 # dahin scheint diskrete Minimierer zu laufen!?
kappa = 9.0/8.0

def u(x):
    mat = np.array([-(1.0/kappa)*math.sin(-kappa*x[1]), -x[0], -1.0*((1.0/kappa)*math.cos(-kappa*x[1])-(1.0/kappa))] )

    out = np.dot(R(alpha), mat)

    return out


def du(x):
    dmat = np.array([[0, math.cos(-kappa*x[1])],
            [-1.0, 0],
            [0, -1.0*(math.sin(-kappa*x[1])) ]])
    
    out = np.dot(R(alpha),dmat)

    return out



# No Rotation:
# def u(x):
#     mat = np.array([-(1.0/kappa)*math.sin(-kappa*x[1]), -x[0], -1.0*((1.0/kappa)*math.cos(-kappa*x[1])-(1.0/kappa))] )

#     return mat


# def du(x):
#     dmat = np.array([[0, math.cos(-kappa*x[1])],
#             [-1.0, 0],
#             [0, -1.0*(math.sin(-kappa*x[1])) ]])
    
#     return dmat
# --------------------------------------------------------


# def u(x):
#     return [-0.33*math.sin(-3.0*x[0]), x[1], 0.33*math.cos(-3.0*x[0])-0.33]


# def du(x):
#     return [[math.cos(-3.0*x[0]),0],
#             [0,1.0],
#             [math.sin(-3.0*x[0]),0]]


#ParameterSet2
# def u(x):
#     return [-0.25*math.sin(-4.0*x[1]), -x[0], 0.25*math.cos(-4.0*x[1])-0.25]


# def du(x):
#     return [[0, math.cos(-4.0*x[1])],
#             [1.0, 0],
#             [0, math.sin(-4.0*x[1])]]

# def u(x):
#     return [-0.5*math.sin(-0.5*x[1]), -x[0], math.cos(-0.5*x[1])-0.5]


# def du(x):
#     return [[0, math.cos(-0.5*x[1])],
#             [1.0, 0],
#             [0, math.sin(-0.5*x[1])]]


# def u(x):
#     return [-1.0*(0.5*math.cos(2.0*x[0])-0.5), -x[1],-0.5*math.sin(2.0*x[0]) ]


# def du(x):
#     return [[-math.cos(2.0*x[0]),0],
#             [0,-1.0],
#             [-math.sin(2.0*x[0]),0]]

udu = (u, du)