import math
import numpy as np

class ParameterSet(dict):
    def __init__(self, *args, **kwargs):
        super(ParameterSet, self).__init__(*args, **kwargs)
        self.__dict__ = self

parameterSet = ParameterSet()

#############################################
#  Paths
#############################################
# parameterSet.outputPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs'
parameterSet.baseName= 'readmicrostructuretest'   #(needed for Output-Filename)

#############################################
#  Material Setup
#############################################
class Microstructure:
    def __init__(self):
        # self.macroPoint = macroPoint
        self.gamma = 1.0    #in the future this might change depending on macroPoint.
        self.phases = 4     #in the future this might change depending on macroPoint.
        #--- Define different material phases:
        #- PHASE 1
        self.phase1_type="isotropic"
        self.materialParameters_phase1 = [2.0, 0]   
        #- PHASE 2
        self.phase2_type="orthotropic"
        self.materialParameters_phase2 = [11.2e3,630,1190,700,230,960,0.63 ,0.49,0.37]  
        self.phase2_axis = 2
        self.phase2_angle = np.pi/2.0
        #- PHASE 3
        self.phase3_type="transversely_isotropic"
        self.materialParameters_phase3 = [1.0, 2.0, 3.0, 4.0, 5.0]
        #- PHASE 4
        self.phase4_type="general_anisotropic"
        self.materialParameters_phase4 = np.array([ [1.0,   4.0,   3.0,   0.0,   0.0,  0.0],
                                                    [2.0,   2.0,   4.0,   0.0,   0.0,  0.0],
                                                    [3.0,  -5.0,   5.0,   0.0,   0.0,  0.0],
                                                    [0.0,   0.0,   0.0,   0.5,   0.0,  0.0],
                                                    [0.0,   0.0,   0.0,   0.0,   0.2,  0.0],
                                                    [0.0,   0.0,   0.0,   0.0,   0.0,  0.1]]);



    #--- Indicator function for material phases
    def indicatorFunction(self,x):
        theta=0.25
        if (abs(x[0]) < (theta/2) and x[2] < 0 ):
            return 1    #Phase1
        elif (abs(x[0]) > (theta/2) and x[2] > 0 ):
            return 2    #Phase2
        elif (abs(x[0]) < (theta/2) and x[2] > 0 ):
            return 3    #Phase3
        else :
            return 4    #Phase4
        
        

    #--- Define prestrain function for each phase (also works with non-constant values)
    def prestrain_phase1(self,x):
        return [[2, 0, 0], [0,2,0], [0,0,2]]

    def prestrain_phase2(self,x):
        return [[1, 0, 0], [0,1,0], [0,0,1]]

    def prestrain_phase3(self,x):
        return [[0, 0, 0], [0,0,0], [0,0,0]]

    def prestrain_phase4(self,x):
        return [[1.0, 0, 0], [0,5.0,0], [0,0,0]]



#############################################
#  Assembly options
#############################################
parameterSet.set_IntegralZero = 1            #(default = false)
parameterSet.set_oneBasisFunction_Zero = 1   #(default = false)
#parameterSet.arbitraryLocalIndex = 7            #(default = 0)
#parameterSet.arbitraryElementNumber = 3         #(default = 0)

#############################################
#  Solver Options, Type: #1: CG - SOLVER , #2: GMRES - SOLVER, #3: QR - SOLVER (default), #4: UMFPACK - SOLVER
#############################################
parameterSet.Solvertype = 3        # recommended to use iterative solver (e.g GMRES) for finer grid-levels
parameterSet.Solver_verbosity = 0  #(default = 2)  degree of information for solver output

#############################################
#  Write/Output options      #(default=false)
#############################################
# --- (Optional output) write Material / prestrain / Corrector functions to .vtk-Files:
parameterSet.write_materialFunctions = 0   # VTK indicator function for material/prestrain definition

# --- (Additional debug output)
parameterSet.print_debug = 0  #(default=false)

# --- Write Correctos to VTK-File:  
parameterSet.write_VTK = 0

# --- Use caching of element matrices:  
parameterSet.cacheElementMatrices = 1

# The grid can be refined several times for a higher resolution in the VTK-file.
parameterSet.subsamplingRefinement = 0

