import os
import subprocess
import sys
import re
import numpy as np
from prettytable import PrettyTable
# import latextable
# from texttable import Texttable
from tabulate import tabulate
# import time
import importlib
# import fileinput


####################################################################
# Usage:
#
#   python3 macro-problem-testsuite.py  CONTAINER_PATH_Flag  <slurm_array_task_id>
#
#   e.g. python3 /home/klaus/Desktop/Dune_master/dune-gfe/bending-isometries-testsuite/bending-isometries-testsuite.py 0 2
#
####################################################################


def SetParameterMaterialFunction(inputFunction, parameterName, parameterValue):
    with open(inputFunction+'.py', 'r') as file:
        filedata = file.read()
        filedata = re.sub('(?m)'+str(parameterName)+'\s?=.*',str(parameterName)+' = '+str(parameterValue),filedata)
        f = open(inputFunction+'.py','w')
        f.write(filedata)
        f.close()
  

# parameterArray=np.array([ ["rho", np.array([0, 1,2])], ["parameterSet.maxProximalNewtonSteps", [0, 10, 100]] ], dtype=object)

# Python list version:
# parameterArray=[ ["rho", [0, 1,2]], ["parameterSet.maxProximalNewtonSteps", [0, 10, 100]] ]

# parameterArray=[ ["rho", [0,1,2,5]]]

# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------------------------------------------------------------------------------
compute_BendingIsometry = True
create_outputTable = False


# 1. Experiment   2. ExperimentPathExtension   3. MacroGridLevels, 4. Executable 5.(additional) parameterArray: [["ParameterName", ParameterValue], ...]
scenarios = [
                    #  ["buckling_experiment" ,              "/buckling_experiment",  [3,3], "/macro-problem" , [["rho", [0,1,2,3,4,5]]] ],   
                    ["cylindrical_2variableBC" ,          "/macro-problem/variableBC",           [3,3], "/macro-problem" , [["delta", [0.75]]] ],   
                    ["cylindrical_2variableBC" ,          "/macro-problem/variableBC",           [3,3], "/macro-problem" , [["delta", [0.0]]] ],   
                    ["cylindrical_2variableBC_wood" ,     "/macro-problem/variableBC",           [3,3], "/macro-problem" , [["delta", [0.1,0.25,0.5,1.0,1.5,2.0]]] ],   
                    ["buckling_experiment" ,              "/macro-problem/buckling_experiment",  [3,3], "/macro-problem" , [["rho", [1.0]]] ],   
                    ["buckling_experiment_Top" ,          "/macro-problem/buckling_experiment",  [3,3], "/macro-problem" , [["rho", [3,4,5]]] ],   
                    ["buckling_experiment_xAlignedTop" ,  "/macro-problem/buckling_experiment",  [3,3], "/macro-problem" , [["rho", [5]]] ],   
                    ["buckling_experiment_xAlignedLow" ,  "/macro-problem/buckling_experiment",  [3,3], "/macro-problem" , [["rho", [5]]] ],   
                    ["self-folding-box",                  "/macro-problem/self-folding-box",     [5,5], "/macro-problem" , [["rho", [500]]]            ],
                    ["twisted-valley",                    "/macro-problem/Rumpf-Experiments",  [2,2], "/macro-problem" , [["test", [1.0]]]            ],
                    ["diagonal-trusses",                  "/macro-problem/Rumpf-Experiments",  [2,2], "/macro-problem" ,  [["test", [1.0]]]            ],
                    ["cylindrical_1",                                         "",  [0,0], "/micro-problem"  , [["mu_phase1", [90,100,110]]] ],
                    ["parametrized_laminate",                                 "",  [0,0], "/micro-problem"  ,         []            ],
             ]


print('sys.argv[0]', sys.argv[0])
print('sys.argv[1]', sys.argv[1])
print('sys.argv[2]', sys.argv[2])


CONTAINER = int(sys.argv[1])
slurm_array_task_id = int(sys.argv[2])

parameterFile  = scenarios[slurm_array_task_id][0]
pathExtension  = scenarios[slurm_array_task_id][1]
gridLevels     = scenarios[slurm_array_task_id][2]
executable     = scenarios[slurm_array_task_id][3]
parameterArray = scenarios[slurm_array_task_id][4]

#Test 
# parameterArray = [[" -rho ", [4.0]]] 


print('pathExtension: ', pathExtension)





# conforming_DiscreteJacobian = scenarios[slurm_array_task_id][1]
# INSTRUMENTED = scenarios[slurm_array_task_id][3]


#Todo conforming option.
conformity = ""
INSTRUMENTED = 0
conforming_DiscreteJacobian = 0

# if conforming_DiscreteJacobian:
#     conformity = "_conforming"
# else:
#     conformity = "_nonconforming"


#Get current working directory:
cwd = os.getcwd()

modulePath = os.path.abspath(os.path.join(cwd, os.pardir))
print("Parent Directory is: ", modulePath)


# for experiment in scenarios: ..


#Path for parameterFile
if CONTAINER:

    # Barnard_Workspace = "/data/horse/ws/s7603593-microstructure/"

    #--- Barnard/CONTAINER version
    pythonPath = "/dune/dune-microstructure/experiment" + pathExtension
    # resultPath = "outputs" + "_" + scenarios[slurm_array_task_id][0]
    # resultPath = Barnard_Workspace + "outputs" + "_" + scenarios[slurm_array_task_id][0]


    resultPath = "outputs" + "_" + scenarios[slurm_array_task_id][0]


    instrumentedPath =  resultPath + "/instrumented" 
    executablePath = "/dune/dune-microstructure/build-cmake/src"
    # try:
    #   os.mkdir(resultPath)
    #   os.mkdir(instrumentedPath)
    # except OSError as error:
    #   print(error)
    
    # resultPath = "/" + resultPath
    # instrumentedPath = "/"
else :
    #--- Local version
    # pythonPath = "/home/klaus/Desktop/Dune_bendIso/dune-microstructure/experiment" + pathExtension
    # resultPath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/outputs' + "_" + scenarios[slurm_array_task_id][0]
    # instrumentedPath = resultPath + "/instrumented"
    # executablePath = '/home/klaus/Desktop/Dune_bendIso/dune-microstructure/build-cmake/src'
    pythonPath =  modulePath + "/experiment" + pathExtension
    resultPath = modulePath + "/outputs" + "_" + scenarios[slurm_array_task_id][0]
    instrumentedPath = resultPath + "/instrumented"
    executablePath = modulePath + '/build-cmake/src'

    # try:
    #     os.mkdir(resultPath)
    #     os.mkdir(instrumentedPath)
    # except OSError as error:
    #     print(error)

try:
    os.mkdir(resultPath)
    os.mkdir(instrumentedPath)
except OSError as error:
    print(error)
    

print('pythonPath: ', pythonPath)


### Access ParameterFile variables.
sys.path.insert(0, pythonPath)
__import__(parameterFile, fromlist=['parameterSet'])
imported_module = importlib.import_module(parameterFile)
parameterSet = getattr(imported_module, "parameterSet")



#--- Set update grid path if unstructured/GMSH-grid is used.
if( parameterSet.structuredGrid == 'false'):
    print('Update gridPath')
    SetParameterMaterialFunction(pythonPath + "/" + scenarios[slurm_array_task_id][0], 'gridPath' , "'" + str(pythonPath) + "'")


# dim = parameterSet.dim
# maxSolverSteps = parameterSet.maxProximalNewtonSteps


# executable = executablePath + "/macro-problem"
executable = executablePath + executable



path = os.getcwd()
print("Path: ", path)
############################################

print('parameterArray:', parameterArray)
print('len(parameterArray):',           len(parameterArray))
print('len(parameterArray[0][1]): ', len(parameterArray[0][1]))
print('Number of different additional parameters:', len(parameterArray))
# print('len(parameterArray)[1]:',        len(parameterArray[1]))
# print('len(parameterArray[0][1]):',     len(parameterArray[0][1]))
# print('parameterArray[0][1][1]:',       parameterArray[0][1][1])

# print('np.shape(parameterArray)[1]:', np.shape(parameterArray)[1])
# print('np.shape(parameterArray[0][1]):', np.shape(parameterArray[0][1]))
# print('np.shape(parameterArray[0][1])[1]:', np.shape(parameterArray[0][1])[1])



# if len(parameterArray) == 0:
#     nParams = 1
# else: 
#     nParams = len(parameterArray)

    # if len(parameterArray) == 0:
    #     paramValues = 1
    # else: 
    #     paramValues = len(parameterArray[i][1])

# ------ Loops through Parameters for Material Function -----------
# for i in range(0, np.shape(parameterArray)[1]):
#     for v in range(0,np.shape(parameterArray[i][1])[1]):
for i in range(0, len(parameterArray)):
    for v in range(0,len(parameterArray[i][1])):
        print("------------------")
        print("New Loop")
        print("------------------")
        print('i:', i)
        print('v:', v)
        print('len(parameterArray[i][1]):', len(parameterArray[i][1]))
        # print('parameterArray[i][v]:', parameterArray[i][v])

        print('parameterName:'  , parameterArray[i][0])
        print('parameterValue: ', parameterArray[i][1][v])
        # print('np.shape(parameterArray)[1]:', np.shape(parameterArray)[1])
        # print('np.shape(parameterArray[i][1]):', np.shape(parameterArray[i][1]))
        # print('np.shape(parameterArray[i][1])[1]:', np.shape(parameterArray[i][1])[1])
        parameterName =  parameterArray[i][0]
        parameterValue = parameterArray[i][1][v]


        

        ## Alternative version: actually changing the parset file..
        # Change Parameters:
        print('...change input parameter:' , parameterName)
        SetParameterMaterialFunction(pythonPath + "/" + scenarios[slurm_array_task_id][0], parameterName , parameterValue)



        baseName = scenarios[slurm_array_task_id][0] + "_" + parameterArray[i][0] + str(parameterArray[i][1][v])

        #--- Compute discrete bending isometries
        if compute_BendingIsometry:
            processList = []
            for macroGridLevel in range(gridLevels[0],gridLevels[1]+1):
                # if conforming_DiscreteJacobian:
                #     conformity = "_conforming"
                # else:
                #     conformity = "_nonconforming"

                # LOGFILE = resultPath + "/" + parameterFile + conformity + "_macroGridLevel" + str(macroGridLevel) + parameterName  + ".log"
                LOGFILE = resultPath + "/" + parameterFile  + "_macroGridLevel" + str(macroGridLevel) + "_" + parameterName + "_" + str(parameterValue) + ".log"

                print('LOGFILE:', LOGFILE)
                print('executable:', executable)
                print('parameterFile:', parameterFile)
                print('resultPath:', resultPath)

                # testArray = [ " -rho " + str(8.0), " -beta " + str(0.10) ]

                # start_time = time.time()
                p = subprocess.Popen(executable + " " + pythonPath + " " + parameterFile
                                                + " -macroGridLevel " + str(macroGridLevel)
                                                + " -resultPath " + str(resultPath)
                                                + " -baseName " + str(baseName)
                                                + " -instrumentedPath " + str(instrumentedPath)
                                                + " -conforming_DiscreteJacobian " + str(conforming_DiscreteJacobian)
                                                + " -instrumented " + str(INSTRUMENTED)
                                                # + " -" + parameterName + " " + str(parameterValue)
                                                + " | tee " + LOGFILE, shell=True)

                p.wait() # wait
                # print("--- %s seconds ---" % (time.time() - start_time))
                print('------FINISHED PROGRAM on macroGridLevel:' + str(macroGridLevel))
                processList.append(p)

                # Wait for all simulation subprocesses before proceeding to the error measurement step
                exit_codes = [p.wait() for p in processList]


# ------------  Create Output Table  -----------------------------------------------------------------------------
if create_outputTable:
    #Setup Output-Table:
    x = PrettyTable()
    constraintError = 0.0

    x.title = parameterFile
    x.field_names = ["r", "#Triang/#DOF", "#Steps", "Energy difference", "discrete energy", "L2-Error" , "H1-Error", "L2-Isometry-Error", "wall-time" ]
    x.align["k"] = "l" # Left align
    x.padding_width = 1 # One space between column edges and contents (default)

    rows = []
    rows.append(["r", "#Triang/#DOF", "#Steps",r"$\lvert \cI(u) - \cI^h(u_h) \rvert$", "$\cI^h(u_h)$", r"$\lVert u - u_h_{L^2}\rVert_0$" , "wall-time" ])

    for macroGridLevel in range(gridLevels[0],gridLevels[1]+1):
        
        LOGFILE = resultPath + "/" + parameterFile + conformity + "_macroGridLevel" + str(macroGridLevel) + ".log"
        # Read Energy Values:
        with open(LOGFILE, 'r') as file:
            output  = file.read()


        tmp_energyDiff = re.findall(r'(?m)Energy difference: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_energyAnalytical = re.findall(r'(?m)Analytical energy: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_energyDiscrete = re.findall(r'(?m)Energy of discrete solution: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_step = re.findall(r'(?m)Step Number: (\d+)',output)
        tmp_reg = re.findall(r'(?m)regularization parameter: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)' ,output)
        tmp_dofs = re.findall(r'(?m)Degrees of Freedom: (\d+)',output)
        tmp_time = re.findall(r'(?m)Total time elapsed: (-?\d+\.?\d+\d?)',output)
        tmp_isoErrorL2 = re.findall(r'(?m)L2-Isometry-error: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_isoErrorL1 = re.findall(r'(?m)L1-Isometry-error: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_L2Error = re.findall(r'(?m)L2-error: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_H1Error = re.findall(r'(?m)H1-error: (-?\d?\d?\d?\.?\d+[Ee]?[+\-]?\d?\d?)',output)
        tmp_elements = re.findall(r'(?m)elements: (\d+)',output)

        # print('tmp_step:',tmp_step)
        print('tmp_step last:', int(tmp_step[-1]))
        print('tmp_dofs:',tmp_dofs)
        print('tmp_dofs last:', int(tmp_dofs[-1]))
        # print('type tmp_energy:', type(tmp_energy))
        print('tmp_energyDiscrete:', tmp_energyDiscrete)
        print('tmp_energyDiff:', tmp_energyDiff)
        print('tmp_energyAnalytical:', tmp_energyAnalytical)
        # print('tmp_energy last:', tmp_energy[-1])
        print('tmp_time:', tmp_time)
        print('tmp_isoErrorL2 :', tmp_isoErrorL2 )
        print('tmp_isoErrorL1 :', tmp_isoErrorL1 )
        print('tmp_L2Error :', tmp_L2Error )
        print('tmp_H1Error :', tmp_H1Error )
        print('tmp_elements :', tmp_elements )


        x.add_row([macroGridLevel, tmp_elements[-1], int(tmp_step[-1]), tmp_energyDiff[-1], tmp_energyDiscrete[-1], tmp_L2Error[-1],tmp_H1Error[-1], tmp_isoErrorL2[-1], tmp_time[-1]])
        # rows.append.([macroGridLevel, tmp_elements[-1], int(tmp_step[-1]), tmp_energyDiff[-1], tmp_energyDiscrete[-1], tmp_L2Error[-1],tmp_H1Error[-1], tmp_isoErrorL2[-1], tmp_time[-1]])



    # Write Table to text-file.
    tablefile = open(resultPath + "/" + parameterFile + conformity   + "_table"   + ".txt", "w")
    tablefile.write(str(x))
    tablefile.write('\n')
    # tablefile.write(str(tabulate(rows, headers='firstrow', tablefmt='latex_raw')))
    # tablefile.write(str(tabulate(rows, headers='firstrow')))
    tablefile.close()


    # Print Table
    print(x)
    # print(tabulate(rows, headers='firstrow', tablefmt='latex_raw'))
    # print(tabulate(rows, headers='firstrow'))


